////////////////////////////////////////////////////////////////////////////
// Barebones effect to encode geometry attributes for the GPU interaction
// library.
//
// Copyright (C) 2007 Harlen Costa Batagelo. All rights reserved.
//
// Permission is hereby granted, free of charge, to any person obtaining
// a copy of this software and associated documentation files (the 
// "Software"), to deal in the Software without restriction, including
// without limitation the rights to use, copy, modify, merge, publish,
// distribute, sublicense, and/or sell copies of the Software, and to
// permit persons to whom the Software is furnished to do so, subject to
// the following conditions:
//
// The above copyright notice and this permission notice shall be
// included in all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
// IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
// CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
// TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
// SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////
// Global variables
////////////////////////////////////////////////////////////////////////////
float2   g_fVtIdxC; // 1D-2D constant (vertex attributes)
float4x4 g_mWVP;    // World*view*proj
float4x4 g_mITW;    // Inverse transpose of the world

// Vertex position
texture Tex0;
sampler s0 = sampler_state
{
  Texture   = (Tex0);
  MipFilter = NONE; MinFilter = POINT; MagFilter = POINT;
  AddressU  = Wrap; AddressV  = Wrap;
};

// Vertex normal
texture Tex1;
sampler s1 = sampler_state
{
  Texture   = (Tex1);
  MipFilter = NONE; MinFilter = POINT; MagFilter = POINT;
  AddressU  = Wrap; AddressV  = Wrap;
};

// Vertex texture coordinates
texture Tex2;
sampler s2 = sampler_state
{
  Texture   = (Tex2);
  MipFilter = NONE; MinFilter = POINT; MagFilter = POINT;
  AddressU  = Wrap; AddressV  = Wrap;
};

// Vertex tangent and handedness
texture Tex3;
sampler s3 = sampler_state
{
  Texture   = (Tex3);
  MipFilter = NONE; MinFilter = POINT; MagFilter = POINT;
  AddressU  = Wrap; AddressV  = Wrap;
};

// Vertex principal curvature and direction #1
texture Tex4;
sampler s4 = sampler_state
{
  Texture   = (Tex4);
  MipFilter = NONE; MinFilter = POINT; MagFilter = POINT;
  AddressU  = Wrap; AddressV  = Wrap;
};

// Vertex principal curvature and direction #2
texture Tex5;
sampler s5 = sampler_state
{
  Texture   = (Tex5);
  MipFilter = NONE; MinFilter = POINT; MagFilter = POINT;
  AddressU  = Wrap; AddressV  = Wrap;
};

// 2nd fundamental form
texture Tex6;
sampler s6 = sampler_state
{
  Texture   = (Tex6);
  MipFilter = NONE; MinFilter = POINT; MagFilter = POINT;
  AddressU  = Wrap; AddressV  = Wrap;
};

// Tensor of curvature derivative
texture Tex7;
sampler s7 = sampler_state
{
  Texture   = (Tex7);
  MipFilter = NONE; MinFilter = POINT; MagFilter = POINT;
  AddressU  = Wrap; AddressV  = Wrap;
};

// Output data from encoder pixel shader
struct PS_OUT
{
  float4 vColor[[@10]] : COLOR;
  float  fDepth : DEPTH;
};

////////////////////////////////////////////////////////////////////////////
// Convert 1D index (i) to 2D UV index.
// c = {1/W, 1/(W*H)} where W and H are the width and height of the
//     texture map
////////////////////////////////////////////////////////////////////////////
float2 I1DTo2D( float i, float2 c ) { return float2( i, i ) * c; } 

// Add VSDEFORM
[@1]

// Add PSDEFORM
[@2]

// Add fUserDefVS
[@14]

// Add fPSDeform
[@3]

////////////////////////////////////////////////////////////////////////////
// Encoder vertex shader.
////////////////////////////////////////////////////////////////////////////
PSDEFORM EncodeVS( VSDEFORM In )
{
  PSDEFORM Out;

  // UV coordinates of the vertex attribute texture
  // @4 = vertex semantic with ID
  float2 vVtUV       = I1DTo2D( In.[@4].x, g_fVtIdxC );
  float3 vSampledPos = tex2Dlod( s0, float4( vVtUV, 0, 0 ) );  

  // Transform deformed position
  Out.vPos  = mul( float4( vSampledPos, 1.0f ), g_mWVP );  
  Out.vTPos = Out.vPos;

[@5]
[@13]
[@12]
}

////////////////////////////////////////////////////////////////////////////
// Encoder pixel shader.
////////////////////////////////////////////////////////////////////////////
PS_OUT EncodePS( PSDEFORM In )
{
  PSDEFORM DefOut = In;
  PS_OUT Out;

[@6]
[@7]
  
  Out.fDepth = DefOut.vTPos.z / DefOut.vTPos.w;

  return Out;
}

////////////////////////////////////////////////////////////////////////////
// Effect technique for encoding attributes of indexed and 
// non-indexed geometry.
////////////////////////////////////////////////////////////////////////////
technique Encode
{
    pass P0
    {
        VertexShader = compile vs_3_0 EncodeVS( );
        PixelShader  = compile ps_3_0 EncodePS( );
    }
}
