////////////////////////////////////////////////////////////////////////////
// Barebones vertex shader to encode geometry attributes for the GPU 
// interaction library.
//
// Copyright (C) 2007 Harlen Costa Batagelo. All rights reserved.
//
// Permission is hereby granted, free of charge, to any person obtaining
// a copy of this software and associated documentation files (the 
// "Software"), to deal in the Software without restriction, including
// without limitation the rights to use, copy, modify, merge, publish,
// distribute, sublicense, and/or sell copies of the Software, and to
// permit persons to whom the Software is furnished to do so, subject to
// the following conditions:
//
// The above copyright notice and this permission notice shall be
// included in all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
// IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
// CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
// TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
// SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////
// Global variables
////////////////////////////////////////////////////////////////////////////

// Size of a dimension of the vertex attribute map, in texels.
uniform int g_iVtMapSize;

uniform sampler2D s0; // Vertex position
uniform sampler2D s1; // Vertex normal
uniform sampler2D s2; // Vertex texture coordinates
uniform sampler2D s3; // Vertex tangent and handedness
uniform sampler2D s4; // Vertex principal curvature and direction #1
uniform sampler2D s5; // Vertex principal curvature and direction #2
uniform sampler2D s6; // 2nd fundamental form
uniform sampler2D s7; // Tensor of curvature derivative

////////////////////////////////////////////////////////////////////////////
// Convert 1D index (i) to 2D UV index.
// i = 1D index
// s = size of a screen line, in pixels
////////////////////////////////////////////////////////////////////////////
vec2 I1DTo2D(float i, float s) { return vec2(mod(i,s),int(i/s)); }

// PSDEFORM
[@2]

// fUserDefVS
[@14]

////////////////////////////////////////////////////////////////////////////
// Encoder vertex shader.
////////////////////////////////////////////////////////////////////////////
void main( ) // PSDEFORM EncodeVS( VSDEFORM In )
{
    PSDEFORM Out;

    // UV coordinates of the vertex attribute texture
    // @4 = vertex semantic with ID
    vec2 vVtUV       = I1DTo2D( [@4].x, float(g_iVtMapSize) );
    vec3 vSampledPos = texture2D( s0, vVtUV ).xyz;

    // Transform deformed position
    Out.vPos  = gl_ModelViewProjectionMatrix * vec4( vSampledPos, 1.0 );
    Out.vTPos = Out.vPos;

[@5]
[@13]
[@12]

    // Return contents of Out
[@15]  
}