////////////////////////////////////////////////////////////////////////////////
// $RCSfile:$ $Revision:$
// $Source:$
// $Date:$ $Author:$ $State:$
////////////////////////////////////////////////////////////////////////////////
/// \file CIntObj.cpp 
/// \brief Definition of CIntObj methods.
///
/// \author Harlen Costa Batagelo (mailto:harlen@dca.fee.unicamp.br)
///
/// See Copyright Notice at the end of the file.
///
/// $Id:$
////////////////////////////////////////////////////////////////////////////////

#include "CIntObj.h"

#include <vector>
#include <algorithm>
using std::find;
using namespace std;

//#define DEBUG_VS ///< Uncomment this line to debug vertex shaders.
//#define DEBUG_PS ///< Uncomment this line to debug pixel shaders. 

#if defined(D3D9)
/// Chunk of effect file containing the vertex shader and pixel shader
/// responsible for updating the texture of deformed vertex attributes:
/// vertex position and texture coordinates (D3D9).
static const char *g_szVtDefTex = {
"float4x4 g_mP; // Proj matrix.                                         \n"
"// Size of a dimension of the vertex attribute map, in texels.         \n"
"int g_iVtMapSize;                                                      \n"
"                                                                       \n"
"// Convert 1D index to 2D screen pixels.                               \n"
"// i = 1D index, s = size of a screen line, in pixels                  \n"
"float2 I1DTo2D( float i, float s )                                     \n"
"{ return float2( i%s, (int)(i/s) ); }                                  \n"
"                                                                       \n"
"struct VSDEFORM_PSOUT                                                  \n"
"{                                                                      \n"
"	float4 vGridPos : POSITION;  // Grid position in screen space       \n"
"	float4 vPos     : TEXCOORD0; // Deformed position                   \n"
"   float4 vTex     : TEXCOORD1; // Deformed texture coordinates        \n"
"};                                                                     \n"
"                                                                       \n"
"// Vertex shader for vertex-level deformations                         \n"
"VSDEFORM_PSOUT VSDeformVS( VSDEFORM In )                               \n"
"{                                                                      \n"
"	VSDEFORM Def = fPreVSDeform( In );                                  \n"
"	VSDEFORM_PSOUT Out;                                                 \n"
"	Out.vGridPos = mul(                                                 \n"
"	    float4( I1DTo2D( Def.[@1].x, g_iVtMapSize ), 0, 1 ), g_mP );    \n"
"	Out.vPos     = Def.vPos;                                            \n"
"   Out.vTex     = Def.[@2];                                            \n"
"	return Out;                                                         \n"
"}                                                                      \n"
"                                                                       \n"
"// Pixel shader for vertex-level deformations                          \n"
"struct PSOUT { float4 vColor[2] : COLOR; };                            \n"
"PSOUT VSDeformPS( VSDEFORM_PSOUT In )                                  \n"
"{                                                                      \n"
"	PSOUT Out;                                                          \n"
"	Out.vColor[0] = In.vPos;                                            \n"
"	Out.vColor[1] = In.vTex;                                            \n"
"	return Out;                                                         \n"
"}                                                                      \n"
"                                                                       \n"
"// Effect technique for vertex-level deformations                      \n"
"technique VSDeform                                                     \n"
"{                                                                      \n"
"    pass P0                                                            \n"
"    {                                                                  \n"
"        VertexShader = compile vs_3_0 VSDeformVS();                    \n"
"        PixelShader  = compile ps_3_0 VSDeformPS();                    \n"
"    }                                                                  \n"
"}                                                                      \n"
};

/// Chunk of effect file containing the vertex shader and pixel shader
/// responsible for updating the texture of deformed vertex attributes:
/// vertex position (D3D9).
static const char *g_szVtDef = {
"float4x4 g_mP; // Proj matrix.                                          \n"
"// Size of a dimension of the vertex attribute map, in texels.          \n"
"int g_iVtMapSize;                                                       \n"
"                                                                        \n"
"// Convert 1D index to 2D screen pixels.                                \n"
"// i = 1D index, s = size of a screen line, in pixels                   \n"
"float2 I1DTo2D(float i, float s)                                        \n"
"{ return float2(i%s,(int)(i/s)); }                                      \n"
"                                                                        \n"
"struct VSDEFORM_PSOUT                                                   \n"
"{                                                                       \n"
"	float4 vGridPos : POSITION;  // Grid position in screen space        \n"
"	float4 vPos     : TEXCOORD0; // Deformed position                    \n"
"};                                                                      \n"
"                                                                        \n"
"// Vertex shader for vertex-level deformations                          \n"
"VSDEFORM_PSOUT VSDeformVS( VSDEFORM In )                                \n"
"{                                                                       \n"
"	VSDEFORM Def = fPreVSDeform( In );                                   \n"
"	VSDEFORM_PSOUT Out;                                                  \n"
"	Out.vGridPos = mul(                                                  \n"
"	    float4( I1DTo2D( Def.[@1].x, g_iVtMapSize ), 0, 1 ), g_mP );     \n"
"	Out.vPos     = Def.vPos;                                             \n"
"	return Out;                                                          \n"
"}                                                                       \n"
"                                                                        \n"
"// Pixel shader for vertex-level deformations                           \n"
"float4 VSDeformPS( VSDEFORM_PSOUT In ) : COLOR                          \n"
"{                                                                       \n"
"	return In.vPos;                                                      \n"
"}                                                                       \n"
"                                                                        \n"
"// Effect technique for vertex-level deformations                       \n"
"technique VSDeform                                                      \n"
"{                                                                       \n"
"    pass P0                                                             \n"
"    {                                                                   \n"
"        VertexShader = compile vs_2_0 VSDeformVS();                     \n"
"        PixelShader  = compile ps_2_0 VSDeformPS();                     \n"
"    }                                                                   \n"
"}                                                                       \n"
};
#elif defined(OGL2)
/// Chunk of vertex shader responsible for updating the texture of deformed
/// vertex attributes: vertex position and texture coordinates (OGL2).
static const char *g_szVtDefTex = {
"// Size of a dimension of the vertex attribute map, in texels.             \n"
"uniform int g_iVtMapSize;                                                  \n"
"                                                                           \n"
"// Convert 1D index to 2D screen pixels.                                   \n"
"// i = 1D index, s = size of a screen line, in pixels                      \n"
"vec2 I1DTo2D(float i, float s) { return vec2(mod(i,s),int(i/s))+vec2(.5); }\n"
"                                                                           \n"
"// Vertex shader for vertex-level deformations                             \n"
"void main( )                                                               \n"
"{                                                                          \n"
"    VSDEFORM In;                                                           \n"
"[@3]                                                                       \n"
"    VSDEFORM Def = fPreVSDeform( In );                                     \n"
"    gl_Position = gl_ProjectionMatrix *                                    \n" 
"        vec4( I1DTo2D( Def.[@1].x, float(g_iVtMapSize) ), 0, 1 );          \n"
"    gl_TexCoord[0] = Def.vPos;                                             \n"
"    gl_TexCoord[1] = Def.[@2];                                             \n"
"}                                                                          \n"
};

/// Chunk of vertex shader responsible for updating the texture of deformed
/// vertex attributes: vertex position (OGL2).
static const char *g_szVtDef = {
"// Size of a dimension of the vertex attribute map, in texels.             \n"
"uniform int g_iVtMapSize;                                                  \n"
"                                                                           \n"
"// Convert 1D index to 2D screen pixels.                                   \n"
"// i = 1D index, s = size of a screen line, in pixels                      \n"
"vec2 I1DTo2D(float i, float s) { return vec2(mod(i,s),int(i/s))+vec2(.5); }\n"
"                                                                           \n"
"// Vertex shader for vertex-level deformations                             \n"
"void main( )                                                               \n"
"{                                                                          \n"
"    VSDEFORM In;                                                           \n"
"[@3]                                                                       \n"
"    VSDEFORM Def = fPreVSDeform( In );                                     \n"
"    gl_Position = gl_ProjectionMatrix *                                    \n" 
"        vec4( I1DTo2D( Def.[@1].x, float(g_iVtMapSize) ), 0, 1 );          \n"
"    gl_TexCoord[0] = Def.vPos;                                             \n"
"}                                                                          \n"
};

/// Fragment shader responsible for updating the texture of deformed vertex
/// attributes: vertex position and texture coordinates (OGL2).
static const char *g_szVtDefPSTex = {
"// Pixel shader for vertex-level deformations                              \n"
"void main( )                                                               \n"
"{                                                                          \n"
"	gl_FragData[0] = gl_TexCoord[0];                                        \n"
"	gl_FragData[1] = gl_TexCoord[1];                                        \n"
"}                                                                          \n"
};

/// Fragment shader responsible for updating the texture of deformed vertex
/// attributes: vertex position (OGL2).
static const char *g_szVtDefPS = {
"// Pixel shader for vertex-level deformations                              \n"
"void main( )                                                               \n"
"{                                                                          \n"
"	gl_FragData[0] = gl_TexCoord[0];                                        \n"
"}                                                                          \n"
};

// Used for debugging
void checkGLErrors( const char *label )
{
    GLenum errCode;
    const GLubyte *errStr;
    if( ( errCode = glGetError( ) ) != GL_NO_ERROR ) 
	{
		errStr = gluErrorString(errCode);
		printf("OpenGL ERROR: ");
		printf((char*)errStr);
		printf("(Label: ");
		printf(label);
		printf(")\n.");
    }
}

#endif

/// Global variables used to indicate which parameters are used in each 
/// render pass when creating the encoding shaders.
bool g_bAttNormal  = false,
	 g_bAttTBN     = false,
	 g_bAttCurv    = false,
	 g_bAttTensor2 = false,
	 g_bAttTensor3 = false;

////////////////////////////////////////////////////////////////////////////////
/// \brief Get the VSSEMANTIC bound to the given INTSEMANTIC.
///
/// \param iIntSemantic INTSEMANTIC to look for.
/// \param iVsSemantic Resulting VSSEMANTIC or VSSEMANTIC_END if binding was
///        not found (out).
/// \param iIdx When there are many instances of INTSEMANTIC, this is the index
///        (zero based) to identify which one to return.
///
/// \return True if the binding was found.
////////////////////////////////////////////////////////////////////////////////
bool CIntObj::GetSemantic( INTSEMANTIC iIntSemantic, VSSEMANTIC &iVsSemantic,
						   int iIdx )
{
	int iCt = 0;
	int iIdxCt = 0;
	iVsSemantic = VSSEMANTIC_END;
	while( m_pManager->m_pBindings[iCt].iVSSemantic != VSSEMANTIC_END )
	{
		if( m_pManager->m_pBindings[iCt].iIntSemantic == iIntSemantic )
		{
			iVsSemantic = m_pManager->m_pBindings[iCt].iVSSemantic;

			if( iIdxCt == iIdx )
				break;
			else
				iIdxCt++;
		}
		iCt++;
	}

	if( iVsSemantic == VSSEMANTIC_END )
		return false;

	return true;
} // CIntObj::GetSemantic

////////////////////////////////////////////////////////////////////////////////
/// \brief Get the string with the semantic of the vertex shader input
///        corresponding to a given vertex semantic.
///
/// \param iVtSemantic Any value of the VSSEMANTIC enum.
/// \param szResult Resulting string (e.g. TEXCOORD1). This should be a 
///        string with allocated memory of at least 10 bytes.
////////////////////////////////////////////////////////////////////////////////
void CIntObj::GetSemanticStr( VSSEMANTIC iVtSemantic, char *szResult )
{
	switch( iVtSemantic )
	{
	case VSSEMANTIC_NORMAL   : strcpy( szResult, "NORMAL"    ); break;
	case VSSEMANTIC_COLOR0   : strcpy( szResult, "COLOR0"    ); break;
	case VSSEMANTIC_COLOR1   : strcpy( szResult, "COLOR1"    ); break;
	case VSSEMANTIC_TEXCOORD0: strcpy( szResult, "TEXCOORD0" ); break;
	case VSSEMANTIC_TEXCOORD1: strcpy( szResult, "TEXCOORD1" ); break;
	case VSSEMANTIC_TEXCOORD2: strcpy( szResult, "TEXCOORD2" ); break;
	case VSSEMANTIC_TEXCOORD3: strcpy( szResult, "TEXCOORD3" ); break;
	case VSSEMANTIC_TEXCOORD4: strcpy( szResult, "TEXCOORD4" ); break;
	case VSSEMANTIC_TEXCOORD5: strcpy( szResult, "TEXCOORD5" ); break;
	case VSSEMANTIC_TEXCOORD6: strcpy( szResult, "TEXCOORD6" ); break;
	case VSSEMANTIC_TEXCOORD7: strcpy( szResult, "TEXCOORD7" ); break;
	}
} // CIntObj::GetSemanticStr

////////////////////////////////////////////////////////////////////////////////
/// \brief Get the string with the parameter of the vertex shader input
///        corresponding to a given vertex semantic.
///
/// \param iVtSemantic Any value of the VSSEMANTIC enum.
/// \param szResult Resulting string. This should be a string with allocated
///        memory of at least 10 bytes.
////////////////////////////////////////////////////////////////////////////////
void CIntObj::GetParamStr( VSSEMANTIC iVtSemantic, char *szResult )
{
	switch( iVtSemantic )
	{
	case VSSEMANTIC_NORMAL   : strcpy( szResult, "vNormal" ); break;
	case VSSEMANTIC_COLOR0   : strcpy( szResult, "vColor0" ); break;
	case VSSEMANTIC_COLOR1   : strcpy( szResult, "vColor1" ); break;
	case VSSEMANTIC_TEXCOORD0: strcpy( szResult, "vTex0"   ); break;
	case VSSEMANTIC_TEXCOORD1: strcpy( szResult, "vTex1"   ); break;
	case VSSEMANTIC_TEXCOORD2: strcpy( szResult, "vTex2"   ); break;
	case VSSEMANTIC_TEXCOORD3: strcpy( szResult, "vTex3"   ); break;
	case VSSEMANTIC_TEXCOORD4: strcpy( szResult, "vTex4"   ); break;
	case VSSEMANTIC_TEXCOORD5: strcpy( szResult, "vTex5"   ); break;
	case VSSEMANTIC_TEXCOORD6: strcpy( szResult, "vTex6"   ); break;
	case VSSEMANTIC_TEXCOORD7: strcpy( szResult, "vTex7"   ); break;
	}
} // CIntObj::GetParamStr

////////////////////////////////////////////////////////////////////////////////
/// \brief Get the string with the built-in GLSL parameter of the vertex shader 
///        input corresponding to a given vertex semantic.
///
/// \param iVtSemantic Any value of the VSSEMANTIC enum.
/// \param szResult Resulting string. This should be a string with allocated
///        memory of at least 18 bytes.
////////////////////////////////////////////////////////////////////////////////
void CIntObj::GetParamStrGLSL( VSSEMANTIC iVtSemantic, char *szResult )
{
	switch( iVtSemantic )
	{
	case VSSEMANTIC_NORMAL   : strcpy( szResult, "gl_Normal"         ); break;
	case VSSEMANTIC_COLOR0   : strcpy( szResult, "gl_Color"          ); break;
	case VSSEMANTIC_COLOR1   : strcpy( szResult, "gl_SecondaryColor" ); break;
	case VSSEMANTIC_TEXCOORD0: strcpy( szResult, "gl_MultiTexCoord0" ); break;
	case VSSEMANTIC_TEXCOORD1: strcpy( szResult, "gl_MultiTexCoord1" ); break;
	case VSSEMANTIC_TEXCOORD2: strcpy( szResult, "gl_MultiTexCoord2" ); break;
	case VSSEMANTIC_TEXCOORD3: strcpy( szResult, "gl_MultiTexCoord3" ); break;
	case VSSEMANTIC_TEXCOORD4: strcpy( szResult, "gl_MultiTexCoord4" ); break;
	case VSSEMANTIC_TEXCOORD5: strcpy( szResult, "gl_MultiTexCoord5" ); break;
	case VSSEMANTIC_TEXCOORD6: strcpy( szResult, "gl_MultiTexCoord6" ); break;
	case VSSEMANTIC_TEXCOORD7: strcpy( szResult, "gl_MultiTexCoord7" ); break;
	}
} // CIntObj::GetParamStrGLSL

////////////////////////////////////////////////////////////////////////////////
/// \brief Create a pixel shader encoder for the given render pass.
///
/// This is a helper function used by CIntObj::CreateShaders.
///
/// \param iCurRP Number of the current render pass (starting from 0).
/// \param iCurRT Number of the current render target (starting from 0).
/// \param iCurComp Number of current color component used (from 0 to 3).
/// \param szShader Pointer to the text string containing the shader code.
/// \param szReplace Text string to replace the tag [@7] in the shader code.
///
/// \return True on success.
////////////////////////////////////////////////////////////////////////////////
#if defined(D3D9)
bool CIntObj::CreatePSEncoder( int iCurRP, int iCurRT, int iCurComp, 
							   char **szShader, char *szReplace )
#elif defined(OGL2)
bool CIntObj::CreatePSEncoder( int iCurRP, int iCurRT, int iCurComp, 
							   char **szVSShader, char **szPSShader, 
							   char *szReplace )
#endif
{
#if defined(D3D9)
	DWORD dwShaderFlags = 0;

#ifdef DEBUG_VS
	dwShaderFlags |= D3DXSHADER_FORCE_VS_SOFTWARE_NOOPT;
#endif
#ifdef DEBUG_PS
	dwShaderFlags |= D3DXSHADER_FORCE_PS_SOFTWARE_NOOPT;
#endif

#endif

	const int iTempSz = 8192;
	char chComp[5] = "rgba";
	char szTemp[iTempSz] = {0};

	if( iCurComp == 0 && iCurRT )
		iCurRT--;
	else
	{
		// Fill color components not used
		while( iCurComp < 4 )
		{
#if defined(D3D9)
			_snprintf( szTemp, iTempSz, "  Out.vColor[%d].", iCurRT );
#elif defined(OGL2)
			_snprintf( szTemp, iTempSz, "  gl_FragData[%d].", iCurRT );
#endif
			strcat   ( szReplace, szTemp );
			strncat  ( szReplace, &chComp[iCurComp], 1 );
			strcat   ( szReplace, " = 1.0;\n" );

			iCurComp++;
		}
	}

	// Replace [@7] with pixel shader output (pixel shader)
	char *szOut = NULL;
#if defined(D3D9)
	ReplaceString( 1, *szShader, "[@7]", szReplace, &szOut );
	SAFE_DELETE_ARRAY( *szShader ); *szShader = szOut;
#elif defined(OGL2)
	ReplaceString( 1, *szPSShader, "[@7]", szReplace, &szOut );
	SAFE_DELETE_ARRAY( *szPSShader ); *szPSShader = szOut;
#endif
	szReplace[0] = 0x00;

#if defined(D3D9)
	// Replace [@10] with number of render targets used
	szOut = NULL;
	_snprintf( szTemp, iTempSz, "%d", iCurRT + 1 );
	ReplaceString( 1, *szShader, "[@10]", szTemp, &szOut ); // m_iNumMRTs
	SAFE_DELETE_ARRAY( *szShader ); 
	*szShader = szOut;
#endif

	szTemp[0] = 0x00;

	// Using normal?
	if( g_bAttNormal || g_bAttTBN )
	{
		strcat( szTemp, 
#if defined(D3D9)
			"  float3 vN = tex2Dlod( s1, float4( vVtUV, 0, 0 ) ).xyz;\n"
//			"  Out.vTN = mul( float4(vN, 0), g_mITW );\n\n" );
			"  Out.vTN = vN;\n\n" 
#elif defined(OGL2)
			"  vec3 vN = texture2D( s1, vVtUV ).xyz;\n"
			"  gl_TexCoord[9] = vN;\n\n"
#endif
		);
	}
	else if( m_pManager->m_bAttNormal || m_pManager->m_bAttTBN )
	{
#if defined(D3D9)
		strcat( szTemp, "  Out.vTN = 0;\n\n" );
#elif defined(OGL2)
		strcat( szTemp, "  gl_TexCoord[9] = 0;\n\n" );
#endif
	}

	// Using TBN frame?
	if( g_bAttTBN )
	{
		strcat( szTemp, 
#if defined(D3D9)
			"  float4 vT = tex2Dlod( s3, float4( vVtUV, 0, 0 ) );\n"
			"  float3 vB = cross( vT.xyz, vN ) * vT.w;\n"
//			"  Out.vTT = mul( float4(vT.xyz,0), g_mITW );\n"
//			"  Out.vTB = mul( float4(vB, 0), g_mITW );\n\n" );
			"  Out.vTT = vT.xyz;\n"
			"  Out.vTB = vB;\n\n"
#elif defined(OGL2)
			"  vec4 vT = texture2D( s3, vVtUV );\n"
			"  vec3 vB = cross( vT.xyz, vN ) * vT.w;\n"
//			"  gl_TexCoord[10] = mul( vec4(vT.xyz,0), g_mITW );\n"
//			"  gl_TexCoord[11] = mul( vec4(vB, 0), g_mITW );\n\n" );
			"  gl_TexCoord[10] = vT.xyz;\n"
			"  gl_TexCoord[11] = vB;\n\n"
#endif
		);
	}
	else if( m_pManager->m_bAttTBN )
	{
#if defined(D3D9)
		strcat( szTemp, "  Out.vTT = 0;\n"
			            "  Out.vTB = 0;\n\n" );
#elif defined(OGL2)
		strcat( szTemp, "  gl_TexCoord[10] = 0;\n"
			            "  gl_TexCoord[11] = 0;\n\n" );
#endif
	}

	// Using curvature?
	if( g_bAttCurv )
	{
		strcat( szTemp,
#if defined(D3D9)
			"  float4 vP1K1 = tex2Dlod( s4, float4( vVtUV, 0, 0 ) );\n"
			"  float4 vP2K2 = tex2Dlod( s5, float4( vVtUV, 0, 0 ) );\n"
			"  Out.vP1K1  = vP1K1;\n"
			"  Out.vP2K2  = vP2K2;\n"
//			"  Out.vP1K1  = float4( mul( float4(vP1K1.xyz,0), g_mITW ).xyz, vP1K1.w );\n"
//			"  Out.vP2K2  = float4( mul( float4(vP2K2.xyz,0), g_mITW ).xyz, vP2K2.w );\n"
#elif defined(OGL2)
			"  vec4 vP1K1 = texture2D( s4, vVtUV );\n"
			"  vec4 vP2K2 = texture2D( s5, vVtUV );\n"
			"  gl_TexCoord[12] = vP1K1;\n"
			"  gl_TexCoord[13] = vP2K2;\n"
//			"  Out.vP1K1  = float4( mul( float4(vP1K1.xyz,0), g_mITW ).xyz, vP1K1.w );\n"
//			"  Out.vP2K2  = float4( mul( float4(vP2K2.xyz,0), g_mITW ).xyz, vP2K2.w );\n"
#endif
			);
	}
	else if( m_pManager->m_bAttCurv )
	{
#if defined(D3D9)
		strcat( szTemp, "  Out.vP1K1 = 0;\n"
			            "  Out.vP2K2 = 0;\n\n" );
#elif defined(OGL2)
		strcat( szTemp, "  gl_TexCoord[12] = 0;\n"
			            "  gl_TexCoord[13] = 0;\n\n" );
#endif
	}

	// Using 2nd fundamental form?
	if( g_bAttTensor2 )
	{
		strcat( szTemp,	
#if defined(D3D9)
			"  Out.vTensor2 = tex2Dlod( s6, float4( vVtUV, 0, 0 ) );\n"	
#elif defined(OGL2)
			"  gl_TexCoord[14] = texture2D( s6, vVtUV );\n"	
#endif
			);
	}
	else if( m_pManager->m_bAttTensor2 )
	{
#if defined(D3D9)
		strcat( szTemp, "  Out.vTensor2 = 0;\n" );
#elif defined(OGL2)
		strcat( szTemp, "  gl_TexCoord[14] = 0;\n" );
#endif
	}

	// Using tensor of curvature derivative?
	if( g_bAttTensor3 )
	{
		strcat( szTemp,
#if defined(D3D9)
			"  Out.vTensor3 = tex2Dlod( s7, float4( vVtUV, 0, 0 ) );\n"	
#elif defined(OGL2)
			"  gl_TexCoord[15] = texture2D( s7, vVtUV );\n"	
#endif
			);
	}
	else if( m_pManager->m_bAttTensor3 )
	{
#if defined(D3D9)
		strcat( szTemp, "  Out.vTensor3 = 0;\n" );
#elif defined(OGL2)
		strcat( szTemp, "  gl_TexCoord[15] = 0;\n" );
#endif
	}

	// Replace [@5] with output of PSDEFORM (vertex shader)
	szOut = NULL;
#if defined(D3D9)
	ReplaceString( 1, *szShader, "[@5]", szTemp, &szOut );
	SAFE_DELETE_ARRAY( *szShader ); *szShader = szOut;
#elif defined(OGL2)
	ReplaceString( 1, *szVSShader, "[@5]", szTemp, &szOut );
	SAFE_DELETE_ARRAY( *szVSShader ); *szVSShader = szOut;
#endif

	//
	// DEBUG: Write shader to file
	//
	FILE *fHandle = fopen( "enc_out.fx", "wb" );
	if( fHandle == NULL )
		return false;

#if defined(D3D9)
	fwrite( *szShader, sizeof( char ), strlen( *szShader ), fHandle );
#elif defined(OGL2)
	fwrite( *szVSShader, sizeof( char ), strlen( *szVSShader ), fHandle );
	fprintf( fHandle, "\n\n//------------------------\n\n" );
	fwrite( *szPSShader, sizeof( char ), strlen( *szPSShader ), fHandle );
#endif

	fclose( fHandle );

	//
	// Create encoding (indexed geometry) effect from string
	//
#if defined(D3D9)
	LPD3DXBUFFER pd3dBuffer = NULL;
	HRESULT hrResult = D3DXCreateEffect( m_pd3dDevice, *szShader,
		(UINT) strlen( *szShader ), NULL, NULL, dwShaderFlags, NULL, 
		&m_pEncodeIFx[iCurRP], &pd3dBuffer );

	// If there are errors, print them
	if( pd3dBuffer )
		fprintf( stderr, (char*) pd3dBuffer->GetBufferPointer( ) );

	if( FAILED( hrResult ) )
	{
		// TODO: Maximum number of texture samplers in vertex shader exceeded.
		return false;
	}

	// Get handle to uniform parameters in the effect
	m_hEncodeIFxIdxC[iCurRP] = m_pEncodeIFx[iCurRP]->GetParameterByName( 
		NULL, "g_fVtIdxC" );
	m_hEncodeIFxWVP[iCurRP]  = m_pEncodeIFx[iCurRP]->GetParameterByName( 
		NULL, "g_mWVP" );
	m_hEncodeIFxITW[iCurRP]  = m_pEncodeIFx[iCurRP]->GetParameterByName( 
		NULL, "g_mITW" );
#elif defined(OGL2)
		// Create program object
		m_hEncodeIPO[iCurRP] = glCreateProgramObjectARB( );

		// Create shader objects
		GLhandleARB hVSSO = glCreateShaderObjectARB( GL_VERTEX_SHADER_ARB   );
		GLhandleARB hPSSO = glCreateShaderObjectARB( GL_FRAGMENT_SHADER_ARB );
		glShaderSourceARB( hVSSO, 1, (const GLcharARB**) &szVSShader, NULL );
		glShaderSourceARB( hPSSO, 1, (const GLcharARB**) &szPSShader, NULL );
		glCompileShaderARB( hVSSO );
		glCompileShaderARB( hPSSO );

		// Check compilation status
		GLint iResult = 0;
		glGetObjectParameterivARB( hVSSO, GL_OBJECT_COMPILE_STATUS_ARB, 
								   &iResult );
		if( !iResult )
		{
	//		glGetInfoLogARB( hVSS0 );
			return false;
		}
		glGetObjectParameterivARB( hPSSO, GL_OBJECT_COMPILE_STATUS_ARB, 
								   &iResult );
		if( !iResult )
		{
	//		glGetInfoLogARB( hPSS0 );
			return false;
		}

		// Attach shaders to program
		glAttachObjectARB( m_hEncodeIPO[iCurRP], hVSSO );
		glAttachObjectARB( m_hEncodeIPO[iCurRP], hPSSO );
		// Flag shaders for deletion when they are no longer referenced
		if( hVSSO ) { glDeleteObjectARB( hVSSO ); hVSSO = 0; }
		if( hPSSO ) { glDeleteObjectARB( hPSSO ); hPSSO = 0; }

		glLinkProgramARB( m_hEncodeIPO[iCurRP] );

		// Check linkage status
		glGetObjectParameterivARB( m_hEncodeIPO[iCurRP], 
								   GL_OBJECT_LINK_STATUS_ARB, &iResult );
		if( !iResult )
		{
	//		glGetInfoLogARB( m_hEncodeIPO[iCurRP] );
			return false;
		}

		// Use vertex-level deformation shader
		glUseProgramObjectARB( m_hEncodeIPO[iCurRP] );

		// Get handlers to uniform parameters
		m_iEncodeIPOVtMapSize[iCurRP] = glGetUniformLocationARB( 
			m_hEncodeIPO[iCurRP], "g_iVtMapSize" );

		// Disable program object
		glUseProgramObjectARB( 0 );
#endif

	return true;
} // CIntObj::CreatePSEncoder

////////////////////////////////////////////////////////////////////////////////
/// \brief Class constructor.
///
/// Initialize member variables to default values.
///
/// \param szAdjWriteFile Name of the file where the adjacency data is to
///        be written. This parameter is optional and should be set to NULL
///        when not used.
/// \param szAdjReadFile Name of the file from which the adjacency data is to
///        be read. This parameter is optional and should be set to NULL when
///        not used.
////////////////////////////////////////////////////////////////////////////////
CIntObj::CIntObj( CIntManager *pManager, const char *szAdjWriteFile, 
				  const char *szAdjReadFile )
{
	m_pManager = pManager;

	m_bVisible        = true;

	m_szPreDefVS      = NULL;
	m_szDefPS         = NULL;
	m_szPostDefVS     = NULL;
	m_bUsePreDefVS    = false;
	m_bUsePxDef       = false;
	m_bUsePostDefVS   = false;

	m_iNumVt          = 0;
	m_pVtPos          = NULL;
	m_pVtTexCoord     = NULL;
	m_pVtNormal       = NULL;
	m_iVtAttTexSize   = 1;

	m_iNumFace        = 0;
	m_pFaceIdx        = NULL;
	m_iFaceIdxTexSize = 1;

	m_pUserData       = NULL;
	m_fUpdate         = NULL;
	m_fRenderIdx      = NULL;
	m_fRenderNonIdx   = NULL;

	m_bDefTex         = false;

	m_szAdjWriteFile  = NULL;
	m_szAdjReadFile   = NULL;

	// Make a copy of the file names
	if( szAdjWriteFile )
	{
		m_szAdjWriteFile = new char[ strlen( szAdjWriteFile ) + 1 ];
		m_szAdjWriteFile[0] = NULL;

		strcpy( m_szAdjWriteFile, szAdjWriteFile );
	}

	if( szAdjReadFile )
	{
		m_szAdjReadFile = new char[ strlen( szAdjReadFile ) + 1 ];
		m_szAdjReadFile[0] = NULL;

		strcpy( m_szAdjReadFile, szAdjReadFile );
	}

#if defined(D3D9)
	m_pd3dDevice = NULL;

	m_pVtDefFx        = NULL;
	m_hVtDefFxProj    = NULL;
	m_hVtDefFxVtMapSz = NULL;

	for( int iCt = 0; iCt < MAX_RENDER_PASSES; iCt++ )
	{
		m_pEncodeIFx    [iCt] = NULL;
		m_hEncodeIFxIdxC[iCt] = NULL;
		m_hEncodeIFxWVP [iCt] = NULL;
		m_hEncodeIFxITW [iCt] = NULL;
	}

	m_pEncodeNIFx         = NULL;
	m_hEncodeNIFxIdxC     = NULL;
	m_hEncodeNIFxWVP      = NULL;
	m_hEncodeNIFxITW      = NULL;

	m_pRenderFx           = NULL;
	m_hRenderFxW          = NULL;
	m_hRenderFxV          = NULL;
	m_hRenderFxWVP        = NULL;
	m_hRenderFxITW        = NULL;

	m_pd3dVtPosTex        = NULL;
	m_pd3dVtTexCoordTex   = NULL;
	m_pd3dVtNormalTex     = NULL;
	m_pd3dVtTanHTex       = NULL;
	m_pd3dVtTensor2Tex    = NULL;
	m_pd3dVtTensor3Tex    = NULL;
	m_pd3dVtP1K1Tex       = NULL;
	m_pd3dVtP2K2Tex       = NULL;

	for( int iCt = 0; iCt < 4; iCt++ )
		m_pd3dVtCacheTex[iCt] = NULL;

	m_pd3dFaceIdxTex      = NULL;
	m_pd3dAdjFaceTex      = NULL;
	m_pd3dAdjVtTex        = NULL;
	m_pd3dAdjAddTex       = NULL;

	m_bInitDOCalled       = false;
	m_bRestoreDOCalled    = false;
	m_bInvalidateDOCalled = false;
	m_bDeleteDOCalled     = false;

	D3DXMatrixIdentity( &m_mLocal );
#elif defined(OGL2)
	m_hVtDefPO            = 0;
	m_iVtDefPOVtMapSize   = 0;

	for( int iCt = 0; iCt < MAX_RENDER_PASSES; iCt++ )
	{
		m_hEncodeIPO         [iCt] = 0;
		m_iEncodeIPOVtMapSize[iCt] = 0;
	}

	m_hEncodeNIPO          = 0;
	m_iEncodeNIPOVtMapSize = 0;

	m_hRenderPO            = 0;
	m_iRenderPOVtMapSize   = 0;

	m_uiVtTexFBO           = 0;
	m_uiVtPosTexRT         = 0;
	m_uiVtTexCoordTexRT    = 0;
	m_uiVtNormalTexRT      = 0;
	m_uiVtTanHTexRT        = 0;
	m_uiVtTensor2TexRT     = 0;
	m_uiVtTensor3TexRT     = 0;
	m_uiVtP1K1TexRT        = 0;
	m_uiVtP2K2TexRT        = 0;

	for( int iCt = 0; iCt < 4; iCt++ )
		m_uiVtCacheTex[iCt] = 0;
	
	m_uiFaceIdxTex         = 0;
	m_uiAdjFaceTex         = 0;
	m_uiAdjVtTex           = 0;
	m_uiAdjAddTex          = 0;

	for( int iCt = 0; iCt < 16; iCt++ )
		m_mLocal[iCt] = 0.0f;
	m_mLocal[0] = m_mLocal[4] = m_mLocal[8] = m_mLocal[12] = 1.0f;
#endif
} // CIntObj::CIntObj

////////////////////////////////////////////////////////////////////////////////
/// \brief Class destructor.
///
/// Delete resources allocated in the constructor.
////////////////////////////////////////////////////////////////////////////////
CIntObj::~CIntObj( )
{
#if defined(OGL2)
	// Disable shaders
	glUseProgramObjectARB( 0 );

	// Clean up program object (and automatically the attached shader objects)
	if( m_hVtDefPO ) { glDeleteObjectARB( m_hVtDefPO ); m_hVtDefPO = 0; }
#endif

	SAFE_DELETE_ARRAY( m_szAdjReadFile  );
	SAFE_DELETE_ARRAY( m_szAdjWriteFile );

	SAFE_DELETE_ARRAY( m_pVtPos         );
	SAFE_DELETE_ARRAY( m_pVtTexCoord    );
	SAFE_DELETE_ARRAY( m_pVtNormal      );
	SAFE_DELETE_ARRAY( m_pFaceIdx       );
	SAFE_DELETE_ARRAY( m_szPreDefVS     );
	SAFE_DELETE_ARRAY( m_szDefPS        );
	SAFE_DELETE_ARRAY( m_szPostDefVS    );
} // CIntObj::~CIntObj

////////////////////////////////////////////////////////////////////////////////
/// \brief Set the vertex attributes of the original geometry: 3D position 
///        in object space and texture coordinates.
///
/// This is necessary when vertex-level deformation functions are used, or when
/// computing differential geometry attributes of the model. The texture
/// coordinates are demanded only for computing TBN frames.
///
/// \param pVtPos Pointer to an array of 3D coordinates containing the position
///        of each vertex, in object space. The array should be packed like
///        [x1, y1, z1, x2, y2, z2, ..., xn, yn, zn], where x1, y1 and z1 are 
///        the 3D coordinates of the first vertex; x2, y2 and z2 are the 3D
///        coordinates of the second vertex; and so on.
/// \param pVtTexCoord Pointer to an array of 2D texture coordinates of each 
///        vertex. The array should be packed like [u1, v1, u2, v2, ..., 
///        un, vn], where u1 and v1 are the texture coordinates of the first 
///        vertex; u2 and v2 are the texture coordinates of the second vertex; 
///        and so on. This parameter may be NULL.
/// \param pVtNormal Pointer to an array of 3D coordinates containing the
///        normal vector of each vertex, in object space. The array should be 
///        packed like [x1, y1, z1, x2, y2, z2, ..., xn, yn, zn], where x1, y1 
///        and z1 are the 3D coordinates of the first vertex; x2, y2 and z2 
///        are the 3D coordinates of the second vertex; and so on. This
///        parameter may be NULL.
/// \param iSize Number of vertices in the arrays.
///
/// \return  0 on success.
///         -1 if an argument is invalid.
///         -2 if there is not enough memory.
///         -3 if there is an error while creating the internal data structures.
////////////////////////////////////////////////////////////////////////////////
int CIntObj::SetVertexBuffer( float *pVtPos, float *pVtTexCoord, 
							  float *pVtNormal, int iSize )
{
	if( pVtPos == NULL || iSize <= 0 )
		return -1;

	// Make a copy of the vertex position buffer
	SAFE_DELETE_ARRAY( m_pVtPos );
	m_pVtPos = new float[ iSize * 3 ];
	if( m_pVtPos == NULL ) return -2;
	memcpy( m_pVtPos, pVtPos, iSize * 3 * sizeof( float ) );

	// Make a copy of the vertex texture coordinates buffer
	SAFE_DELETE_ARRAY( m_pVtTexCoord );
	if( pVtTexCoord )
	{
		m_pVtTexCoord = new float[ iSize * 2 ];
		if( m_pVtTexCoord == NULL ) return -2;
		memcpy( m_pVtTexCoord, pVtTexCoord, iSize * 2 * sizeof( float ) );
	}

	// Make a copy of the vertex normal buffer
	SAFE_DELETE_ARRAY( m_pVtNormal );
	if( pVtNormal )
	{
		m_pVtNormal = new float[ iSize * 3 ];
		if( m_pVtNormal == NULL ) return -2;
		memcpy( m_pVtNormal, pVtNormal, iSize * 3 * sizeof( float ) );
	}

	m_iNumVt = iSize;

	// Compute minimum size required by a vertex attribute texture
	int iPow = 1;
	while( 1 )
	{
		m_iVtAttTexSize = 1 << iPow++;
		if( m_iVtAttTexSize * m_iVtAttTexSize >= m_iNumVt )
			break;
	}

	// Need to re-create render target textures
	if( DestroyRTs( ) )	return -3;
	if( CreateRTs ( ) ) return -3;

	return 0;
} // CIntObj::SetVertexBuffer

////////////////////////////////////////////////////////////////////////////////
/// \brief Set the adjacency data of each vertex.
///
/// For triangle meshes, the adjacency data is the index buffer of the mesh,
/// considering that the model is described by a triangle list. The indices
/// refer to the vertices provided by CIntObj::SetVertexBuffer.
///
/// \param pFaceIdx Pointer to an array of triangle indices. Each set of three
///        indices in the array refers to the vertex indices of a triangle. 
/// \param iSize Number of triangles in the array.
///
/// \return  0 on success.
///         -1 if an argument is invalid.
///         -2 if there is not enough memory.
///         -3 if there is an error while creating the internal data structures.
////////////////////////////////////////////////////////////////////////////////
int CIntObj::SetIndexBuffer( int *pFaceIdx, int iSize )
{
	if( pFaceIdx == NULL || iSize <= 0 )
		return -1;

	// Make a copy of the index buffer
	SAFE_DELETE_ARRAY( m_pFaceIdx );
	m_pFaceIdx = new int[ iSize * 3 ];
	if( m_pFaceIdx == NULL ) return -2;
	memcpy( m_pFaceIdx, pFaceIdx, sizeof( int ) * iSize * 3 );

	m_iNumFace = iSize;

	// Compute minimum size required by a face index texture
	int iPow = 1;
	while( 1 )
	{
		m_iFaceIdxTexSize = 1 << iPow++;
		if( m_iFaceIdxTexSize * m_iFaceIdxTexSize >= m_iNumFace )
			break;
	}

	// Need to re-create render target textures
	if( DestroyRTs( ) )	return -3;
	if( CreateRTs ( ) ) return -3;

	// Need to re-create textures which are not render target textures
	if( DestroyNRTs( ) ) return -3;
	if( CreateNRTs ( ) ) return -3;

	return 0;
} // CIntObj::SetIndexBuffer

////////////////////////////////////////////////////////////////////////////////
/// \brief Set the vertex-level deformation function of this model to be used
///        before the estimation of differential geometry properties.
///
/// \param szFunc Pointer to the null-terminated string containing the code of 
///        the deformation function (in GLSL for OpenGL applications, and HLSL
///        for Direct3D applications). This is a function with name fPreVSDeform, 
///        that accepts a parameter of type VSDEFORM (D3D-only) and returns a 
///        VSDEFORM. An example that deforms the vertex position by a sine
///        function is shown as follows:
///
///        In Direct3D:
///
///        VSDEFORM fPreVSDeform( VSDEFORM In )
///        {
///            VSDEFORM Out;
///            // Deformation code:
///            Out.vPos = In.vPos + sin( In.vPos );
///            return Out;
///        }
///
///        In OpenGL:
///
///        VSDEFORM fPreVSDeform( )
///        {
///            VSDEFORM Out;
///            // Deformation code:
///            Out.vPos = gl_Vertex + sin( gl_Vertex );
///            return Out;
///        }
///
///        If szFunc is set to NULL, the vertex deformation will be disabled.
///        VSDEFORM is a data structure with parameters defined according 
///        to the vertex semantics of the original geometry. See the 
///        documentation of VSSEMANTIC.
/// \param bDefTex Whether the vertex-level deformation function also
///        deform texture coordinates. In that case, the VSDEFORM structure
///        will have a member that corresponds to the attribute with
///        semantic INTSEMANTIC_TEXCOORD.
///
/// \return  0 on success.
///         -1 if there are not enough memory to create the vertex deformation
///            shader.
///         -2 if there is not a vertex semantic binding with 
///            INTSEMANTIC_VERTEXID.
///         -3 if there is a shader compilation error.
///         -4 if bDefTex is true and there is not a vertex semantic binding
///            with INTSEMANTIC_TEXCOORD.
////////////////////////////////////////////////////////////////////////////////
int CIntObj::SetPreVertexDeform( char *szFunc /*, bool bDefTex*/ )
{
	if( szFunc == NULL )
	{
		// Disable vertex-level deformation
		SAFE_DELETE_ARRAY( m_szPreDefVS );
		m_bUsePreDefVS = false;
		return 0;
	}

	//m_bDefTex = bDefTex;
	m_bDefTex = false;

	// Look for INTSEMANTIC_VERTEXID vertex semantic
	int iCt = 0;
	VSSEMANTIC iVtIDSemantic;
	if( !GetSemantic( INTSEMANTIC_VERTEXID, iVtIDSemantic ) )
		return -2;

	iCt = 0;
	VSSEMANTIC iVtTexCoordSemantic = VSSEMANTIC_END;
	if( m_bDefTex )
	{
		if( !GetSemantic( INTSEMANTIC_TEXCOORD, iVtTexCoordSemantic ) )
			return -4;
	}

	// Create text string containing the vertex-level deformation shader
	SAFE_DELETE_ARRAY( m_szPreDefVS );
	int iAllocSize = (int) strlen( m_pManager->m_szVSDeform ) + 
					 (int) strlen( szFunc ) + 64 * 80;
	m_szPreDefVS = new char[ iAllocSize ];
	if( m_szPreDefVS == NULL )
		return -1;
	m_szPreDefVS[0] = 0x00;

	// First append definition of VSDEFORM
	strcat( m_szPreDefVS, m_pManager->m_szVSDeform );
	strcat( m_szPreDefVS, "\n" );

	// Then the user-defined deformation function
	strcat( m_szPreDefVS, szFunc );
	strcat( m_szPreDefVS, "\n" );

	if( m_bDefTex )
	{
		// Then the rest
		strcat( m_szPreDefVS, g_szVtDefTex );

		// Replace occurrence of [@1] with
		// structure member associated with INTSEMANTIC_VERTEXID
		char *szReplace = new char[32];
		if( szReplace == NULL )
			return -1;

		GetParamStr( iVtIDSemantic, szReplace );

		char *szOut;
		ReplaceString( 1, m_szPreDefVS, "[@1]", szReplace, &szOut );
		SAFE_DELETE_ARRAY( m_szPreDefVS );
		m_szPreDefVS = szOut;

		// Replace occurrence of [@2] with
		// structure member associated with INTSEMANTIC_TEXCOORD
		GetParamStr( iVtTexCoordSemantic, szReplace );

		char *szOut2;
		ReplaceString( 1, m_szPreDefVS, "[@2]", szReplace, &szOut2 );
		SAFE_DELETE_ARRAY( m_szPreDefVS );
		m_szPreDefVS = szOut2;

#if defined(OGL2)
		// Replace occurrence of [@3] with text
		// contained in m_pManager->m_szInputVSDeform
		char *szOut3;
		ReplaceString( 1, m_szPreDefVS, "[@3]", 
			m_pManager->m_szInputVSDeform, &szOut3 );
		SAFE_DELETE_ARRAY( m_szPreDefVS );
		m_szPreDefVS = szOut3;
#endif

		SAFE_DELETE_ARRAY( szReplace );

#if defined(D3D9)
		// Shaders are created in InitDeviceObjects
		DeleteDeviceObjects( );
		InitDeviceObjects  ( );
#elif defined(OGL2)

		// Re-initialize the shaders
		int iRes = 0;
		if( iRes = DestroyShaders( ) )
			return iRes;
		if( iRes = CreateShaders ( ) )
			return iRes;
#endif
	}
	else
	{
		// Then the rest
		strcat( m_szPreDefVS, g_szVtDef );

		// Replace occurrence of [@1] with
		// structure member associated with INTSEMANTIC_VERTEXID
		char *szReplace = new char[32];
		if( szReplace == NULL )
			return -1;

		GetParamStr( iVtIDSemantic, szReplace );

		char *szOut;
		ReplaceString( 1, m_szPreDefVS, "[@1]", szReplace, &szOut );
		SAFE_DELETE_ARRAY( m_szPreDefVS );
		m_szPreDefVS = szOut;

#if defined(OGL2)
		// Replace occurrence of [@3] with text
		// contained in m_pManager->m_szInputVSDeform
		char *szOut3;
		ReplaceString( 1, m_szPreDefVS, "[@3]", 
			m_pManager->m_szInputVSDeform, &szOut3 );
		SAFE_DELETE_ARRAY( m_szPreDefVS );
		m_szPreDefVS = szOut3;
#endif

		SAFE_DELETE_ARRAY( szReplace );

#if defined(D3D9)
		// Shaders are created in InitDeviceObjects
		DeleteDeviceObjects( );
		InitDeviceObjects  ( );
#elif defined(OGL2)
		// Re-initialize the shaders
		int iRes = 0;
		if( iRes = DestroyShaders( ) )
			return iRes;
		if( iRes = CreateShaders ( ) )
			return iRes;
#endif
	}

	m_bUsePreDefVS = true;

	return 0;
} // CIntObj::SetPreVertexDeform

////////////////////////////////////////////////////////////////////////////////
/// \brief Set the pixel-level deformation function of this model.
///
/// \param szFunc Pointer to the null-terminated string containing the code of 
///        the deformation function (in GLSL for OpenGL applications, and HLSL
///        for Direct3D applications). This is a function with name fPSDeform, 
///        that accepts a parameter of type PSDEFORM (D3D-only) and returns a 
///        PSDEFORM.
///
///        If szFunc is set to NULL, the fragment deformation will be disabled.
///        PSDEFORM is a data structure with parameters defined according 
///        to the vertex semantics of the original geometry. See the 
///        documentation of VSSEMANTIC.
///
/// \return  0 on success.
///         -1 if there are not enough memory to create the pixel deformation
///            shader.
////////////////////////////////////////////////////////////////////////////////
int CIntObj::SetPixelDeform( char *szFunc )
{
	if( szFunc == NULL )
	{
		// Disable fragment-level deformation
		m_bUsePxDef = false;
		return 0;
	}

	// Create text string containing the pixel-level deformation shader
	SAFE_DELETE_ARRAY( m_szDefPS );
	int iAllocSize = (int) strlen( m_pManager->m_szPSDeform ) + 
					 (int) strlen( szFunc ) + 16;
	m_szDefPS = new char[ iAllocSize ];
	if( m_szDefPS == NULL )
		return -1;
	m_szDefPS[0] = 0x00;

	strcat( m_szDefPS, szFunc );
	strcat( m_szDefPS, "\n" );

	m_bUsePxDef = true;

	// Re-initialize the shaders
	int iRes = 0;
	if( iRes = DestroyShaders( ) )
		return iRes;
	if( iRes = CreateShaders ( ) )
		return iRes;

	return iRes;
} // CIntObj::SetPixelDeform

////////////////////////////////////////////////////////////////////////////////
/// \brief Set a vertex shader used after the deformation and estimation of
///        geometric properties.
///
/// \param szFunc Pointer to the null-terminated string containing the code of 
///        the shader (in GLSL for OpenGL applications, and HLSL
///        for Direct3D applications). This is a function with name 
///        fPostVSDeform, that accepts a parameter of type PSDEFORM (D3D-only) 
///        and returns a PSDEFORM. It runs on a vertex processor.
///
///        If szFunc is set to NULL, the user-def vertex shader will not be
///        used. PSDEFORM is a data structure with parameters defined according 
///        to the vertex semantics of the original geometry. See the 
///        documentation of VSSEMANTIC.
///
/// \return  0 on success.
///         -1 if there are not enough memory to create the shader.
////////////////////////////////////////////////////////////////////////////////
int CIntObj::SetPostVertexDeform( char *szFunc )
{
	if( szFunc == NULL )
	{
		// Disable user-defined vertex shader after per-vertex deformation
		m_bUsePostDefVS = false;
		return 0;
	}

	// Create text string containing the shader
	SAFE_DELETE_ARRAY( m_szPostDefVS );
	int iAllocSize = (int) strlen( m_pManager->m_szVSDeform ) + 
					 (int) strlen( szFunc ) + 64 * 80;
	m_szPostDefVS = new char[ iAllocSize ];
	if( m_szPostDefVS == NULL )
		return -1;
	m_szPostDefVS[0] = 0x00;

	strcat( m_szPostDefVS, szFunc );
	strcat( m_szPostDefVS, "\n" );

	m_bUsePostDefVS = true;

	// Re-initialize the shaders
	int iRes = 0;
	if( iRes = DestroyShaders( ) )
		return iRes;
	if( iRes = CreateShaders ( ) )
		return iRes;

	return iRes;
} // CIntObj::SetPostDefVS

////////////////////////////////////////////////////////////////////////////////
/// \brief Enable or disable the vertex-level deformation shader before the
///        estimation of differential geometry properties.
///
/// \param bEnable Whether to enable the shader.
///
/// \return  None.
////////////////////////////////////////////////////////////////////////////////
void CIntObj::TogglePreVertexDeform( bool bEnable )
{
	m_bUsePreDefVS = bEnable;
} // CIntObj::TogglePreVertexDeform

////////////////////////////////////////////////////////////////////////////////
/// \brief Enable or disable the pixel-level deformation shader.
///
/// \param bEnable Whether to enable the pixel-level deformation shader.
///
/// \return  None.
////////////////////////////////////////////////////////////////////////////////
void CIntObj::TogglePixelDeform ( bool bEnable )
{
	m_bUsePxDef = bEnable;
} // CIntObj::TogglePixelDeform

////////////////////////////////////////////////////////////////////////////////
/// \brief Enable or disable the user-defined vertex shader after vertex
///        deformation and estimation of geometric quantities.
///
/// \param bEnable Whether to enable the user-defined vertex shader.
///
/// \return  None.
////////////////////////////////////////////////////////////////////////////////
void CIntObj::TogglePostVertexDeform( bool bEnable )
{
	m_bUsePostDefVS = bEnable;
} // CIntObj::TogglePostVertexDeform

////////////////////////////////////////////////////////////////////////////////
/// \brief Enable or disable the rendering of this object for encoding.
///
/// \param bVisible Whether to enable the rendering of the object.
///
/// \return  None.
////////////////////////////////////////////////////////////////////////////////
void CIntObj::ToggleVisible( bool bVisible )
{
	m_bVisible = bVisible;
} // CIntObj::ToggleVisible

////////////////////////////////////////////////////////////////////////////////
/// \brief Set the callback function used during the rendering pass for 
///        updating the vertex attributes of the geometry.
///
/// The callback function shouldn't set the render state related to polygon 
/// rasterization mode (points, lines, or solid), i.e., D3DFILLMODE in D3D 
/// and GL_POLYGON_MODE in OpenGL. Also, shaders should not be set inside the 
/// callback function.
///
/// \param fUpdate Pointer to the function that updates geometry.
///
/// \return  None.
////////////////////////////////////////////////////////////////////////////////
void CIntObj::SetUpdateCallback( void fUpdate( CIntObj*, void* ) )
{
	m_fUpdate = fUpdate;
} // CIntObj::SetUpdateCallback

////////////////////////////////////////////////////////////////////////////////
/// \brief Set the callback functions used for rendering indexed and 
///        non-indexed geometry.
///
/// Shaders should not be set inside the callback function.
///
/// \param fIdx Pointer to the function that renders indexed geometry. It
///        should contain as parameters a pointer to the CIntObj object,
///        followed by an integer containing the number of the render pass,
///        and a pointer to the user-defined data.
/// \param fNonIdx Pointer to the function that renders non-indexed geometry.
///        It should contain as parameters a pointer to the CIntObj object,
///        followed by a pointer to the user-defined data.
///
/// \return  None.
////////////////////////////////////////////////////////////////////////////////
void CIntObj::SetRenderCallback( void fIdx   ( CIntObj*, int, void* ), 
								 void fNonIdx( CIntObj*, void* ) )
{
	m_fRenderIdx    = fIdx;
	m_fRenderNonIdx = fNonIdx;
} // CIntObj::SetRenderCallback

#if defined(D3D9)

////////////////////////////////////////////////////////////////////////////////
/// \brief Initialize device dependent objects that are not lost on resets.
///        This is a D3D-only function.
///
/// The device has been created. Resources that are not lost on reset are 
/// created here -- resources in \c D3DPOOL_MANAGED,
/// \c D3DPOOL_SCRATCH, or \c D3DPOOL_SYSTEMMEM. Image surfaces created via
/// CreateImageSurface() are never lost and are created here.  Vertex
/// shaders and pixel shaders are also created here as they are not
/// lost on rest. Paired with CIntObj::DeleteDeviceObjects().
///
/// \return \c S_OK on success.
////////////////////////////////////////////////////////////////////////////////
HRESULT CIntObj::InitDeviceObjects( )
{
	HRESULT hrResult = S_OK;

	if( m_bInitDOCalled )
		return hrResult;

	m_pd3dDevice = m_pManager->m_pd3dDevice;

	if( CreateShaders( ) )
		return E_FAIL;

	if( CreateNRTs( ) )
		return E_FAIL;

	m_bInitDOCalled   = true;
	m_bDeleteDOCalled = false;

	return hrResult;
} // CIntObj::InitDeviceObjects

////////////////////////////////////////////////////////////////////////////////
/// \brief Restore device dependent objects that get lost after a reset.
///        This is a D3D-only function.
///
/// The device exists, but may have just been Reset. Resources in
/// \c D3DPOOL_DEFAULT and any other device state that persists during
/// rendering should be set here. Render states, matrices, textures,
/// etc., that don't change during rendering are set once here.
/// Paired with CIntObj::InvalidateDeviceObjects().
///
/// \return \c S_OK on success.
////////////////////////////////////////////////////////////////////////////////
HRESULT CIntObj::RestoreDeviceObjects( )
{
	HRESULT hrResult = S_OK;

	if( m_bRestoreDOCalled )
		return hrResult;

	if( CreateRTs( ) )
		return E_FAIL;

	if( m_pVtDefFx )
		m_pVtDefFx->OnResetDevice( );

	for( int iCt = 0; iCt < MAX_RENDER_PASSES; iCt++ )
	{
		if( m_pEncodeIFx[iCt] )
			m_pEncodeIFx[iCt]->OnResetDevice( );
	}

	if(	m_pEncodeNIFx )
		m_pEncodeNIFx->OnResetDevice( );

	if( m_pRenderFx )
		m_pRenderFx->OnResetDevice( );

	m_bRestoreDOCalled    = true;
	m_bInvalidateDOCalled = false;

	return hrResult;
} // CIntObj::RestoreDeviceObjects

////////////////////////////////////////////////////////////////////////////////
/// \brief Invalidate device objects. This is a D3D-only function.
///
/// Invalidate device objects initialized by CIntObj::RestoreDeviceObjects().
/// Paired with CIntObj::RestoreDeviceObjects().
///
/// \return \c S_OK on success.
////////////////////////////////////////////////////////////////////////////////
HRESULT CIntObj::InvalidateDeviceObjects( )
{
	HRESULT hrResult = S_OK;

	if( m_bInvalidateDOCalled )
		return hrResult;

	if( DestroyRTs( ) )
		return E_FAIL;

	if( m_pVtDefFx )
		m_pVtDefFx->OnLostDevice( );

	for( int iCt = 0; iCt < MAX_RENDER_PASSES; iCt++ )
	{
		if( m_pEncodeIFx[iCt] )
			m_pEncodeIFx[iCt]->OnLostDevice( );
	}

	if(	m_pEncodeNIFx )
		m_pEncodeNIFx->OnLostDevice( );

	if( m_pRenderFx )
		m_pRenderFx->OnLostDevice( );

	m_bInvalidateDOCalled = true;
	m_bRestoreDOCalled    = false;

	return hrResult;
} // CIntObj::InvalidateDeviceObjects

////////////////////////////////////////////////////////////////////////////////
/// \brief Delete device objects. This is a D3D-only function.
///
/// Called when the manager is exiting, or the device is being changed,
/// this function deletes any device dependent objects.
/// Paired with CIntObj::InitDeviceObjects().
///
/// \return \c S_OK on success.
////////////////////////////////////////////////////////////////////////////////
HRESULT CIntObj::DeleteDeviceObjects( )
{
	HRESULT hrResult = S_OK;

	if( m_bDeleteDOCalled )
		return hrResult;

	if( DestroyShaders( ) )
		return E_FAIL;

	if( DestroyNRTs( ) )
		return E_FAIL;

	m_bDeleteDOCalled = true;
	m_bInitDOCalled   = false;

	return hrResult;
} // CIntObj::DeleteDeviceObjects

#endif

////////////////////////////////////////////////////////////////////////////////
/// \brief Create the render target textures for storing geometry attributes.
///
/// \return 0 on success.
///         -1 on error.
////////////////////////////////////////////////////////////////////////////////
int CIntObj::CreateRTs( )
{
#if defined(D3D9)
	HRESULT hrResult = S_OK;

	// Set up data for quad rendering (vertex maps)
    m_pScreenQuadVt[0].p = D3DXVECTOR4( -0.5f, -0.5f, 0.0f, 1.0f );
    m_pScreenQuadVt[1].p = D3DXVECTOR4( m_iVtAttTexSize - 0.5f, -0.5f, 
									    0.0f, 1.0f );
	m_pScreenQuadVt[2].p = D3DXVECTOR4( -0.5f, m_iVtAttTexSize - 0.5f, 
									    0.0f, 1.0f );
    m_pScreenQuadVt[3].p = D3DXVECTOR4( m_iVtAttTexSize - 0.5f, 
									    m_iVtAttTexSize - 0.5f, 0.0f, 1.0f );

    m_pScreenQuadVt[0].t = D3DXVECTOR2( 0.0f, 0.0f );
    m_pScreenQuadVt[1].t = D3DXVECTOR2( 1.0f, 0.0f );
    m_pScreenQuadVt[2].t = D3DXVECTOR2( 0.0f, 1.0f );
    m_pScreenQuadVt[3].t = D3DXVECTOR2( 1.0f, 1.0f );

	// Set up data for quad rendering (face maps)
    m_pScreenQuadFace[0].p = D3DXVECTOR4( -0.5f, -0.5f, 0.0f, 1.0f );
    m_pScreenQuadFace[1].p = D3DXVECTOR4( m_iFaceIdxTexSize - 0.5f, -0.5f, 
									      0.0f, 1.0f );
	m_pScreenQuadFace[2].p = D3DXVECTOR4( -0.5f, m_iFaceIdxTexSize - 0.5f, 
									      0.0f, 1.0f );
    m_pScreenQuadFace[3].p = D3DXVECTOR4( m_iFaceIdxTexSize - 0.5f, 
									      m_iFaceIdxTexSize - 0.5f, 
										  0.0f, 1.0f );

    m_pScreenQuadFace[0].t = D3DXVECTOR2( 0.0f, 0.0f );
    m_pScreenQuadFace[1].t = D3DXVECTOR2( 1.0f, 0.0f );
    m_pScreenQuadFace[2].t = D3DXVECTOR2( 0.0f, 1.0f );
    m_pScreenQuadFace[3].t = D3DXVECTOR2( 1.0f, 1.0f );

	// Create 'vertex result textures'
	for( int iCt = 0; iCt < m_pManager->m_iMaxVtCacheTex; iCt++ )
	{
		int iSize = m_iFaceIdxTexSize;
/*		int iSize = m_iVtAttTexSize;

		// When computing TBN frames, the first three RTs are used
		// to store face attributes
		if( m_pManager->m_bAttTBN && iCt < 3 )
			iSize = m_iFaceIdxTexSize;

		// When computing normals, the first RT is used to store
		// face attributes
		if( m_pManager->m_bAttNormal && iCt < 1 )
			iSize = m_iFaceIdxTexSize;
*/
		hrResult = m_pd3dDevice->CreateTexture( iSize, iSize,
												1, D3DUSAGE_RENDERTARGET,
												D3DFMT_A32B32G32R32F,
												D3DPOOL_DEFAULT,
												&m_pd3dVtCacheTex[iCt], NULL );
		if( FAILED( hrResult ) )
			return -1;
	}

	// Create 'vertex position' texture
	if( m_pVtPos )
	{
		// Create render target texture
		hrResult = m_pd3dDevice->CreateTexture( m_iVtAttTexSize, 
												m_iVtAttTexSize,
												1, D3DUSAGE_RENDERTARGET, 
												D3DFMT_A32B32G32R32F,
												D3DPOOL_DEFAULT, 
												&m_pd3dVtPosTex, NULL );
		if( FAILED( hrResult ) )
			return -1;

		// In order to fill the vertex position texture, we need to
		// create a temporary texture in system memory, which is uploaded
		// to video memory by using IDirect3DDevice9::UpdateTexture

		// Create temporary system memory texture
		LPDIRECT3DTEXTURE9 pd3dTempTex;
		hrResult = m_pd3dDevice->CreateTexture( m_iVtAttTexSize, 
												m_iVtAttTexSize,
												1, 0, 
												D3DFMT_A32B32G32R32F,
												D3DPOOL_SYSTEMMEM, 
												&pd3dTempTex, NULL );
		if( FAILED( hrResult ) )
			return -1;

		D3DLOCKED_RECT rcLockedRect;
		hrResult = pd3dTempTex->LockRect( 0, &rcLockedRect, NULL, 0 );
		if( SUCCEEDED( hrResult ) )
		{
			int iVert = 0;
			for( int iY = 0; iY < m_iVtAttTexSize; iY++ )
			{
				for( int iX = 0; iX < m_iVtAttTexSize; iX++ )
				{
					BYTE *pLockedRectB = (BYTE*) rcLockedRect.pBits;
					float *vTexel = (float*) 
						( pLockedRectB + (iX*4*sizeof(float) + 
						iY*rcLockedRect.Pitch ) );

					if( iVert >= m_iNumVt )
					{
						// Null texels
						vTexel[0] = vTexel[1] = vTexel[2] = vTexel[3] = -1.0f;
					}
					else
					{
						// Get vertex position for this texel
						vTexel[0] = m_pVtPos[iVert*3+0];
						vTexel[1] = m_pVtPos[iVert*3+1];
						vTexel[2] = m_pVtPos[iVert*3+2];
						vTexel[3] = 0.0f;
					}

					// Go to next texel
					iVert++;
				}
			}
			pd3dTempTex->UnlockRect( 0 );
		}

		// Copy from system memory to video memory
		hrResult = m_pd3dDevice->UpdateTexture( pd3dTempTex, 
											    m_pd3dVtPosTex );
		if( FAILED( hrResult ) )
			return -1;

		SAFE_RELEASE( pd3dTempTex );
	}

	// Create 'vertex texture coordinates' texture
	if( m_pVtTexCoord )
	{
		// Create render target texture
		hrResult = m_pd3dDevice->CreateTexture( m_iVtAttTexSize, 
												m_iVtAttTexSize, 
												1, D3DUSAGE_RENDERTARGET, 
												D3DFMT_A32B32G32R32F,
												D3DPOOL_DEFAULT, 
												&m_pd3dVtTexCoordTex, NULL );
		if( FAILED( hrResult ) )
			return hrResult;

		// In order to fill the vertex texcoord texture, we need to
		// create a temporary texture in system memory, which is uploaded
		// to video memory by using IDirect3DDevice9::UpdateTexture

		// Create temporary system memory texture
		LPDIRECT3DTEXTURE9 pd3dTempTex;
		hrResult = m_pd3dDevice->CreateTexture( m_iVtAttTexSize, 
												m_iVtAttTexSize,
												1, 0, 
												D3DFMT_A32B32G32R32F,
												D3DPOOL_SYSTEMMEM, 
												&pd3dTempTex, NULL );
		if( FAILED( hrResult ) )
			return -1;

		// Fill texture
		D3DLOCKED_RECT rcLockedRect;
		hrResult = pd3dTempTex->LockRect( 0, &rcLockedRect, NULL, 0 );
		if( SUCCEEDED( hrResult ) )
		{
			int iVert = 0;
			for( int iY = 0; iY < m_iVtAttTexSize; iY++ )
			{
				for( int iX = 0; iX < m_iVtAttTexSize; iX++ )
				{
					BYTE *pLockedRectB = (BYTE*) rcLockedRect.pBits;
					float *vTexel = (float*) 
						( pLockedRectB + (iX*4*sizeof(float) + 
						iY*rcLockedRect.Pitch ) );

					if( iVert >= m_iNumVt )
					{
						// Null texels
						vTexel[0] = vTexel[1] = vTexel[2] = vTexel[3] = -1.0f;
					}
					else
					{
						// Get vertex tex coord for this texel
						vTexel[0] = m_pVtTexCoord[iVert*2+0];
						vTexel[1] = m_pVtTexCoord[iVert*2+1];
						vTexel[2] = 0.0f;
						vTexel[3] = 0.0f;
					}

					// Go to next texel
					iVert++;
				}
			}
			pd3dTempTex->UnlockRect( 0 );
		}

		// Copy from system memory to video memory
		hrResult = m_pd3dDevice->UpdateTexture( pd3dTempTex, 
											    m_pd3dVtTexCoordTex );
		if( FAILED( hrResult ) )
			return -1;

		SAFE_RELEASE( pd3dTempTex );
	}

	// Create 'vertex normal' render target texture
	hrResult = m_pd3dDevice->CreateTexture( m_iVtAttTexSize, 
											m_iVtAttTexSize, 
											1, D3DUSAGE_RENDERTARGET, 
											D3DFMT_A32B32G32R32F,
											D3DPOOL_DEFAULT, 
											&m_pd3dVtNormalTex, NULL );
	if( FAILED( hrResult ) )
		return hrResult;

	if( m_pVtNormal )
	{
		// In order to fill the vertex normal texture, we need to
		// create a temporary texture in system memory, which is uploaded
		// to video memory by using IDirect3DDevice9::UpdateTexture

		// Create temporary system memory texture
		LPDIRECT3DTEXTURE9 pd3dTempTex;
		hrResult = m_pd3dDevice->CreateTexture( m_iVtAttTexSize, 
												m_iVtAttTexSize,
												1, 0, 
												D3DFMT_A32B32G32R32F,
												D3DPOOL_SYSTEMMEM, 
												&pd3dTempTex, NULL );
		if( FAILED( hrResult ) )
			return -1;

		// Fill texture
		D3DLOCKED_RECT rcLockedRect;
		hrResult = pd3dTempTex->LockRect( 0, &rcLockedRect, NULL, 0 );
		if( SUCCEEDED( hrResult ) )
		{
			int iVert = 0;
			for( int iY = 0; iY < m_iVtAttTexSize; iY++ )
			{
				for( int iX = 0; iX < m_iVtAttTexSize; iX++ )
				{
					BYTE *pLockedRectB = (BYTE*) rcLockedRect.pBits;
					float *vTexel = (float*) 
						( pLockedRectB + (iX*4*sizeof(float) + 
						iY*rcLockedRect.Pitch ) );

					if( iVert >= m_iNumVt )
					{
						// Null texels
						vTexel[0] = vTexel[1] = vTexel[2] = vTexel[3] = -1.0f;
					}
					else
					{
						// Get vertex normal for this texel
						vTexel[0] = m_pVtNormal[iVert*3+0];
						vTexel[1] = m_pVtNormal[iVert*3+1];
						vTexel[2] = m_pVtNormal[iVert*3+2];
						vTexel[3] = 0.0f;
					}

					// Go to next texel
					iVert++;
				}
			}
			pd3dTempTex->UnlockRect( 0 );
		}

		// Copy from system memory to video memory
		hrResult = m_pd3dDevice->UpdateTexture( pd3dTempTex, 
												m_pd3dVtNormalTex );
		if( FAILED( hrResult ) )
			return -1;

		SAFE_RELEASE( pd3dTempTex );
	}

	if( m_pManager->m_bAttTBN )
	{
		// Create 'tangent + handedness' render target texture
		hrResult = m_pd3dDevice->CreateTexture( m_iVtAttTexSize, 
												m_iVtAttTexSize, 
												1, D3DUSAGE_RENDERTARGET, 
												D3DFMT_A32B32G32R32F,
												D3DPOOL_DEFAULT, 
												&m_pd3dVtTanHTex, NULL );
		if( FAILED( hrResult ) )
			return hrResult;
	}

	if( m_pManager->m_bAttTensor2 || m_pManager->m_bAttTensor3 ||
		m_pManager->m_bAttCurv )
	{
		// Create 'tensor2' render target texture
		hrResult = m_pd3dDevice->CreateTexture( m_iVtAttTexSize, 
												m_iVtAttTexSize, 
												1, D3DUSAGE_RENDERTARGET, 
												D3DFMT_A32B32G32R32F,
												D3DPOOL_DEFAULT, 
												&m_pd3dVtTensor2Tex, NULL );
		if( FAILED( hrResult ) )
			return hrResult;

		// Create 'p1k1' render target texture
		hrResult = m_pd3dDevice->CreateTexture( m_iVtAttTexSize, 
												m_iVtAttTexSize, 
												1, D3DUSAGE_RENDERTARGET, 
												D3DFMT_A32B32G32R32F,
												D3DPOOL_DEFAULT, 
												&m_pd3dVtP1K1Tex, NULL );
		if( FAILED( hrResult ) )
			return hrResult;

		// Create 'p2k2' render target texture
		hrResult = m_pd3dDevice->CreateTexture( m_iVtAttTexSize, 
												m_iVtAttTexSize, 
												1, D3DUSAGE_RENDERTARGET, 
												D3DFMT_A32B32G32R32F,
												D3DPOOL_DEFAULT, 
												&m_pd3dVtP2K2Tex, NULL );
		if( FAILED( hrResult ) )
			return hrResult;

		if( m_pManager->m_bAttTensor3 )
		{
			// Create 'tensor3' render target texture
			hrResult = m_pd3dDevice->CreateTexture( m_iVtAttTexSize, 
													m_iVtAttTexSize, 
													1, D3DUSAGE_RENDERTARGET, 
													D3DFMT_A32B32G32R32F,
													D3DPOOL_DEFAULT, 
													&m_pd3dVtTensor3Tex, 
													NULL );
			if( FAILED( hrResult ) )
				return hrResult;
		}
	}
#elif defined(OGL2)
	// Create FBO and bind it
    glGenFramebuffersEXT( 1, &m_uiVtTexFBO ); 
	glBindFramebufferEXT( GL_FRAMEBUFFER_EXT, m_uiVtTexFBO );

	// Create 'vertex result textures'
	glGenTextures( m_pManager->m_iMaxVtCacheTex, m_uiVtCacheTex );
	for( int iCt = 0; iCt < m_pManager->m_iMaxVtCacheTex; iCt++ )
	{
		glBindTexture  ( GL_TEXTURE_2D, m_uiVtCacheTex[iCt] );
		glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST );
		glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST );
		glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP );
		glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP );
		glTexEnvi      ( GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE );

		int iSize = m_iVtAttTexSize;

		// When computing TBN frames, the first three RTs are used
		// to store face attributes
		if( m_pManager->m_bAttTBN && iCt < 3 )
			iSize = m_iFaceIdxTexSize;

		// When computing normals, the first RT is used
		// to store face attributes
		if( m_pManager->m_bAttNormal && iCt < 1 )
			iSize = m_iFaceIdxTexSize;

		glTexImage2D   ( GL_TEXTURE_2D, 0, GL_RGBA32F_ARB, 
						 iSize, iSize, 0, GL_RGBA, GL_FLOAT, NULL );
	}

	// Create textures
	glGenTextures( 1, &m_uiVtPosTexRT      );
	glGenTextures( 1, &m_uiVtTexCoordTexRT );
	glGenTextures( 1, &m_uiVtNormalTexRT   );

	// Set up 'vertex position' texture
	glBindTexture  ( GL_TEXTURE_2D, m_uiVtPosTexRT );
	glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST );
	glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST );
	glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP );
	glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP );
	glTexEnvi      ( GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE );
	glTexImage2D   ( GL_TEXTURE_2D, 0, GL_RGB32F_ARB, 
					 m_iVtAttTexSize, m_iVtAttTexSize, 0, GL_RGB, GL_FLOAT, 
					 m_pVtPos );

	// Set up 'vertex texture coordinates' texture
	glBindTexture  ( GL_TEXTURE_2D, m_uiVtTexCoordTexRT );
	glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST );
	glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST );
	glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP );
	glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP );
	glTexEnvi      ( GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE );
	glTexImage2D   ( GL_TEXTURE_2D, 0, GL_RGB32F_ARB, 
					 m_iVtAttTexSize, m_iVtAttTexSize, 0, GL_LUMINANCE_ALPHA, 
					 GL_FLOAT, m_pVtTexCoord );

	// Set up 'vertex normal' texture
	glBindTexture  ( GL_TEXTURE_2D, m_uiVtNormalTexRT );
	glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST );
	glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST );
	glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP );
	glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP );
	glTexEnvi      ( GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE );
	glTexImage2D   ( GL_TEXTURE_2D, 0, GL_RGB32F_ARB, 
					 m_iVtAttTexSize, m_iVtAttTexSize, 0, GL_RGB, GL_FLOAT, 
					 m_pVtNormal );

	if( m_pManager->m_bAttTBN )
	{
		// Set up 'tangent + handedness' texture
		glBindTexture  ( GL_TEXTURE_2D, m_uiVtTanHTexRT );
		glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST );
		glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST );
		glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP );
		glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP );
		glTexEnvi      ( GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE );
		glTexImage2D   ( GL_TEXTURE_2D, 0, GL_RGB32F_ARB, 
						 m_iVtAttTexSize, m_iVtAttTexSize, 0, GL_RGB, GL_FLOAT, 
						 NULL );
	}

	if( m_pManager->m_bAttTensor2 || m_pManager->m_bAttTensor3 ||
		m_pManager->m_bAttCurv )
	{
		// Set up 'tensor2' texture
		glBindTexture  ( GL_TEXTURE_2D, m_uiVtTensor2TexRT );
		glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST );
		glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST );
		glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP );
		glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP );
		glTexEnvi      ( GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE );
		glTexImage2D   ( GL_TEXTURE_2D, 0, GL_RGB32F_ARB, 
						 m_iVtAttTexSize, m_iVtAttTexSize, 0, GL_RGB, GL_FLOAT, 
						 NULL );

		// Set up 'p1k1' texture
		glBindTexture  ( GL_TEXTURE_2D, m_uiVtP1K1TexRT );
		glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST );
		glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST );
		glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP );
		glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP );
		glTexEnvi      ( GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE );
		glTexImage2D   ( GL_TEXTURE_2D, 0, GL_RGBA32F_ARB, 
						 m_iVtAttTexSize, m_iVtAttTexSize, 0, GL_RGBA, GL_FLOAT, 
						 NULL );

		// Set up 'p2k2' texture
		glBindTexture  ( GL_TEXTURE_2D, m_uiVtP1K1TexRT );
		glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST );
		glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST );
		glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP );
		glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP );
		glTexEnvi      ( GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE );
		glTexImage2D   ( GL_TEXTURE_2D, 0, GL_RGBA32F_ARB, 
						 m_iVtAttTexSize, m_iVtAttTexSize, 0, GL_RGBA, GL_FLOAT, 
						 NULL );

		if( m_pManager->m_bAttTensor3 )
		{
			// Set up 'tensor3' texture
			glBindTexture  ( GL_TEXTURE_2D, m_uiVtTensor3TexRT );
			glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST );
			glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST );
			glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP );
			glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP );
			glTexEnvi      ( GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE );
			glTexImage2D   ( GL_TEXTURE_2D, 0, GL_RGB32F_ARB, 
							 m_iVtAttTexSize, m_iVtAttTexSize, 0, GL_RGB, 
							 GL_FLOAT, NULL );
		}
	}

	// Re-enable rendering to the back buffer
    glBindFramebufferEXT( GL_FRAMEBUFFER_EXT, 0 );
#endif

	return 0;
} // CIntObj::CreateRTs

////////////////////////////////////////////////////////////////////////////////
/// \brief Release the render target textures created with 
///        CIntObj::CreateRTs.
///
/// \return 0 on success.
///         -1 on error.
////////////////////////////////////////////////////////////////////////////////
int CIntObj::DestroyRTs( )
{
#if defined(D3D9)
	// Release render textures with cached results of vertex-level computations
	for( int iCt = 0; iCt < m_pManager->m_iMaxVtCacheTex; iCt++ )
		SAFE_RELEASE( m_pd3dVtCacheTex[iCt] );

	// Release render textures with vertex attributes
	SAFE_RELEASE( m_pd3dVtPosTex      );
	SAFE_RELEASE( m_pd3dVtTexCoordTex );
	SAFE_RELEASE( m_pd3dVtNormalTex   );
	SAFE_RELEASE( m_pd3dVtTanHTex     );
	SAFE_RELEASE( m_pd3dVtTensor2Tex  );
	SAFE_RELEASE( m_pd3dVtTensor3Tex  );
	SAFE_RELEASE( m_pd3dVtP1K1Tex     );
	SAFE_RELEASE( m_pd3dVtP2K2Tex     );
#elif defined(OGL2)
	// Release render textures with cached results of vertex-level computations
	glDeleteTextures( m_pManager->m_iMaxVtCacheTex, m_uiVtCacheTex );
	for( int iCt = 0; iCt < m_pManager->m_iMaxVtCacheTex; iCt++ )
		m_uiVtCacheTex[iCt] = 0;

	// Release render textures with vertex attributes
	if( m_uiVtPosTexRT      ) 
	{
		glDeleteTextures( 1, &m_uiVtPosTexRT      );
		m_uiVtPosTexRT = 0;
	}
	if( m_uiVtTexCoordTexRT ) 
	{
		glDeleteTextures( 1, &m_uiVtTexCoordTexRT );
		m_uiVtTexCoordTexRT = 0;
	}
	if( m_uiVtNormalTexRT   ) 
	{
		glDeleteTextures( 1, &m_uiVtNormalTexRT   );
		m_uiVtNormalTexRT = 0;
	}
	if( m_uiVtTanHTexRT     ) 
	{
		glDeleteTextures( 1, &m_uiVtTanHTexRT     );
		m_uiVtTanHTexRT = 0;
	}
	if( m_uiVtTensor2TexRT  ) 
	{
		glDeleteTextures( 1, &m_uiVtTensor2TexRT  );
		m_uiVtTensor2TexRT = 0;
	}
	if( m_uiVtTensor3TexRT  ) 
	{
		glDeleteTextures( 1, &m_uiVtTensor3TexRT  );
		m_uiVtTensor3TexRT = 0;
	}
	if( m_uiVtP1K1TexRT     ) 
	{
		glDeleteTextures( 1, &m_uiVtP1K1TexRT     );
		m_uiVtP1K1TexRT = 0;
	}
	if( m_uiVtP2K2TexRT     ) 
	{
		glDeleteTextures( 1, &m_uiVtP2K2TexRT     );
		m_uiVtP2K2TexRT = 0;
	}

	// Release the FBO
	if( m_uiVtTexFBO ) 
	{
		glDeleteFramebuffersEXT( 1, &m_uiVtTexFBO );
		m_uiVtTexFBO = 0;
	}
#endif

	return 0;
} // CIntObj::DestroyRTs

////////////////////////////////////////////////////////////////////////////////
/// \brief Create the non-render-target textures used as data structures for 
///        computing attributes of interaction.
///
/// \return 0 on success.
///         -1 on error.
////////////////////////////////////////////////////////////////////////////////
int CIntObj::CreateNRTs( )
{
	bool bAdjRead = false;

	vector<int> *pAdjFacesSTL = NULL;
	int          iNumAdjFace  = 0;
	int         *pAdjFaces    = NULL;
	vector<int> *pAdjVtsSTL   = NULL;
	int          iNumAdjVt    = 0;
	int         *pAdjVts      = NULL;
	int			*pAdjAddress  = NULL;
	
	m_iAdjFaceTexSize = 0;
	m_iAdjVtTexSize   = 0;

	// Read adjacency data from disk
	if( m_szAdjReadFile )
	{
		FILE *fHandle = NULL;

		if( ( fHandle = fopen( m_szAdjReadFile, "rb" ) ) == NULL )
			return -1;

		// pAdjAddress (int) number of elements = m_iNumVt * 4
		pAdjAddress = new int[ m_iNumVt * 4 ];
		if( pAdjAddress == NULL )
			return -1;
		fread( pAdjAddress, sizeof(int), m_iNumVt * 4, fHandle );

		// iNumAdjVt
		fscanf( fHandle, "%d", &iNumAdjVt );

		// pAdjVts (int) number of elements = iNumAdjVt
		pAdjVts = new int[ iNumAdjVt ];
		if( pAdjVts == NULL )
			return -1;
		fread( pAdjVts, sizeof(int), iNumAdjVt, fHandle );

		// iNumAdjFace
		fscanf( fHandle, "%d", &iNumAdjFace );

		// pAdjFaces (int) iNumAdjFace
		pAdjFaces = new int[ iNumAdjFace ];
		if( pAdjFaces == NULL )
			return -1;
		fread( pAdjFaces, sizeof(int), iNumAdjFace, fHandle );

		fclose( fHandle );

		bAdjRead = true;
	}

	// Create auxiliary data structures used for creating the textures 
	// with adjacency data
	if( m_pFaceIdx == NULL )
		return 0;

	if( !bAdjRead )
	{
		// Create 'adjacent faces' temporary STL set
		pAdjFacesSTL = new vector<int>[ m_iNumVt ];
		if( pAdjFacesSTL == NULL )
			return -1;

		for( int iFace = 0; iFace < m_iNumFace; iFace++ )
		{
			int iFaceIdx = iFace*3;
			pAdjFacesSTL[m_pFaceIdx[iFaceIdx+0]].push_back( iFace );
			pAdjFacesSTL[m_pFaceIdx[iFaceIdx+1]].push_back( iFace );
			pAdjFacesSTL[m_pFaceIdx[iFaceIdx+2]].push_back( iFace );
		}

		// Determine size of 'adjacent faces' texture
		for( int iVt = 0; iVt < m_iNumVt; iVt++ )
			iNumAdjFace += (int) pAdjFacesSTL[iVt].size( );
	}

	// Compute minimum size required by the 'adjacent faces' texture
	int iPow = 1;
	m_iAdjFaceTexSize = 0;
	while( 1 )
	{
		m_iAdjFaceTexSize = 1 << iPow++;
		if( m_iAdjFaceTexSize * m_iAdjFaceTexSize >= iNumAdjFace )
			break;
	}

	if( !bAdjRead )
	{
		// Now create an array of adjacent faces in a format which 
		// is easier to assign to a texture further
		pAdjFaces = new int[ iNumAdjFace ];
		if( pAdjFaces == NULL )
			return -1;
		int iCt = 0;
		for( int iVt = 0; iVt < m_iNumVt; iVt++ )
		{
			vector<int>::iterator stlIt = pAdjFacesSTL[iVt].begin( );
			while( stlIt != pAdjFacesSTL[iVt].end( ) )
			{
				pAdjFaces[iCt] = (*stlIt);

				iCt++;
				stlIt++;
			}
		}

		// Create 'adjacent vertices' temporary STL set
		pAdjVtsSTL = new vector<int>[ m_iNumVt ];
		if( pAdjVtsSTL == NULL )
			return -1;

		for( int iFace = 0; iFace < m_iNumFace; iFace++ )
		{
			int iIdx, iAdd;
			vector<int> vCur;
			int iFaceIdx = iFace*3;

			for( int iVt = 0; iVt < 3; iVt++ )
			{
				iIdx = m_pFaceIdx[iFaceIdx+iVt];
				vCur = pAdjVtsSTL[iIdx];
				iAdd = m_pFaceIdx[iFaceIdx+((iVt+1)%3)];
				if( find( vCur.begin( ), vCur.end( ), iAdd ) == vCur.end( ) )
					pAdjVtsSTL[iIdx].push_back( iAdd );
				iAdd = m_pFaceIdx[iFaceIdx+((iVt+2)%3)];
				if( find( vCur.begin( ), vCur.end( ), iAdd ) == vCur.end( ) )
					pAdjVtsSTL[iIdx].push_back( iAdd );
			}
		}

		// Determine size of 'adjacent vertices' texture
		for( int iVt = 0; iVt < m_iNumVt; iVt++ )
			iNumAdjVt += (int) pAdjVtsSTL[iVt].size( );
	}

	// Compute minimum size required by the 'adjacent vertices' texture
	iPow = 1;
	m_iAdjVtTexSize = 0;
	while( 1 )
	{
		m_iAdjVtTexSize = 1 << iPow++;
		if( m_iAdjVtTexSize * m_iAdjVtTexSize >= iNumAdjVt )
			break;
	}

	if( !bAdjRead )
	{
		// Now create an array of adjacent vertices in a format which 
		// is easier to assign to a texture further
		pAdjVts = new int[ iNumAdjVt ];
		if( pAdjVts == NULL )
			return -1;
		int iCt = 0;
		for( int iVt = 0; iVt < m_iNumVt; iVt++ )
		{
			vector<int>::iterator stlIt = pAdjVtsSTL[iVt].begin( );
			while( stlIt != pAdjVtsSTL[iVt].end( ) )
			{
				pAdjVts[iCt] = (*stlIt);

				iCt++;
				stlIt++;
			}
		}

		// Create temporary array of addresses of the beginning of the
		// array of adjacent faces and vertices.
		// Each group of 4 elements corresponds to a vertex and contains the 
		// following values:
		// 1) Address to the 1st adjacent face in the face adjacency texture
		// 2) Number of adjacent faces
		// 3) Address to the 1st adjacent vertex in the vertex adjacency 
		//    texture
		// 4) Number of adjacent vertices
		pAdjAddress = new int[ m_iNumVt * 4 ];
		if( pAdjAddress == NULL )
			return -1;
		memset( pAdjAddress, 0, sizeof( int ) * m_iNumVt * 4 );

		int iCtVt = 0, iCtFace = 0;
		for( int iVt = 0; iVt < m_iNumVt; iVt++ )
		{
			pAdjAddress[iVt*4+0] = iCtFace;
			pAdjAddress[iVt*4+1] = (int) pAdjFacesSTL[iVt].size( );
			pAdjAddress[iVt*4+2] = iCtVt;
			pAdjAddress[iVt*4+3] = (int) pAdjVtsSTL[iVt].size( );

			iCtFace += (int) pAdjFacesSTL[iVt].size( );
			iCtVt   += (int) pAdjVtsSTL[iVt].size( );
		}
	}

	// Write adjacency data to disk if a file name was given
	if( m_szAdjWriteFile )
	{
		FILE *fHandle = NULL;

		if( ( fHandle = fopen( m_szAdjWriteFile, "wb" ) ) == NULL )
			return -1;

		// pAdjAddress (int) number of elements = m_iNumVt * 4
		fwrite( pAdjAddress, sizeof(int), m_iNumVt * 4, fHandle );

		// iNumAdjVt
		fprintf( fHandle, "%d", iNumAdjVt );

		// pAdjVts (int) number of elements = iNumAdjVt
		fwrite( pAdjVts, sizeof(int), iNumAdjVt, fHandle );

		// iNumAdjFace
		fprintf( fHandle, "%d", iNumAdjFace );

		// pAdjFaces (int) iNumAdjFace
		fwrite( pAdjFaces, sizeof(int), iNumAdjFace, fHandle );

		fclose( fHandle );
	}

#if defined(D3D9)
	HRESULT hrResult = S_OK;

	// Create 'face indices' texture
	hrResult = m_pd3dDevice->CreateTexture( m_iFaceIdxTexSize, 
											m_iFaceIdxTexSize,
											1, 0, 
											D3DFMT_A32B32G32R32F,
											D3DPOOL_MANAGED, 
											&m_pd3dFaceIdxTex, NULL );
	if( FAILED( hrResult ) )
		return -1;

	D3DLOCKED_RECT rcLockedRect;
	hrResult = m_pd3dFaceIdxTex->LockRect( 0, &rcLockedRect, NULL, 0 );
	if( SUCCEEDED( hrResult ) )
	{
		int iFace = 0;
		for( int iY = 0; iY < m_iFaceIdxTexSize; iY++ )
		{
			for( int iX = 0; iX < m_iFaceIdxTexSize; iX++ )
			{
				BYTE *pLockedRectB = (BYTE*) rcLockedRect.pBits;
				float *vTexel = (float*) 
					( pLockedRectB + (iX*4*sizeof(float) + 
					iY*rcLockedRect.Pitch ) );

				if( iFace >= m_iNumFace )
				{
					// Null texels
					vTexel[0] = vTexel[1] = vTexel[2] = vTexel[3] = -1.0f;
				}
				else
				{
					// Get vertex indices for this texel
					vTexel[0] = (float) m_pFaceIdx[iFace*3+0];
					vTexel[1] = (float) m_pFaceIdx[iFace*3+1];
					vTexel[2] = (float) m_pFaceIdx[iFace*3+2];
					vTexel[3] = 0.0f;
				}

				// Go to next texel
				iFace++;
			}
		}
		m_pd3dFaceIdxTex->UnlockRect( 0 );
	}

	// Create 'adjacent faces' texture
	hrResult = m_pd3dDevice->CreateTexture( m_iAdjFaceTexSize, 
											m_iAdjFaceTexSize,
											1, 0, 
											D3DFMT_R32F,
											D3DPOOL_MANAGED, 
											&m_pd3dAdjFaceTex, NULL );
	if( FAILED( hrResult ) )
		return -1;

	hrResult = m_pd3dAdjFaceTex->LockRect( 0, &rcLockedRect, NULL, 0 );
	if( SUCCEEDED( hrResult ) )
	{
		int iFaceIdx = 0;
		for( int iY = 0; iY < m_iAdjFaceTexSize; iY++ )
		{
			for( int iX = 0; iX < m_iAdjFaceTexSize; iX++ )
			{
				BYTE *pLockedRectB = (BYTE*) rcLockedRect.pBits;
				float *vTexel = (float*) 
					( pLockedRectB + (iX*1*sizeof(float) + 
					iY*rcLockedRect.Pitch ) );

				if( iFaceIdx >= iNumAdjFace )
				{
					// Null texels
					vTexel[0] = -1.0f;
				}
				else
				{
					// Get face index for this texel
					vTexel[0] = (float) pAdjFaces[iFaceIdx];
				}

				// Go to next texel
				iFaceIdx++;
			}
		}
		m_pd3dAdjFaceTex->UnlockRect( 0 );
	}

	// Create 'adjacent vertices' texture
	hrResult = m_pd3dDevice->CreateTexture( m_iAdjVtTexSize, 
											m_iAdjVtTexSize,
											1, 0, 
											D3DFMT_R32F,
											D3DPOOL_MANAGED, 
											&m_pd3dAdjVtTex, NULL );
	if( FAILED( hrResult ) )
		return -1;

	hrResult = m_pd3dAdjVtTex->LockRect( 0, &rcLockedRect, NULL, 0 );
	if( SUCCEEDED( hrResult ) )
	{
		int iVtIdx = 0;
		for( int iY = 0; iY < m_iAdjVtTexSize; iY++ )
		{
			for( int iX = 0; iX < m_iAdjVtTexSize; iX++ )
			{
				BYTE *pLockedRectB = (BYTE*) rcLockedRect.pBits;
				float *vTexel = (float*) 
					( pLockedRectB + (iX*1*sizeof(float) + 
					iY*rcLockedRect.Pitch ) );

				if( iVtIdx >= iNumAdjVt )
				{
					// Null texels
					vTexel[0] = -1.0f;
				}
				else
				{
					// Get face index for this texel
					vTexel[0] = (float) pAdjVts[iVtIdx];
				}

				// Go to next texel
				iVtIdx++;
			}
		}
		m_pd3dAdjVtTex->UnlockRect( 0 );
	}

	// Create 'adjacency addresses' texture
	hrResult = m_pd3dDevice->CreateTexture( m_iVtAttTexSize, 
											m_iVtAttTexSize, 
											1, 0, 
											D3DFMT_A32B32G32R32F,
											D3DPOOL_MANAGED, 
											&m_pd3dAdjAddTex, NULL );
	if( FAILED( hrResult ) )
		return -1;

	hrResult = m_pd3dAdjAddTex->LockRect( 0, &rcLockedRect, NULL, 0 );
	if( SUCCEEDED( hrResult ) )
	{
		int iVtIdx = 0;
		for( int iY = 0; iY < m_iVtAttTexSize; iY++ )
		{
			for( int iX = 0; iX < m_iVtAttTexSize; iX++ )
			{
				BYTE *pLockedRectB = (BYTE*) rcLockedRect.pBits;
				float *vTexel = (float*) 
					( pLockedRectB + (iX*4*sizeof(float) + 
					iY*rcLockedRect.Pitch ) );

				if( iVtIdx >= m_iNumVt )
				{
					// Null texels
					vTexel[0] = vTexel[1] = vTexel[2] = vTexel[3] = -1.0f;
					iX = iY = m_iVtAttTexSize;
				}
				else
				{
					// Get face index for this texel
					vTexel[0] = (float) pAdjAddress[iVtIdx*4+0];
					vTexel[1] = (float) pAdjAddress[iVtIdx*4+1];
					vTexel[2] = (float) pAdjAddress[iVtIdx*4+2];
					vTexel[3] = (float) pAdjAddress[iVtIdx*4+3];
				}

				// Go to next texel
				iVtIdx++;
			}
		}
		m_pd3dAdjAddTex->UnlockRect( 0 );
	}
#elif defined(OGL2)
	// Create 'face indices' texture
	glGenTextures( 1, &m_uiFaceIdxTex );
	glBindTexture  ( GL_TEXTURE_2D, m_uiFaceIdxTex );
	glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST );
	glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST );
	glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP );
	glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP );
	glTexEnvi      ( GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE );
	glTexImage2D   ( GL_TEXTURE_2D, 0, GL_RGB32F_ARB, 
					 m_iFaceIdxTexSize, m_iFaceIdxTexSize, 0, GL_RGB, GL_INT, 
					 m_pFaceIdx );

	// Create 'adjacent faces' texture
	glGenTextures( 1, &m_uiAdjFaceTex );
	glBindTexture  ( GL_TEXTURE_2D, m_uiAdjFaceTex );
	glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST );
	glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST );
	glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP );
	glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP );
	glTexEnvi      ( GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE );
	glTexImage2D   ( GL_TEXTURE_2D, 0, GL_ALPHA32F_ARB,
					 m_iAdjFaceTexSize, m_iAdjFaceTexSize, 0, GL_RED, GL_FLOAT, 
					 pAdjFaces );

	// Create 'adjacent vertices' texture
	glGenTextures( 1, &m_uiAdjVtTex );
	glBindTexture  ( GL_TEXTURE_2D, m_uiAdjVtTex );
	glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST );
	glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST );
	glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP );
	glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP );
	glTexEnvi      ( GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE );
	glTexImage2D   ( GL_TEXTURE_2D, 0, GL_ALPHA32F_ARB, 
					 m_iAdjVtTexSize, m_iAdjVtTexSize, 0, GL_RED, GL_FLOAT, 
					 pAdjVts );

	// Create 'adjacency addresses' texture
	glGenTextures( 1, &m_uiAdjAddTex );
	glBindTexture  ( GL_TEXTURE_2D, m_uiAdjAddTex );
	glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST );
	glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST );
	glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP );
	glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP );
	glTexEnvi      ( GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE );
	glTexImage2D   ( GL_TEXTURE_2D, 0, GL_RGBA32F_ARB, 
					 m_iVtAttTexSize, m_iVtAttTexSize, 0, GL_RGBA, GL_INT, 
					 pAdjAddress );
#endif

	SAFE_DELETE_ARRAY( pAdjFacesSTL );
	SAFE_DELETE_ARRAY( pAdjFaces    );
	SAFE_DELETE_ARRAY( pAdjVtsSTL   );
	SAFE_DELETE_ARRAY( pAdjVts      );
	SAFE_DELETE_ARRAY( pAdjAddress  );

	return 0;
} // CIntObj::CreateNRTs

////////////////////////////////////////////////////////////////////////////////
/// \brief Release the textures created with CIntObj::CreateNRTs.
///
/// \return 0 on success.
///         -1 on error.
////////////////////////////////////////////////////////////////////////////////
int CIntObj::DestroyNRTs( )
{
#if defined(D3D9)
	// Release textures with vertex adjacency data
	SAFE_RELEASE( m_pd3dFaceIdxTex );
	SAFE_RELEASE( m_pd3dAdjFaceTex );
	SAFE_RELEASE( m_pd3dAdjVtTex   );
	SAFE_RELEASE( m_pd3dAdjAddTex  );
#elif defined(OGL2)
	// Release textures with vertex adjacency data
	if( m_uiFaceIdxTex ) 
	{
		glDeleteTextures( 1, &m_uiFaceIdxTex );
		m_uiFaceIdxTex = 0;
	}
	if( m_uiAdjFaceTex ) 
	{
		glDeleteTextures( 1, &m_uiAdjFaceTex );
		m_uiAdjFaceTex = 0;
	}
	if( m_uiAdjVtTex   ) 
	{
		glDeleteTextures( 1, &m_uiAdjVtTex   );
		m_uiAdjVtTex = 0;
	}
	if( m_uiAdjAddTex  ) 
	{
		glDeleteTextures( 1, &m_uiAdjAddTex  );
		m_uiAdjAddTex = 0;
	}
#endif

	return 0;
} // CIntObj::DestroyNRTs

////////////////////////////////////////////////////////////////////////////////
/// \brief Create the shaders.
///
/// \return 0 on success.
///         -1 on error.
///         -2 if there is not a required semantic of a vertex semantic 
///            binding.
////////////////////////////////////////////////////////////////////////////////
int CIntObj::CreateShaders( )
{
#if defined(D3D9)
	HRESULT hrResult S_OK;

	// Define DEBUG_VS and/or DEBUG_PS to debug vertex and/or pixel shaders 
	// with the shader debugger. Debugging vertex shaders requires either REF 
	// or software vertex processing, and debugging pixel shaders requires REF.  
	// The D3DXSHADER_FORCE_*_SOFTWARE_NOOPT flag improves the debug experience 
	// in the shader debugger.  It enables source level debugging, prevents 
	// instruction reordering, prevents dead code elimination, and forces the 
	// compiler to compile against the next higher available software target, 
	// which ensures that the unoptimized shaders do not exceed the shader model 
	// limitations.  Setting these flags will cause slower rendering since the 
	// shaders will be unoptimized and forced into software. See the DirectX 
	// documentation for more information about using the shader debugger.
	DWORD dwShaderFlags = 0;
#ifdef DEBUG_VS
	dwShaderFlags |= D3DXSHADER_FORCE_VS_SOFTWARE_NOOPT;
#endif
#ifdef DEBUG_PS
	dwShaderFlags |= D3DXSHADER_FORCE_PS_SOFTWARE_NOOPT;
#endif

	//
	// Create vertex-level deformation effect
	//

	if( m_szPreDefVS )
	{
		SAFE_RELEASE( m_pVtDefFx );

		//
		// DEBUG: Write shader to file
		//
		FILE *fHandle = fopen( "out_update.fx", "wb" );
		if( fHandle == NULL )
			return -1;

		fwrite( m_szPreDefVS, sizeof( char ), strlen( m_szPreDefVS ), fHandle );

		fclose( fHandle );


		// Load effect from file in memory
		LPD3DXBUFFER pd3dBuffer = NULL;
		hrResult = D3DXCreateEffect( m_pd3dDevice, m_szPreDefVS,
			(UINT) strlen( m_szPreDefVS ), NULL, NULL, dwShaderFlags, NULL, 
			&m_pVtDefFx, &pd3dBuffer );

		// If there are errors, print them
		if( pd3dBuffer )
			fprintf( stderr, (char *) pd3dBuffer->GetBufferPointer( ) );

		if( FAILED( hrResult ) )
			return -1;

		// Get handle to uniform parameters in the effect
		m_hVtDefFxProj    = m_pVtDefFx->GetParameterByName( NULL, "g_mP" );
		m_hVtDefFxVtMapSz = m_pVtDefFx->GetParameterByName( NULL, 
															"g_iVtMapSize" );
	}

	//
	// Create encoder effects (for indexed geometry and non-indexed geometry)
	//

	// First we read the encode.fx file
	FILE *fHandle = fopen( "encode.fx", "rb" );
	if( fHandle == NULL )
		return -1;

	// Get file size
	fseek( fHandle, 0, SEEK_END );
	int iFileSize = ftell( fHandle );
	fseek( fHandle, 0, SEEK_SET );

	// Compute total size
	int iTotalSize = iFileSize + (int) strlen( m_pManager->m_szVSDeform ) +
					 256 * 80;

	// Allocate memory for encoder shader
	char *pFx = new char[ iTotalSize ];
	if( pFx == NULL )
		return -1;
	pFx[0] = 0x00;

	// Read effect file
	fread( pFx, sizeof( char ), iFileSize, fHandle );

	fclose( fHandle );

	pFx[ iFileSize ] = 0x00;

	// Replace [@1] with VSDEFORM data structure
	char *szOut = NULL;
	ReplaceString( 1, pFx, "[@1]", m_pManager->m_szVSDeform, &szOut );
	SAFE_DELETE_ARRAY( pFx );
	pFx = szOut;

	// Replace [@2] with PSDEFORM
	szOut = NULL;
	ReplaceString( 1, pFx, "[@2]", m_pManager->m_szPSDeform, &szOut );
	SAFE_DELETE_ARRAY( pFx );
	pFx = szOut;

	// Replace tokens with fPSDeform
	if( !m_bUsePxDef )
	{
		szOut = NULL;
		ReplaceString( 1, pFx, "[@3]", " ", &szOut );
		SAFE_DELETE_ARRAY( pFx );
		pFx = szOut;

		szOut = NULL;
		ReplaceString( 1, pFx, "[@6]", " ", &szOut );
		SAFE_DELETE_ARRAY( pFx );
		pFx = szOut;
	}
	else
	{
		szOut = NULL;
		ReplaceString( 1, pFx, "[@3]", m_szDefPS, &szOut );
		SAFE_DELETE_ARRAY( pFx );
		pFx = szOut;

		szOut = NULL;
		ReplaceString( 1, pFx, "[@6]", 
			"  DefOut = fPSDeform( In );\n", &szOut );
		SAFE_DELETE_ARRAY( pFx );
		pFx = szOut;
	}

	// Replace tokens with fPostVSDeform
	if( !m_bUsePostDefVS )
	{
		szOut = NULL;
		ReplaceString( 1, pFx, "[@14]", " ", &szOut );
		SAFE_DELETE_ARRAY( pFx );
		pFx = szOut;

		szOut = NULL;
		ReplaceString( 1, pFx, "[@12]", "  return Out;\n", &szOut );
		SAFE_DELETE_ARRAY( pFx );
		pFx = szOut;
	}
	else
	{
		szOut = NULL;
		ReplaceString( 1, pFx, "[@14]", m_szPostDefVS, &szOut );
		SAFE_DELETE_ARRAY( pFx );
		pFx = szOut;

		szOut = NULL;
		ReplaceString( 1, pFx, "[@12]",
			"  return fPostVSDeform( Out );\n", &szOut );
		SAFE_DELETE_ARRAY( pFx );
		pFx = szOut;
	}

	// Look for INTSEMANTIC_VERTEXID vertex semantic
	VSSEMANTIC iVtIDSemantic;
	if( !GetSemantic( INTSEMANTIC_VERTEXID, iVtIDSemantic ) )
		return -2;

	// Replace [@4] with structure member associated with 
	// INTSEMANTIC_VERTEXID
	char *szReplace = new char[32];
    if( szReplace == NULL )
		return -1;
	szReplace[0] = 0x00;

	GetParamStr( iVtIDSemantic, szReplace );

	szOut = NULL;
	ReplaceString( 1, pFx, "[@4]", szReplace, &szOut );
	SAFE_DELETE_ARRAY( pFx );
	pFx = szOut;

	// Replace [@5] with remaining code of vertex shader encoder
	SAFE_DELETE_ARRAY( szReplace );
	szReplace = new char[256 * 80];
	if( szReplace == NULL )
		return -1;
	szReplace[0] = 0x00;

	// Add parameters of VSDEFORM
	int iCt = 0;
	while( m_pManager->m_pBindings[iCt].iVSSemantic != VSSEMANTIC_END )
	{
		char szAtt[32] = {0};

		switch( m_pManager->m_pBindings[iCt].iVSSemantic )
		{
		case VSSEMANTIC_NORMAL   : strcpy( szAtt, "vNormal" ); break;
		case VSSEMANTIC_COLOR0   : strcpy( szAtt, "vColor0" ); break;
		case VSSEMANTIC_COLOR1   : strcpy( szAtt, "vColor1" ); break;
		case VSSEMANTIC_TEXCOORD0: strcpy( szAtt, "vTex0"   ); break;
		case VSSEMANTIC_TEXCOORD1: strcpy( szAtt, "vTex1"   ); break;
		case VSSEMANTIC_TEXCOORD2: strcpy( szAtt, "vTex2"   ); break;
		case VSSEMANTIC_TEXCOORD3: strcpy( szAtt, "vTex3"   ); break;
		case VSSEMANTIC_TEXCOORD4: strcpy( szAtt, "vTex4"   ); break;
		case VSSEMANTIC_TEXCOORD5: strcpy( szAtt, "vTex5"   ); break;
		case VSSEMANTIC_TEXCOORD6: strcpy( szAtt, "vTex6"   ); break;
		case VSSEMANTIC_TEXCOORD7: strcpy( szAtt, "vTex7"   ); break;
		}

		strcat( szReplace, "  Out." );
		strcat( szReplace, szAtt );
		strcat( szReplace, " = In." );
		strcat( szReplace, szAtt );
		strcat( szReplace, ";\n" );

		iCt++;
	}

	szOut = NULL;
	ReplaceString( 1, pFx, "[@13]", szReplace, &szOut );
	SAFE_DELETE_ARRAY( pFx );
	pFx = szOut;

	szReplace[0] = 0x00;

	//
	// Create encoder effect for non-indexed geometry
	//
	if( m_pManager->m_bAttUserDefNI )
	{
		// First we make a copy of pFx
		char *pFx2 = new char[ (int) strlen( pFx ) + 1 ];
		if( pFx2 == NULL )
			return -1;
		pFx2[0] = 0x00;
		strcpy( pFx2, pFx );

		// Using normal?
		if( m_pManager->m_bAttNormal || m_pManager->m_bAttTBN )
		{
			strcat( szReplace, "  Out.vTN = 0;\n\n" );
		}

		// Using TBN frame?
		if( m_pManager->m_bAttTBN )
		{
			strcat( szReplace, "  Out.vTT = 0;\n"
							   "  Out.vTB = 0;\n\n" );
		}

		// Using curvature?
		if( m_pManager->m_bAttCurv )
		{
			strcat( szReplace, "  Out.vP1K1  = 0;\n"
							   "  Out.vP2K2  = 0;\n" );
		}

		// Using 2nd fundamental form?
		if( m_pManager->m_bAttTensor2 )
		{
			strcat( szReplace, "  Out.vTensor2 = 0;\n"	);
		}

		// Using tensor of curvature derivative?
		if( m_pManager->m_bAttTensor3 )
		{
			strcat( szReplace, "  Out.vTensor3 = 0;\n" );
		}

		szOut = NULL;
		ReplaceString( 1, pFx2, "[@5]", szReplace, &szOut );
		SAFE_DELETE_ARRAY( pFx2 );
		pFx2 = szOut;

		szReplace[0] = 0x00;

		// PS_OUT returns only one vColor
		szOut = NULL;
		ReplaceString( 1, pFx2, "[@10]", "1", &szOut );
		SAFE_DELETE_ARRAY( pFx2 );
		pFx2 = szOut;

		szOut = NULL;
		ReplaceString( 1, pFx2, "[@7]", 
			"  Out.vColor[0] = DefOut.[@8];\n", &szOut );
		SAFE_DELETE_ARRAY( pFx2 );
		pFx2 = szOut;

		// Look for INTSEMANTIC_USERDEFNI vertex semantic
		VSSEMANTIC iUserDefSemantic;
		if( !GetSemantic( INTSEMANTIC_USERDEFNI, iUserDefSemantic ) )
			return -2;

		// Replace [@8] with structure member associated with 
		// INTSEMANTIC_USERDEFNI
		char szAtt[32] = {0};
		GetParamStr( iUserDefSemantic, szAtt );

		szOut = NULL;
		ReplaceString( 1, pFx2, "[@8]", szAtt, &szOut );
		SAFE_DELETE_ARRAY( pFx2 );
		pFx2 = szOut;

		SAFE_RELEASE( m_pEncodeNIFx );


		// DEBUG: Write shader to file
		fHandle = fopen( "enc_out.fx", "wb" );
		if( fHandle == NULL )
			return -1;

		fwrite( pFx2, sizeof( char ), strlen( pFx2 ), fHandle );

		fclose( fHandle );


		// Load effect from file in memory
		LPD3DXBUFFER pd3dBuffer = NULL;
		hrResult = D3DXCreateEffect( m_pd3dDevice, pFx2,
			(UINT) strlen( pFx2 ), NULL, NULL, dwShaderFlags, NULL, 
			&m_pEncodeNIFx, &pd3dBuffer );

		// If there are errors, print them
		if( pd3dBuffer )
			fprintf( stderr, (char*) pd3dBuffer->GetBufferPointer( ) );

		if( FAILED( hrResult ) )
			return -1;

		// Get handle to uniform parameters in the effect
		m_hEncodeNIFxIdxC = m_pEncodeNIFx->GetParameterByName( 
			NULL, "g_fVtIdxC" );
		m_hEncodeNIFxWVP  = m_pEncodeNIFx->GetParameterByName( 
			NULL, "g_mWVP" );
		m_hEncodeNIFxITW  = m_pEncodeNIFx->GetParameterByName( 
			NULL, "g_mITW" );

		SAFE_DELETE_ARRAY( pFx2 );
	}

	SAFE_DELETE_ARRAY( szReplace );

	//
	// Create encoder effect for indexed geometry
	//

	// TODO: Make as many copies of pFx as there are render passes
	//       Then, for each copy:
	//       - Replace [@10] with number of render targets used in current pass.
	//       - Replace [@7] with source code to encode data.

	for( int iCt = 0; iCt < MAX_RENDER_PASSES; iCt++ )
		SAFE_RELEASE( m_pEncodeIFx[iCt] );

	int iRP   = 0; // Current render pass
	int iRT   = 0; // Current render target
	int iComp = 0; // Current color component

	const int iTempSz     = 256;
	char chComp [5]       = "rgba";
	char szTemp [iTempSz] = {0};
	char szTemp2[iTempSz] = {0};

	g_bAttNormal = g_bAttTBN = g_bAttCurv =	false;
	g_bAttTensor2 = g_bAttTensor3 = false;

	//
	// Create an effect for each render pass
	//

	// Make a copy of pFx for the 1st render pass
	char *pFx2 = new char[ (int) strlen( pFx ) + 1 ];
	if( pFx2 == NULL )
		return -1;
	pFx2[0] = 0x00;
	strcpy( pFx2, pFx );

	szReplace = new char[256 * 80];
	if( szReplace == NULL )
		return -1;
	szReplace[0] = 0x00;

	int iVtSamplers = 1;

	// ATTTYPE_DEPTH
	if( m_pManager->m_bAttDepth )
	{
		// Components must fit in the same render pass
		if( 1 > ( m_pManager->m_iNumMRTs - 
			( iRT % m_pManager->m_iNumMRTs ) ) * 4 - iComp ||
			iVtSamplers+0 > 4 )
		{ 
			if( !CreatePSEncoder( iRP, iRT, iComp, &pFx2, szReplace ) )
				return -1;

			// Make a copy of pFx for the next render pass
			strcpy( pFx2, pFx );
			iComp = 0; iRT = 0; iRP++; 
			iVtSamplers = 1;

			g_bAttNormal = g_bAttTBN = g_bAttCurv =	false;
			g_bAttTensor2 = g_bAttTensor3 = false;
		}

		// Depth
		if( iComp == 4 ) { iComp = 0; iRT++; };
		_snprintf( szTemp, iTempSz, "  Out.vColor[%d].", iRT );
		strcat ( szReplace, szTemp );
		strncat( szReplace, &chComp[iComp], 1 );	
		strcat ( szReplace, " = DefOut.vTPos.z / DefOut.vTPos.w;\n" );
//		strcat ( szReplace, " = DefOut.vTPos.z;\n" );
		iComp++;
	}

	// ATTTYPE_TEXCOORD
	if( m_pManager->m_bAttTexCoord )
	{
		// Components must fit in the same render pass
		if( 2 > ( m_pManager->m_iNumMRTs - 
			( iRT % m_pManager->m_iNumMRTs ) ) * 4 - iComp ||
			iVtSamplers+0 > 4 )
		{ 
			if( !CreatePSEncoder( iRP, iRT, iComp, &pFx2, szReplace ) )
				return -1;

			// Make a copy of pFx for the next render pass
			strcpy( pFx2, pFx );
			iComp = 0; iRT = 0; iRP++; 
			iVtSamplers = 1;

			g_bAttNormal = g_bAttTBN = g_bAttCurv =	false;
			g_bAttTensor2 = g_bAttTensor3 = false;
		}

		VSSEMANTIC iTexCoordSemantic;
		GetSemantic( INTSEMANTIC_TEXCOORD, iTexCoordSemantic );
		GetParamStr( iTexCoordSemantic, szTemp2 );

		// U
		if( iComp == 4 ) { iComp = 0; iRT++; };
		_snprintf( szTemp, iTempSz, "  Out.vColor[%d].", iRT );
		strcat   ( szReplace, szTemp );
		strncat  ( szReplace, &chComp[iComp], 1 );	

		_snprintf( szTemp, iTempSz, " = DefOut.%s.r;\n", szTemp2 );
		strcat   ( szReplace, szTemp );
		iComp++; 

		// V
		if( iComp == 4 ) { iComp = 0; iRT++; };
		_snprintf( szTemp, iTempSz, "  Out.vColor[%d].", iRT );
		strcat   ( szReplace, szTemp );
		strncat  ( szReplace, &chComp[iComp], 1 );	

		_snprintf( szTemp, iTempSz, " = DefOut.%s.g;\n", szTemp2 );
		strcat   ( szReplace, szTemp );
		iComp++; 
	}

	// ATTTYPE_TBN
	if( m_pManager->m_bAttTBN )
	{
		// Components must fit in the same render pass
		if( 6 > ( m_pManager->m_iNumMRTs - 
			( iRT % m_pManager->m_iNumMRTs ) ) * 4 - iComp ||
			iVtSamplers+2 > 4 )
		{
			if( !CreatePSEncoder( iRP, iRT, iComp, &pFx2, szReplace ) )
				return -1;

			// Make a copy of pFx for the next render pass
			strcpy( pFx2, pFx );
			iComp = 0; iRT = 0; iRP++; 
			iVtSamplers = 1;

			g_bAttNormal = g_bAttTBN = g_bAttCurv =	false;
			g_bAttTensor2 = g_bAttTensor3 = false;
		}

		iVtSamplers += 2;
		g_bAttTBN = true;

		// T.x
		if( iComp == 4 ) { iComp = 0; iRT++; };
		_snprintf( szTemp, iTempSz, "  Out.vColor[%d].", iRT );
		strcat   ( szReplace, szTemp );
		strncat  ( szReplace, &chComp[iComp], 1 );

		strcat   ( szReplace, " = DefOut.vTT.r;\n" );
		iComp++; 

		// T.y
		if( iComp == 4 ) { iComp = 0; iRT++; };
		_snprintf( szTemp, iTempSz, "  Out.vColor[%d].", iRT );
		strcat   ( szReplace, szTemp );
		strncat  ( szReplace, &chComp[iComp], 1 );

		strcat   ( szReplace, " = DefOut.vTT.g;\n" );
		iComp++; 

		// T.z
		if( iComp == 4 ) { iComp = 0; iRT++; };
		_snprintf( szTemp, iTempSz, "  Out.vColor[%d].", iRT );
		strcat   ( szReplace, szTemp );
		strncat  ( szReplace, &chComp[iComp], 1 );

		strcat   ( szReplace, " = DefOut.vTT.b;\n" );
		iComp++; 

		// B.x
		if( iComp == 4 ) { iComp = 0; iRT++; };
		_snprintf( szTemp, iTempSz, "  Out.vColor[%d].", iRT );
		strcat   ( szReplace, szTemp );
		strncat  ( szReplace, &chComp[iComp], 1 );

		strcat   ( szReplace, " = DefOut.vTB.r;\n" );
		iComp++; 

		// B.y
		if( iComp == 4 ) { iComp = 0; iRT++; };
		_snprintf( szTemp, iTempSz, "  Out.vColor[%d].", iRT );
		strcat   ( szReplace, szTemp );
		strncat  ( szReplace, &chComp[iComp], 1 );

		strcat   ( szReplace, " = DefOut.vTB.g;\n" );
		iComp++; 

		// B.z
		if( iComp == 4 ) { iComp = 0; iRT++; };
		_snprintf( szTemp, iTempSz, "  Out.vColor[%d].", iRT );
		strcat   ( szReplace, szTemp );
		strncat  ( szReplace, &chComp[iComp], 1 );

		strcat   ( szReplace, " = DefOut.vTB.b;\n" );
		iComp++; 
	}

	// ATTTYPE_USERDEFI
	if( m_pManager->m_bAttUserDefI )
	{
		// Components must fit in the same render pass
		if( 4 > ( m_pManager->m_iNumMRTs - 
			( iRT % m_pManager->m_iNumMRTs ) ) * 4 - iComp ||
			iVtSamplers+0 > 4 )
		{ 
			if( !CreatePSEncoder( iRP, iRT, iComp, &pFx2, szReplace ) )
				return -1;

			// Make a copy of pFx for the next render pass
			strcpy( pFx2, pFx );

			iComp = 0; iRT = 0; iRP++; 
			iVtSamplers = 1;

			g_bAttNormal = g_bAttTBN = g_bAttCurv =	false;
			g_bAttTensor2 = g_bAttTensor3 = false;
		}

		VSSEMANTIC iUserDefSemantic;
		GetSemantic( INTSEMANTIC_USERDEFI, iUserDefSemantic );
		GetParamStr( iUserDefSemantic, szTemp2 );

		// r
		if( iComp == 4 ) { iComp = 0; iRT++; };
		_snprintf( szTemp, iTempSz, "  Out.vColor[%d].", iRT );
		strcat   ( szReplace, szTemp );
		strncat  ( szReplace, &chComp[iComp], 1 );	

		_snprintf( szTemp, iTempSz, " = DefOut.%s.r;\n", szTemp2 );
		strcat   ( szReplace, szTemp );
		iComp++; 

		// g
		if( iComp == 4 ) { iComp = 0; iRT++; };
		_snprintf( szTemp, iTempSz, "  Out.vColor[%d].", iRT );
		strcat   ( szReplace, szTemp );
		strncat  ( szReplace, &chComp[iComp], 1 );	

		_snprintf( szTemp, iTempSz, " = DefOut.%s.g;\n", szTemp2 );
		strcat   ( szReplace, szTemp );
		iComp++; 

		// b
		if( iComp == 4 ) { iComp = 0; iRT++; };
		_snprintf( szTemp, iTempSz, "  Out.vColor[%d].", iRT );
		strcat   ( szReplace, szTemp );
		strncat  ( szReplace, &chComp[iComp], 1 );	

		_snprintf( szTemp, iTempSz, " = DefOut.%s.b;\n", szTemp2 );
		strcat   ( szReplace, szTemp );
		iComp++; 

		// a
		if( iComp == 4 ) { iComp = 0; iRT++; };
		_snprintf( szTemp, iTempSz, "  Out.vColor[%d].", iRT );
		strcat   ( szReplace, szTemp );
		strncat  ( szReplace, &chComp[iComp], 1 );	

		_snprintf( szTemp, iTempSz, " = DefOut.%s.a;\n", szTemp2 );
		strcat   ( szReplace, szTemp );
		iComp++; 
	}

	// ATTTYPE_NORMAL
	if( m_pManager->m_bAttNormal )
	{
		// Components must fit in the same render pass
		if( 3 > ( m_pManager->m_iNumMRTs - 
			( iRT % m_pManager->m_iNumMRTs ) ) * 4 - iComp ||
			iVtSamplers+1 > 4 )
		{ 
			if( !CreatePSEncoder( iRP, iRT, iComp, &pFx2, szReplace ) )
				return -1;

			// Make a copy of pFx for the next render pass
			strcpy( pFx2, pFx );

			iComp = 0; iRT = 0; iRP++; 
			iVtSamplers = 1;

			g_bAttNormal = g_bAttTBN = g_bAttCurv =	false;
			g_bAttTensor2 = g_bAttTensor3 = false;
		}

		iVtSamplers++;
		g_bAttNormal = true;

		// N.x
		if( iComp == 4 ) { iComp = 0; iRT++; };
		_snprintf( szTemp, iTempSz, "  Out.vColor[%d].", iRT );
		strcat   ( szReplace, szTemp );
		strncat  ( szReplace, &chComp[iComp], 1 );

		strcat   ( szReplace, " = DefOut.vTN.r;\n" );
		iComp++; 

		// N.y
		if( iComp == 4 ) { iComp = 0; iRT++; };
		_snprintf( szTemp, iTempSz, "  Out.vColor[%d].", iRT );
		strcat   ( szReplace, szTemp );
		strncat  ( szReplace, &chComp[iComp], 1 );

		strcat   ( szReplace, " = DefOut.vTN.g;\n" );
		iComp++; 

		// N.z
		if( iComp == 4 ) { iComp = 0; iRT++; };
		_snprintf( szTemp, iTempSz, "  Out.vColor[%d].", iRT );
		strcat   ( szReplace, szTemp );
		strncat  ( szReplace, &chComp[iComp], 1 );

		strcat   ( szReplace, " = DefOut.vTN.b;\n" );
		iComp++; 
	}

	// ATTTYPE_CURV
	if( m_pManager->m_bAttCurv )
	{
		// Components must fit in the same render pass
		if( 8 > ( m_pManager->m_iNumMRTs - 
			( iRT % m_pManager->m_iNumMRTs ) ) * 4 - iComp ||
			iVtSamplers+2 > 4 )
		{ 
			if( !CreatePSEncoder( iRP, iRT, iComp, &pFx2, szReplace ) )
				return -1;

			// Make a copy of pFx for the next render pass
			strcpy( pFx2, pFx );

			iComp = 0; iRT = 0; iRP++; 
			iVtSamplers = 1;

			g_bAttNormal = g_bAttTBN = g_bAttCurv =	false;
			g_bAttTensor2 = g_bAttTensor3 = false;
		}

		iVtSamplers += 2;
		g_bAttCurv = true;

		// P1.x
		if( iComp == 4 ) { iComp = 0; iRT++; };
		_snprintf( szTemp, iTempSz, "  Out.vColor[%d].", iRT );
		strcat   ( szReplace, szTemp );
		strncat  ( szReplace, &chComp[iComp], 1 );

		strcat   ( szReplace, " = DefOut.vP1K1.r;\n" );
		iComp++; 

		// P1.y
		if( iComp == 4 ) { iComp = 0; iRT++; };
		_snprintf( szTemp, iTempSz, "  Out.vColor[%d].", iRT );
		strcat   ( szReplace, szTemp );
		strncat  ( szReplace, &chComp[iComp], 1 );

		strcat   ( szReplace, " = DefOut.vP1K1.g;\n" );
		iComp++; 

		// P1.z
		if( iComp == 4 ) { iComp = 0; iRT++; };
		_snprintf( szTemp, iTempSz, "  Out.vColor[%d].", iRT );
		strcat   ( szReplace, szTemp );
		strncat  ( szReplace, &chComp[iComp], 1 );

		strcat   ( szReplace, " = DefOut.vP1K1.b;\n" );
		iComp++; 

		// k1
		if( iComp == 4 ) { iComp = 0; iRT++; };
		_snprintf( szTemp, iTempSz, "  Out.vColor[%d].", iRT );
		strcat   ( szReplace, szTemp );
		strncat  ( szReplace, &chComp[iComp], 1 );

		strcat   ( szReplace, " = DefOut.vP1K1.a;\n" );
		iComp++; 

		// P2.x
		if( iComp == 4 ) { iComp = 0; iRT++; };
		_snprintf( szTemp, iTempSz, "  Out.vColor[%d].", iRT );
		strcat   ( szReplace, szTemp );
		strncat  ( szReplace, &chComp[iComp], 1 );

		strcat   ( szReplace, " = DefOut.vP2K2.r;\n" );
		iComp++; 

		// P2.y
		if( iComp == 4 ) { iComp = 0; iRT++; };
		_snprintf( szTemp, iTempSz, "  Out.vColor[%d].", iRT );
		strcat   ( szReplace, szTemp );
		strncat  ( szReplace, &chComp[iComp], 1 );

		strcat   ( szReplace, " = DefOut.vP2K2.g;\n" );
		iComp++; 

		// P2.z
		if( iComp == 4 ) { iComp = 0; iRT++; };
		_snprintf( szTemp, iTempSz, "  Out.vColor[%d].", iRT );
		strcat   ( szReplace, szTemp );
		strncat  ( szReplace, &chComp[iComp], 1 );

		strcat   ( szReplace, " = DefOut.vP2K2.b;\n" );
		iComp++; 

		// k2
		if( iComp == 4 ) { iComp = 0; iRT++; };
		_snprintf( szTemp, iTempSz, "  Out.vColor[%d].", iRT );
		strcat   ( szReplace, szTemp );
		strncat  ( szReplace, &chComp[iComp], 1 );

		strcat   ( szReplace, " = DefOut.vP2K2.a;\n" );
		iComp++; 
	}

	// ATTTYPE_TENSOR2
	if( m_pManager->m_bAttTensor2 )
	{
		// Components must fit in the same render pass
		if( 3 > ( m_pManager->m_iNumMRTs - 
			( iRT % m_pManager->m_iNumMRTs ) ) * 4 - iComp ||
			iVtSamplers+1 > 4 )
		{ 
			if( !CreatePSEncoder( iRP, iRT, iComp, &pFx2, szReplace ) )
				return -1;

			// Make a copy of pFx for the next render pass
			strcpy( pFx2, pFx );

			iComp = 0; iRT = 0; iRP++; 
			iVtSamplers = 1;

			g_bAttNormal = g_bAttTBN = g_bAttCurv =	false;
			g_bAttTensor2 = g_bAttTensor3 = false;
		}

		iVtSamplers++;
		g_bAttTensor2 = true;

		// e
		if( iComp == 4 ) { iComp = 0; iRT++; };
		_snprintf( szTemp, iTempSz, "  Out.vColor[%d].", iRT );
		strcat   ( szReplace, szTemp );
		strncat  ( szReplace, &chComp[iComp], 1 );

		strcat   ( szReplace, " = DefOut.vTensor2.r;\n" );
		iComp++; 

		// f
		if( iComp == 4 ) { iComp = 0; iRT++; };
		_snprintf( szTemp, iTempSz, "  Out.vColor[%d].", iRT );
		strcat   ( szReplace, szTemp );
		strncat  ( szReplace, &chComp[iComp], 1 );

		strcat   ( szReplace, " = DefOut.vTensor2.g;\n" );
		iComp++; 

		// g
		if( iComp == 4 ) { iComp = 0; iRT++; };
		_snprintf( szTemp, iTempSz, "  Out.vColor[%d].", iRT );
		strcat   ( szReplace, szTemp );
		strncat  ( szReplace, &chComp[iComp], 1 );

		strcat   ( szReplace, " = DefOut.vTensor2.b;\n" );
		iComp++; 
	}

	// ATTTYPE_TENSOR3
	if( m_pManager->m_bAttTensor3 )
	{
		// Components must fit in the same render pass
		if( 4 > ( m_pManager->m_iNumMRTs - 
			( iRT % m_pManager->m_iNumMRTs ) ) * 4 - iComp ||
			iVtSamplers+1 > 4 )
		{ 
			if( !CreatePSEncoder( iRP, iRT, iComp, &pFx2, szReplace ) )
				return -1;

			// Make a copy of pFx for the next render pass
			strcpy( pFx2, pFx );

			iComp = 0; iRT = 0; iRP++; 
			iVtSamplers = 1;

			g_bAttNormal = g_bAttTBN = g_bAttCurv =	false;
			g_bAttTensor2 = g_bAttTensor3 = false;
		}

		iVtSamplers++;
		g_bAttTensor3 = true;

		// a
		if( iComp == 4 ) { iComp = 0; iRT++; };
		_snprintf( szTemp, iTempSz, "  Out.vColor[%d].", iRT );
		strcat   ( szReplace, szTemp );
		strncat  ( szReplace, &chComp[iComp], 1 );

		strcat   ( szReplace, " = DefOut.vTensor3.r;\n" );
		iComp++; 

		// b
		if( iComp == 4 ) { iComp = 0; iRT++; };
		_snprintf( szTemp, iTempSz, "  Out.vColor[%d].", iRT );
		strcat   ( szReplace, szTemp );
		strncat  ( szReplace, &chComp[iComp], 1 );

		strcat   ( szReplace, " = DefOut.vTensor3.g;\n" );
		iComp++; 

		// c
		if( iComp == 4 ) { iComp = 0; iRT++; };
		_snprintf( szTemp, iTempSz, "  Out.vColor[%d].", iRT );
		strcat   ( szReplace, szTemp );
		strncat  ( szReplace, &chComp[iComp], 1 );

		strcat   ( szReplace, " = DefOut.vTensor3.b;\n" );
		iComp++; 

		// d
		if( iComp == 4 ) { iComp = 0; iRT++; };
		_snprintf( szTemp, iTempSz, "  Out.vColor[%d].", iRT );
		strcat   ( szReplace, szTemp );
		strncat  ( szReplace, &chComp[iComp], 1 );

		strcat   ( szReplace, " = DefOut.vTensor3.a;\n" );
		iComp++; 
	}

	if( iComp == 0 && iRT )
		iRT--;
	else
	{
		// Fill color components not used
		while( iComp < 4 )
		{
			_snprintf( szTemp, iTempSz, "  Out.vColor[%d].", iRT );
			strcat   ( szReplace, szTemp );
			strncat  ( szReplace, &chComp[iComp], 1 ) ;	
			strcat   ( szReplace, " = 1.0f;\n" );

			iComp++;
		}
	}

	szOut = NULL;
	_snprintf( szTemp, iTempSz, "%d", iRT + 1 );
	ReplaceString( 1, pFx2, "[@10]", szTemp, &szOut );
	SAFE_DELETE_ARRAY( pFx2 );
	pFx2 = szOut;

	szOut = NULL;
	ReplaceString( 1, pFx2, "[@7]", szReplace, &szOut );
	SAFE_DELETE_ARRAY( pFx2 );
	pFx2 = szOut;

	szReplace[0] = 0x00;

	// Using normal?
	if( g_bAttNormal || g_bAttTBN )
	{
		strcat( szReplace, 
			"  float3 vN = tex2Dlod( s1, float4( vVtUV, 0, 0 ) ).xyz;\n"
			//			"  Out.vTN = mul( float4(vN, 0), g_mITW );\n\n" );
			"  Out.vTN = vN;\n\n" );
	}
	else
	{
		if( m_pManager->m_bAttNormal || m_pManager->m_bAttTBN )
		{
			strcat( szReplace, "  Out.vTN = 0;\n\n" );
		}
	}

	// Using TBN frame?
	if( g_bAttTBN )
	{
		strcat( szReplace, 
			"  float4 vT = tex2Dlod( s3, float4( vVtUV, 0, 0 ) );\n"
			"  float3 vB = cross( vT.xyz, vN ) * vT.w;\n"
			//			"  Out.vTT = mul( float4(vT.xyz,0), g_mITW );\n"
			//			"  Out.vTB = mul( float4(vB, 0), g_mITW );\n\n" );
			"  Out.vTT = vT.xyz;\n"
			"  Out.vTB = vB;\n\n" );
	}
	else if( m_pManager->m_bAttTBN )
	{
			strcat( szReplace, "  Out.vTT = 0;\n"
							    "  Out.vTB = 0;\n\n" );
	}

	// Using curvature?
	if( g_bAttCurv )
	{
		strcat( szReplace,
			"  float4 vP1K1 = tex2Dlod( s4, float4( vVtUV, 0, 0 ) );\n"
			"  float4 vP2K2 = tex2Dlod( s5, float4( vVtUV, 0, 0 ) );\n"
			"  Out.vP1K1  = vP1K1;\n"
			"  Out.vP2K2  = vP2K2;\n"
//			"  Out.vP1K1  = float4( mul( float4(vP1K1.xyz,0), g_mITW ).xyz, vP1K1.w );\n"
//			"  Out.vP2K2  = float4( mul( float4(vP2K2.xyz,0), g_mITW ).xyz, vP2K2.w );\n"
			);
	}
	else if( m_pManager->m_bAttCurv )
	{
		strcat( szReplace, "  Out.vP1K1 = 0;\n"
			                "  Out.vP2K2 = 0;\n" );
	}

	// Using 2nd fundamental form?
	if( g_bAttTensor2 )
	{
		strcat( szReplace,
			"  Out.vTensor2 = tex2Dlod( s6, float4( vVtUV, 0, 0 ) );\n"	);
	}
	else if( m_pManager->m_bAttTensor2 )
	{
		strcat( szReplace, "  Out.vTensor2 = 0;\n" );
	}

	// Using tensor of curvature derivative?
	if( g_bAttTensor3 )
	{
		strcat( szReplace,
			"  Out.vTensor3 = tex2Dlod( s7, float4( vVtUV, 0, 0 ) );\n" );
	}
	else if( m_pManager->m_bAttTensor3 )
	{
		strcat( szReplace, "  Out.vTensor3 = 0;\n" );
	}

	szOut = NULL;
	ReplaceString( 1, pFx2, "[@5]", szReplace, &szOut );
	SAFE_DELETE_ARRAY( pFx2 );
	pFx2 = szOut;


	// DEBUG: Write shader to file
	fHandle = fopen( "enc_out.fx", "wb" );
	if( fHandle == NULL )
		return -1;

	fwrite( pFx2, sizeof( char ), strlen( pFx2 ), fHandle );

	fclose( fHandle );

	//
	// Create encoding (indexed geometry) effect from string
	//

	LPD3DXBUFFER pd3dBuffer = NULL;
	hrResult = D3DXCreateEffect( m_pd3dDevice, pFx2,
		(UINT) strlen( pFx2 ), NULL, NULL, dwShaderFlags, NULL, 
		&m_pEncodeIFx[iRP], &pd3dBuffer );

	// If there are errors, print them
	if( pd3dBuffer )
		fprintf( stderr, (char*) pd3dBuffer->GetBufferPointer( ) );

	if( FAILED( hrResult ) )
		return -1;

	// Get handle to uniform parameters in the effect
	m_hEncodeIFxIdxC[iRP] = m_pEncodeIFx[iRP]->GetParameterByName( 
		NULL, "g_fVtIdxC" );
	m_hEncodeIFxWVP[iRP]  = m_pEncodeIFx[iRP]->GetParameterByName( 
		NULL, "g_mWVP" );
	m_hEncodeIFxITW[iRP]  = m_pEncodeIFx[iRP]->GetParameterByName( 
		NULL, "g_mITW" );

	SAFE_DELETE_ARRAY( pFx2 );

	// TODO: New render pass here

	SAFE_DELETE_ARRAY( pFx );
	SAFE_DELETE_ARRAY( szReplace );

	{
		//
		// Load render shader
		//

		// First we read the render.fx file
		FILE *fHandle = fopen( "render.fx", "rb" );
		if( fHandle == NULL )
			return -1;

		// Get file size
		fseek( fHandle, 0, SEEK_END );
		int iFileSize = ftell( fHandle );
		fseek( fHandle, 0, SEEK_SET );

		// Allocate memory for encoder shader
		char *pFx = new char[ iFileSize + 128 ];
		if( pFx == NULL )
			return -1;
		pFx[0] = 0x00;

		// Read effect file
		fread( pFx, sizeof( char ), iFileSize, fHandle );
		pFx[ iFileSize ] = 0x00;

		// Look for INTSEMANTIC_VERTEXID vertex semantic
		int iCt = 0;
		VSSEMANTIC iVtIDSemantic;
		if( !GetSemantic( INTSEMANTIC_VERTEXID, iVtIDSemantic ) )
			return -2;

		// Replace occurrence of [@1] with
		// structure member associated with INTSEMANTIC_VERTEXID
		char *szReplace = new char[32];
		if( szReplace == NULL )
			return -1;

		GetSemanticStr( iVtIDSemantic, szReplace );

		char *szOut;
		ReplaceString( 1, pFx, "[@1]", szReplace, &szOut );
		SAFE_DELETE_ARRAY( pFx );
		pFx = szOut;

		LPD3DXBUFFER pd3dBuffer = NULL;
		hrResult = D3DXCreateEffect( m_pd3dDevice, pFx,
			(UINT) strlen( pFx ), NULL, NULL, 0, NULL, 
			&m_pRenderFx, &pd3dBuffer );

		// If there are errors, print them
		if( pd3dBuffer )
			fprintf( stderr, (char *) pd3dBuffer->GetBufferPointer( ) );

		if( FAILED( hrResult ) )
			return false;

		// Get handle to uniform parameters in the effect
		m_hRenderFxW    = m_pRenderFx->GetParameterByName( NULL, "g_mW"      );
		m_hRenderFxV    = m_pRenderFx->GetParameterByName( NULL, "g_mV"      );
		m_hRenderFxWVP  = m_pRenderFx->GetParameterByName( NULL, "g_mWVP"    );
		m_hRenderFxITW  = m_pRenderFx->GetParameterByName( NULL, "g_mITW"    );
		m_hRenderFxIdxC = m_pRenderFx->GetParameterByName( NULL, "g_fVtIdxC" );

		SAFE_DELETE_ARRAY( szReplace );
		SAFE_DELETE_ARRAY( pFx );
	}

#elif defined(OGL2)
	//
	// Create vertex-level deformation effect
	//

	if( m_szPreDefVS )
	{
		//
		// DEBUG: Write shader to file
		//
		FILE *fHandle = fopen( "out_updatevs.glsl", "wb" );
		if( fHandle == NULL )
			return -1;
		fwrite( m_szPreDefVS, sizeof( char ), strlen( m_szPreDefVS ), fHandle );
		fclose( fHandle );

		fHandle = fopen( "out_updateps.glsl", "wb" );
		if( fHandle == NULL )
			return -1;
		if( m_bDefTex )
			fwrite( g_szVtDefPSTex, sizeof( char ), strlen( g_szVtDefPSTex ), fHandle );
		else
			fwrite( g_szVtDefPS, sizeof( char ), strlen( g_szVtDefPS ), fHandle );
		fclose( fHandle );

		// Destroy previous vertex-level deformation shaders, if any
		if( m_hVtDefPO )
		{
			glUseProgramObjectARB( 0 );
			if( m_hVtDefPO ) 
			{
				glDeleteObjectARB( m_hVtDefPO );
				m_hVtDefPO = 0;
			}
		}

		// Create program object
		m_hVtDefPO = glCreateProgramObjectARB( );

		// Create shader objects
		GLhandleARB hVSSO = glCreateShaderObjectARB( GL_VERTEX_SHADER_ARB   );
		GLhandleARB hPSSO = glCreateShaderObjectARB( GL_FRAGMENT_SHADER_ARB );
		glShaderSourceARB( hVSSO, 1, (const GLcharARB**) &m_szPreDefVS, NULL );
		if( m_bDefTex )
			glShaderSourceARB( hPSSO, 1, (const GLcharARB**) &g_szVtDefPSTex, 
							   NULL );
		else
			glShaderSourceARB( hPSSO, 1, (const GLcharARB**) &g_szVtDefPS, 
							   NULL );

		GLint iResult = 0;

		// Compile vertex shader
		glCompileShaderARB( hVSSO );
		glGetObjectParameterivARB( hVSSO, GL_OBJECT_COMPILE_STATUS_ARB, 
								  &iResult );
		if( iResult == GL_FALSE )
		{
	//		glGetInfoLogARB( hVSS0 );
			return -1;
		}

		// Compile fragment shader
		glCompileShaderARB( hPSSO );
		glGetObjectParameterivARB( hPSSO, GL_OBJECT_COMPILE_STATUS_ARB, 
								   &iResult );
		if( iResult == GL_FALSE )
		{
	//		glGetInfoLogARB( hPSS0 );
			return -1;
		}

		// Attach shaders to program
		glAttachObjectARB( m_hVtDefPO, hVSSO );
		glAttachObjectARB( m_hVtDefPO, hPSSO );
		// Flag shaders for deletion when they are no longer referenced
		if( hVSSO ) { glDeleteObjectARB( hVSSO ); hVSSO = 0; }
		if( hPSSO ) { glDeleteObjectARB( hPSSO ); hPSSO = 0; }

		glLinkProgramARB( m_hVtDefPO );

		// Check linkage status
		glGetObjectParameterivARB( m_hVtDefPO, GL_OBJECT_LINK_STATUS_ARB,
								   &iResult );
		if( !iResult )
		{
	//		glGetInfoLogARB( m_hVtDefPO );
			return -1;
		}

		// Use vertex-level deformation shader
		glUseProgramObjectARB( m_hVtDefPO );

		// Get handlers to uniform parameters
		m_iVtDefPOVtMapSize = glGetUniformLocationARB( m_hVtDefPO, 
													   "g_iVtMapSize" );

		// Disable program object
		glUseProgramObjectARB( 0 );
	}

	//
	// Create encoder effects (for indexed geometry and non-indexed geometry)
	//

	// Read the files encodevs.glsl and encodeps.glsl
	FILE *fHandleVS = fopen( "encodevs.glsl", "rb" );
	if( fHandleVS == NULL )
		return -1;

	FILE *fHandlePS = fopen( "encodeps.glsl", "rb" );
	if( fHandlePS == NULL )
		return -1;

	// Get file size of encodevs.glsl
	fseek( fHandleVS, 0, SEEK_END );
	int iFileSizeVS = ftell( fHandleVS );
	fseek( fHandleVS, 0, SEEK_SET );

	// Compute total size of encodevs.glsl
	int iTotalSizeVS = iFileSizeVS + (int) strlen( m_pManager->m_szVSDeform ) +
					   256 * 80;

	// Get file size of encodeps.glsl
	fseek( fHandlePS, 0, SEEK_END );
	int iFileSizePS = ftell( fHandlePS );
	fseek( fHandlePS, 0, SEEK_SET );

	// Compute total size of encodeps.glsl
	int iTotalSizePS = iFileSizePS + (int) strlen( m_pManager->m_szPSDeform ) +
					   256 * 80;

	// Allocate memory for encoder vertex shader
	char *pVS = new char[ iTotalSizeVS ];
	if( pVS == NULL ) return -1;
	pVS[0] = 0x00;

	// Allocate memory for encoder pixel shader
	char *pPS = new char[ iTotalSizePS ];
	if( pPS == NULL ) return -1;
	pPS[0] = 0x00;

	// Read encodevs.glsl
	fread( pVS, sizeof( char ), iFileSizeVS, fHandleVS );
	pVS[ iFileSizeVS ] = 0x00;
	fclose( fHandleVS );

	// Read encodeps.glsl
	fread( pPS, sizeof( char ), iFileSizePS, fHandlePS );
	pPS[ iFileSizePS ] = 0x00;
	fclose( fHandlePS );

    // Replace [@2] with PSDEFORM (in encodevs.glsl and encodeps.glsl)
	char *szOut = NULL;
	ReplaceString( 1, pVS, "[@2]", m_pManager->m_szPSDeform, &szOut );
	SAFE_DELETE_ARRAY( pVS );
	pVS = szOut;

	szOut = NULL;
	ReplaceString( 1, pPS, "[@2]", m_pManager->m_szPSDeform, &szOut );
	SAFE_DELETE_ARRAY( pPS );
	pPS = szOut;

	// Replace tokens with fPSDeform (in encodeps.glsl)
	if( !m_bUsePxDef )
	{
		szOut = NULL;
		ReplaceString( 1, pPS, "[@3]", " ", &szOut );
		SAFE_DELETE_ARRAY( pPS );
		pPS = szOut;

		szOut = NULL;
		ReplaceString( 1, pPS, "[@6]", " ", &szOut );
		SAFE_DELETE_ARRAY( pPS );
		pPS = szOut;
	}
	else
	{
		szOut = NULL;
		ReplaceString( 1, pPS, "[@3]", m_szDefPS, &szOut );
		SAFE_DELETE_ARRAY( pPS );
		pPS = szOut;

		szOut = NULL;
		ReplaceString( 1, pPS, "[@6]", 
			"  DefOut = fPSDeform( DefOut );\n", &szOut );
		SAFE_DELETE_ARRAY( pPS );
		pPS = szOut;
	}

	// Replace tokens with fPostVSDeform (in encodevs.glsl)
	if( !m_bUsePostDefVS )
	{
		szOut = NULL;
		ReplaceString( 1, pVS, "[@14]", " ", &szOut );
		SAFE_DELETE_ARRAY( pVS );
		pVS = szOut;

		szOut = NULL;
		ReplaceString( 1, pVS, "[@12]", " ", &szOut );
		SAFE_DELETE_ARRAY( pVS );
		pVS = szOut;
	}
	else
	{
		szOut = NULL;
		ReplaceString( 1, pVS, "[@14]", m_szPostDefVS, &szOut );
		SAFE_DELETE_ARRAY( pVS );
		pVS = szOut;

		szOut = NULL;
		ReplaceString( 1, pVS, "[@12]",
			"  Out = fPostVSDeform( Out );\n", &szOut );
		SAFE_DELETE_ARRAY( pVS );
		pVS = szOut;
	}

	// Return contents of Out
	szOut = NULL;
	ReplaceString( 1, pVS, "[@15]", m_pManager->m_szReturnPSDeform, &szOut );
	SAFE_DELETE_ARRAY( pVS );
	pVS = szOut;

	// Look for INTSEMANTIC_VERTEXID vertex semantic
	VSSEMANTIC iVtIDSemantic;
	if( !GetSemantic( INTSEMANTIC_VERTEXID, iVtIDSemantic ) )
		return -2;

	// Replace [@4] with structure member associated with 
	// INTSEMANTIC_VERTEXID (in encodevs.glsl)
	char *szReplace = new char[32];
    if( szReplace == NULL )
		return -1;
	szReplace[0] = 0x00;

	GetParamStrGLSL( iVtIDSemantic, szReplace );

	szOut = NULL;
	ReplaceString( 1, pVS, "[@4]", szReplace, &szOut );
	SAFE_DELETE_ARRAY( pVS );
	pVS = szOut;

	// Replace [@5] with remaining code of vertex shader encoder
	SAFE_DELETE_ARRAY( szReplace );
	szReplace = new char[256 * 80];
	if( szReplace == NULL )
		return -1;
	szReplace[0] = 0x00;

	// Add parameters of PSDEFORM (in encodevs.glsl)
	int iCt = 0;
	while( m_pManager->m_pBindings[iCt].iVSSemantic != VSSEMANTIC_END )
	{
		char szAtt    [32] = {0};
		char szAttGLSL[32] = {0};

		switch( m_pManager->m_pBindings[iCt].iVSSemantic )
		{
		case VSSEMANTIC_NORMAL: 
			strcpy( szAtt, "vNormal" ); 
			strcpy( szAttGLSL, "gl_Normal" ); 
			break;
		case VSSEMANTIC_COLOR0: 
			strcpy( szAtt, "vColor0" ); 
			strcpy( szAttGLSL, "gl_Color" ); 
			break;
		case VSSEMANTIC_COLOR1: 
			strcpy( szAtt, "vColor1" );
			strcpy( szAttGLSL, "gl_SecondaryColor" ); 
			break;
		case VSSEMANTIC_TEXCOORD0: 
			strcpy( szAtt, "vTex0" );
			strcpy( szAttGLSL, "gl_MultiTexCoord0" );
			break;
		case VSSEMANTIC_TEXCOORD1: 
			strcpy( szAtt, "vTex1" );
			strcpy( szAttGLSL, "gl_MultiTexCoord1" );
			break;
		case VSSEMANTIC_TEXCOORD2: 
			strcpy( szAtt, "vTex2" );
			strcpy( szAttGLSL, "gl_MultiTexCoord2" );
			break;
		case VSSEMANTIC_TEXCOORD3: 
			strcpy( szAtt, "vTex3" );
			strcpy( szAttGLSL, "gl_MultiTexCoord3" );
			break;
		case VSSEMANTIC_TEXCOORD4: 
			strcpy( szAtt, "vTex4" );
			strcpy( szAttGLSL, "gl_MultiTexCoord4" );
			break;
		case VSSEMANTIC_TEXCOORD5: 
			strcpy( szAtt, "vTex5" );
			strcpy( szAttGLSL, "gl_MultiTexCoord5" );
			break;
		case VSSEMANTIC_TEXCOORD6: 
			strcpy( szAtt, "vTex6" );
			strcpy( szAttGLSL, "gl_MultiTexCoord6" );
			break;
		case VSSEMANTIC_TEXCOORD7: 
			strcpy( szAtt, "vTex7" );
			strcpy( szAttGLSL, "gl_MultiTexCoord7" );
			break;
		}

		strcat( szReplace, "  Out." );
		strcat( szReplace, szAtt );
		strcat( szReplace, " = " );
		strcat( szReplace, szAttGLSL );
		strcat( szReplace, ";\n" );

		iCt++;
	}

	szOut = NULL;
	ReplaceString( 1, pVS, "[@13]", szReplace, &szOut );
	SAFE_DELETE_ARRAY( pVS );
	pVS = szOut;

	szReplace[0] = 0x00;

/*
	//
	// Create encoder effect for non-indexed geometry
	//
	if( m_pManager->m_bAttUserDefNI )
	{
		// First we make a copy of pFx
		char *pFx2 = new char[ (int) strlen( pFx ) + 1 ];
		if( pFx2 == NULL )
			return -1;
		pFx2[0] = 0x00;
		strcpy( pFx2, pFx );

		// Using normal?
		if( m_pManager->m_bAttNormal || m_pManager->m_bAttTBN )
		{
			strcat( szReplace, "  Out.vTN = 0;\n\n" );
		}

		// Using TBN frame?
		if( m_pManager->m_bAttTBN )
		{
			strcat( szReplace, "  Out.vTT = 0;\n"
							   "  Out.vTB = 0;\n\n" );
		}

		// Using curvature?
		if( m_pManager->m_bAttCurv )
		{
			strcat( szReplace, "  Out.vP1K1  = 0;\n"
							   "  Out.vP2K2  = 0;\n" );
		}

		// Using 2nd fundamental form?
		if( m_pManager->m_bAttTensor2 )
		{
			strcat( szReplace, "  Out.vTensor2 = 0;\n"	);
		}

		// Using tensor of curvature derivative?
		if( m_pManager->m_bAttTensor3 )
		{
			strcat( szReplace, "  Out.vTensor3 = 0;\n" );
		}

		szOut = NULL;
		ReplaceString( 1, pFx2, "[@5]", szReplace, &szOut );
		SAFE_DELETE_ARRAY( pFx2 );
		pFx2 = szOut;

		szReplace[0] = 0x00;

		// PS_OUT returns only one vColor
		szOut = NULL;
		ReplaceString( 1, pFx2, "[@10]", "1", &szOut );
		SAFE_DELETE_ARRAY( pFx2 );
		pFx2 = szOut;

		szOut = NULL;
		ReplaceString( 1, pFx2, "[@7]", 
			"  Out.vColor[0] = DefOut.[@8];\n", &szOut );
		SAFE_DELETE_ARRAY( pFx2 );
		pFx2 = szOut;

		// Look for INTSEMANTIC_USERDEFNI vertex semantic
		VSSEMANTIC iUserDefSemantic;
		if( !GetSemantic( INTSEMANTIC_USERDEFNI, iUserDefSemantic ) )
			return -2;

		// Replace [@8] with structure member associated with 
		// INTSEMANTIC_USERDEFNI
		char szAtt[32] = {0};
		GetParamStr( iUserDefSemantic, szAtt );

		szOut = NULL;
		ReplaceString( 1, pFx2, "[@8]", szAtt, &szOut );
		SAFE_DELETE_ARRAY( pFx2 );
		pFx2 = szOut;

		SAFE_RELEASE( m_pEncodeNIFx );


		// DEBUG: Write shader to file
		fHandle = fopen( "enc_out.fx", "wb" );
		if( fHandle == NULL )
			return -1;

		fwrite( pFx2, sizeof( char ), strlen( pFx2 ), fHandle );

		fclose( fHandle );


		// Load effect from file in memory
		LPD3DXBUFFER pd3dBuffer = NULL;
		hrResult = D3DXCreateEffect( m_pd3dDevice, pFx2,
			(UINT) strlen( pFx2 ), NULL, NULL, dwShaderFlags, NULL, 
			&m_pEncodeNIFx, &pd3dBuffer );

		// If there are errors, print them
		if( pd3dBuffer )
			fprintf( stderr, (char*) pd3dBuffer->GetBufferPointer( ) );

		if( FAILED( hrResult ) )
			return -1;

		// Get handle to uniform parameters in the effect
		m_hEncodeNIFxIdxC = m_pEncodeNIFx->GetParameterByName( 
			NULL, "g_fVtIdxC" );
		m_hEncodeNIFxWVP  = m_pEncodeNIFx->GetParameterByName( 
			NULL, "g_mWVP" );
		m_hEncodeNIFxITW  = m_pEncodeNIFx->GetParameterByName( 
			NULL, "g_mITW" );

		SAFE_DELETE_ARRAY( pFx2 );
	}
*/
	SAFE_DELETE_ARRAY( szReplace );

	//
	// Create encoder effect for indexed geometry
	//

	// TODO: Make as many copies of pFx as there are render passes
	//       Then, for each copy:
	//       - Replace [@7] with source code to encode data.

	glUseProgramObjectARB( 0 );

	for( int iCt = 0; iCt < MAX_RENDER_PASSES; iCt++ )
	{
		if( m_hEncodeIPO[iCt] ) 
		{
			glDeleteObjectARB( m_hEncodeIPO[iCt] );
			m_hEncodeIPO[iCt] = 0;
		}
	}

	int iRP   = 0; // Current render pass
	int iRT   = 0; // Current render target
	int iComp = 0; // Current color component

	const int iTempSz     = 256;
	char chComp [5]       = "rgba";
	char szTemp [iTempSz] = {0};
	char szTemp2[iTempSz] = {0};

	g_bAttNormal = g_bAttTBN = g_bAttCurv =	false;
	g_bAttTensor2 = g_bAttTensor3 = false;

	//
	// Create an effect for each render pass
	//

	// Make a copy of pPS for the 1st render pass
	char *pPS2 = new char[ (int) strlen( pPS ) + 1 ];
	if( pPS2 == NULL )
		return -1;
	pPS2[0] = 0x00;
	strcpy( pPS2, pPS );

	szReplace = new char[256 * 80];
	if( szReplace == NULL )
		return -1;
	szReplace[0] = 0x00;

	int iVtSamplers = 1;

	// ATTTYPE_DEPTH
	if( m_pManager->m_bAttDepth )
	{
		// Components must fit in the same render pass
		if( 1 > ( m_pManager->m_iNumMRTs - 
			( iRT % m_pManager->m_iNumMRTs ) ) * 4 - iComp ||
			iVtSamplers+0 > 4 )
		{ 
			if( !CreatePSEncoder( iRP, iRT, iComp, &pVS, &pPS2, szReplace ) )
				return -1;

			// Make a copy of pPS for the next render pass
			strcpy( pPS2, pPS );
			iComp = 0; iRT = 0; iRP++; 
			iVtSamplers = 1;

			g_bAttNormal = g_bAttTBN = g_bAttCurv =	false;
			g_bAttTensor2 = g_bAttTensor3 = false;
		}

		// Depth
		if( iComp == 4 ) { iComp = 0; iRT++; };
		_snprintf( szTemp, iTempSz, "  gl_FragData[%d].", iRT );
		strcat ( szReplace, szTemp );
		strncat( szReplace, &chComp[iComp], 1 );	
		strcat ( szReplace, " = DefOut.vTPos.z / DefOut.vTPos.w;\n" );
//		strcat ( szReplace, " = DefOut.vTPos.z;\n" );
		iComp++;
	}

	// ATTTYPE_TEXCOORD
	if( m_pManager->m_bAttTexCoord )
	{
		// Components must fit in the same render pass
		if( 2 > ( m_pManager->m_iNumMRTs - 
			( iRT % m_pManager->m_iNumMRTs ) ) * 4 - iComp ||
			iVtSamplers+0 > 4 )
		{ 
			if( !CreatePSEncoder( iRP, iRT, iComp, &pVS, &pPS2, szReplace ) )
				return -1;

			// Make a copy of pPS for the next render pass
			strcpy( pPS2, pPS );
			iComp = 0; iRT = 0; iRP++; 
			iVtSamplers = 1;

			g_bAttNormal = g_bAttTBN = g_bAttCurv =	false;
			g_bAttTensor2 = g_bAttTensor3 = false;
		}

		VSSEMANTIC iTexCoordSemantic;
		GetSemantic( INTSEMANTIC_TEXCOORD, iTexCoordSemantic );
		GetParamStr( iTexCoordSemantic, szTemp2 );

		// U
		if( iComp == 4 ) { iComp = 0; iRT++; };
		_snprintf( szTemp, iTempSz, "  gl_FragData[%d].", iRT );
		strcat   ( szReplace, szTemp );
		strncat  ( szReplace, &chComp[iComp], 1 );	

		_snprintf( szTemp, iTempSz, " = DefOut.%s.r;\n", szTemp2 );
		strcat   ( szReplace, szTemp );
		iComp++; 

		// V
		if( iComp == 4 ) { iComp = 0; iRT++; };
		_snprintf( szTemp, iTempSz, "  gl_FragData[%d].", iRT );
		strcat   ( szReplace, szTemp );
		strncat  ( szReplace, &chComp[iComp], 1 );	

		_snprintf( szTemp, iTempSz, " = DefOut.%s.g;\n", szTemp2 );
		strcat   ( szReplace, szTemp );
		iComp++; 
	}

	// ATTTYPE_TBN
	if( m_pManager->m_bAttTBN )
	{
		// Components must fit in the same render pass
		if( 6 > ( m_pManager->m_iNumMRTs - 
			( iRT % m_pManager->m_iNumMRTs ) ) * 4 - iComp ||
			iVtSamplers+2 > 4 )
		{
			if( !CreatePSEncoder( iRP, iRT, iComp, &pVS, &pPS2, szReplace ) )
				return -1;

			// Make a copy of pPS for the next render pass
			strcpy( pPS2, pPS );
			iComp = 0; iRT = 0; iRP++; 
			iVtSamplers = 1;

			g_bAttNormal = g_bAttTBN = g_bAttCurv =	false;
			g_bAttTensor2 = g_bAttTensor3 = false;
		}

		iVtSamplers += 2;
		g_bAttTBN = true;

		// T.x
		if( iComp == 4 ) { iComp = 0; iRT++; };
		_snprintf( szTemp, iTempSz, "  gl_FragData[%d].", iRT );
		strcat   ( szReplace, szTemp );
		strncat  ( szReplace, &chComp[iComp], 1 );

		strcat   ( szReplace, " = DefOut.vTT.r;\n" );
		iComp++; 

		// T.y
		if( iComp == 4 ) { iComp = 0; iRT++; };
		_snprintf( szTemp, iTempSz, "  gl_FragData[%d].", iRT );
		strcat   ( szReplace, szTemp );
		strncat  ( szReplace, &chComp[iComp], 1 );

		strcat   ( szReplace, " = DefOut.vTT.g;\n" );
		iComp++; 

		// T.z
		if( iComp == 4 ) { iComp = 0; iRT++; };
		_snprintf( szTemp, iTempSz, "  gl_FragData[%d].", iRT );
		strcat   ( szReplace, szTemp );
		strncat  ( szReplace, &chComp[iComp], 1 );

		strcat   ( szReplace, " = DefOut.vTT.b;\n" );
		iComp++; 

		// B.x
		if( iComp == 4 ) { iComp = 0; iRT++; };
		_snprintf( szTemp, iTempSz, "  gl_FragData[%d].", iRT );
		strcat   ( szReplace, szTemp );
		strncat  ( szReplace, &chComp[iComp], 1 );

		strcat   ( szReplace, " = DefOut.vTB.r;\n" );
		iComp++; 

		// B.y
		if( iComp == 4 ) { iComp = 0; iRT++; };
		_snprintf( szTemp, iTempSz, "  gl_FragData[%d].", iRT );
		strcat   ( szReplace, szTemp );
		strncat  ( szReplace, &chComp[iComp], 1 );

		strcat   ( szReplace, " = DefOut.vTB.g;\n" );
		iComp++; 

		// B.z
		if( iComp == 4 ) { iComp = 0; iRT++; };
		_snprintf( szTemp, iTempSz, "  gl_FragData[%d].", iRT );
		strcat   ( szReplace, szTemp );
		strncat  ( szReplace, &chComp[iComp], 1 );

		strcat   ( szReplace, " = DefOut.vTB.b;\n" );
		iComp++; 
	}

	// ATTTYPE_USERDEFI
	if( m_pManager->m_bAttUserDefI )
	{
		// Components must fit in the same render pass
		if( 4 > ( m_pManager->m_iNumMRTs - 
			( iRT % m_pManager->m_iNumMRTs ) ) * 4 - iComp ||
			iVtSamplers+0 > 4 )
		{ 
			if( !CreatePSEncoder( iRP, iRT, iComp, &pVS, &pPS2, szReplace ) )
				return -1;

			// Make a copy of pPS for the next render pass
			strcpy( pPS2, pPS );

			iComp = 0; iRT = 0; iRP++; 
			iVtSamplers = 1;

			g_bAttNormal = g_bAttTBN = g_bAttCurv =	false;
			g_bAttTensor2 = g_bAttTensor3 = false;
		}

		VSSEMANTIC iUserDefSemantic;
		GetSemantic( INTSEMANTIC_USERDEFI, iUserDefSemantic );
		GetParamStr( iUserDefSemantic, szTemp2 );

		// r
		if( iComp == 4 ) { iComp = 0; iRT++; };
		_snprintf( szTemp, iTempSz, "  gl_FragData[%d].", iRT );
		strcat   ( szReplace, szTemp );
		strncat  ( szReplace, &chComp[iComp], 1 );	

		_snprintf( szTemp, iTempSz, " = DefOut.%s.r;\n", szTemp2 );
//		_snprintf( szTemp, iTempSz, " = 1.0;\n" );
		strcat   ( szReplace, szTemp );
		iComp++; 

		// g
		if( iComp == 4 ) { iComp = 0; iRT++; };
		_snprintf( szTemp, iTempSz, "  gl_FragData[%d].", iRT );
		strcat   ( szReplace, szTemp );
		strncat  ( szReplace, &chComp[iComp], 1 );	

		_snprintf( szTemp, iTempSz, " = DefOut.%s.g;\n", szTemp2 );
//		_snprintf( szTemp, iTempSz, " = 1.0;\n" );
		strcat   ( szReplace, szTemp );
		iComp++; 

		// b
		if( iComp == 4 ) { iComp = 0; iRT++; };
		_snprintf( szTemp, iTempSz, "  gl_FragData[%d].", iRT );
		strcat   ( szReplace, szTemp );
		strncat  ( szReplace, &chComp[iComp], 1 );	

		_snprintf( szTemp, iTempSz, " = DefOut.%s.b;\n", szTemp2 );
//		_snprintf( szTemp, iTempSz, " = 1.0;\n" );
		strcat   ( szReplace, szTemp );
		iComp++; 

		// a
		if( iComp == 4 ) { iComp = 0; iRT++; };
		_snprintf( szTemp, iTempSz, "  gl_FragData[%d].", iRT );
		strcat   ( szReplace, szTemp );
		strncat  ( szReplace, &chComp[iComp], 1 );	

		_snprintf( szTemp, iTempSz, " = DefOut.%s.a;\n", szTemp2 );
//		_snprintf( szTemp, iTempSz, " = 1.0;\n" );
		strcat   ( szReplace, szTemp );
		iComp++; 
	}

	// ATTTYPE_NORMAL
	if( m_pManager->m_bAttNormal )
	{
		// Components must fit in the same render pass
		if( 3 > ( m_pManager->m_iNumMRTs - 
			( iRT % m_pManager->m_iNumMRTs ) ) * 4 - iComp ||
			iVtSamplers+1 > 4 )
		{ 
			if( !CreatePSEncoder( iRP, iRT, iComp, &pVS, &pPS2, szReplace ) )
				return -1;

			// Make a copy of pFx for the next render pass
			strcpy( pPS2, pPS );

			iComp = 0; iRT = 0; iRP++; 
			iVtSamplers = 1;

			g_bAttNormal = g_bAttTBN = g_bAttCurv =	false;
			g_bAttTensor2 = g_bAttTensor3 = false;
		}

		iVtSamplers++;
		g_bAttNormal = true;

		// N.x
		if( iComp == 4 ) { iComp = 0; iRT++; };
		_snprintf( szTemp, iTempSz, "  gl_FragData[%d].", iRT );
		strcat   ( szReplace, szTemp );
		strncat  ( szReplace, &chComp[iComp], 1 );

		strcat   ( szReplace, " = DefOut.vTN.r;\n" );
		iComp++; 

		// N.y
		if( iComp == 4 ) { iComp = 0; iRT++; };
		_snprintf( szTemp, iTempSz, "  gl_FragData[%d].", iRT );
		strcat   ( szReplace, szTemp );
		strncat  ( szReplace, &chComp[iComp], 1 );

		strcat   ( szReplace, " = DefOut.vTN.g;\n" );
		iComp++; 

		// N.z
		if( iComp == 4 ) { iComp = 0; iRT++; };
		_snprintf( szTemp, iTempSz, "  gl_FragData[%d].", iRT );
		strcat   ( szReplace, szTemp );
		strncat  ( szReplace, &chComp[iComp], 1 );

		strcat   ( szReplace, " = DefOut.vTN.b;\n" );
		iComp++; 
	}

	// ATTTYPE_CURV
	if( m_pManager->m_bAttCurv )
	{
		// Components must fit in the same render pass
		if( 8 > ( m_pManager->m_iNumMRTs - 
			( iRT % m_pManager->m_iNumMRTs ) ) * 4 - iComp ||
			iVtSamplers+2 > 4 )
		{ 
			if( !CreatePSEncoder( iRP, iRT, iComp, &pVS, &pPS2, szReplace ) )
				return -1;

			// Make a copy of pPS for the next render pass
			strcpy( pPS2, pPS );

			iComp = 0; iRT = 0; iRP++; 
			iVtSamplers = 1;

			g_bAttNormal = g_bAttTBN = g_bAttCurv =	false;
			g_bAttTensor2 = g_bAttTensor3 = false;
		}

		iVtSamplers += 2;
		g_bAttCurv = true;

		// P1.x
		if( iComp == 4 ) { iComp = 0; iRT++; };
		_snprintf( szTemp, iTempSz, "  gl_FragData[%d].", iRT );
		strcat   ( szReplace, szTemp );
		strncat  ( szReplace, &chComp[iComp], 1 );

		strcat   ( szReplace, " = DefOut.vP1K1.r;\n" );
		iComp++; 

		// P1.y
		if( iComp == 4 ) { iComp = 0; iRT++; };
		_snprintf( szTemp, iTempSz, "  gl_FragData[%d].", iRT );
		strcat   ( szReplace, szTemp );
		strncat  ( szReplace, &chComp[iComp], 1 );

		strcat   ( szReplace, " = DefOut.vP1K1.g;\n" );
		iComp++; 

		// P1.z
		if( iComp == 4 ) { iComp = 0; iRT++; };
		_snprintf( szTemp, iTempSz, "  gl_FragData[%d].", iRT );
		strcat   ( szReplace, szTemp );
		strncat  ( szReplace, &chComp[iComp], 1 );

		strcat   ( szReplace, " = DefOut.vP1K1.b;\n" );
		iComp++; 

		// k1
		if( iComp == 4 ) { iComp = 0; iRT++; };
		_snprintf( szTemp, iTempSz, "  gl_FragData[%d].", iRT );
		strcat   ( szReplace, szTemp );
		strncat  ( szReplace, &chComp[iComp], 1 );

		strcat   ( szReplace, " = DefOut.vP1K1.a;\n" );
		iComp++; 

		// P2.x
		if( iComp == 4 ) { iComp = 0; iRT++; };
		_snprintf( szTemp, iTempSz, "  gl_FragData[%d].", iRT );
		strcat   ( szReplace, szTemp );
		strncat  ( szReplace, &chComp[iComp], 1 );

		strcat   ( szReplace, " = DefOut.vP2K2.r;\n" );
		iComp++; 

		// P2.y
		if( iComp == 4 ) { iComp = 0; iRT++; };
		_snprintf( szTemp, iTempSz, "  gl_FragData[%d].", iRT );
		strcat   ( szReplace, szTemp );
		strncat  ( szReplace, &chComp[iComp], 1 );

		strcat   ( szReplace, " = DefOut.vP2K2.g;\n" );
		iComp++; 

		// P2.z
		if( iComp == 4 ) { iComp = 0; iRT++; };
		_snprintf( szTemp, iTempSz, "  gl_FragData[%d].", iRT );
		strcat   ( szReplace, szTemp );
		strncat  ( szReplace, &chComp[iComp], 1 );

		strcat   ( szReplace, " = DefOut.vP2K2.b;\n" );
		iComp++; 

		// k2
		if( iComp == 4 ) { iComp = 0; iRT++; };
		_snprintf( szTemp, iTempSz, "  gl_FragData[%d].", iRT );
		strcat   ( szReplace, szTemp );
		strncat  ( szReplace, &chComp[iComp], 1 );

		strcat   ( szReplace, " = DefOut.vP2K2.a;\n" );
		iComp++; 
	}

	// ATTTYPE_TENSOR2
	if( m_pManager->m_bAttTensor2 )
	{
		// Components must fit in the same render pass
		if( 3 > ( m_pManager->m_iNumMRTs - 
			( iRT % m_pManager->m_iNumMRTs ) ) * 4 - iComp ||
			iVtSamplers+1 > 4 )
		{ 
			if( !CreatePSEncoder( iRP, iRT, iComp, &pVS, &pPS2, szReplace ) )
				return -1;

			// Make a copy of pPS for the next render pass
			strcpy( pPS2, pPS );

			iComp = 0; iRT = 0; iRP++; 
			iVtSamplers = 1;

			g_bAttNormal = g_bAttTBN = g_bAttCurv =	false;
			g_bAttTensor2 = g_bAttTensor3 = false;
		}

		iVtSamplers++;
		g_bAttTensor2 = true;

		// e
		if( iComp == 4 ) { iComp = 0; iRT++; };
		_snprintf( szTemp, iTempSz, "  gl_FragData[%d].", iRT );
		strcat   ( szReplace, szTemp );
		strncat  ( szReplace, &chComp[iComp], 1 );

		strcat   ( szReplace, " = DefOut.vTensor2.r;\n" );
		iComp++; 

		// f
		if( iComp == 4 ) { iComp = 0; iRT++; };
		_snprintf( szTemp, iTempSz, "  gl_FragData[%d].", iRT );
		strcat   ( szReplace, szTemp );
		strncat  ( szReplace, &chComp[iComp], 1 );

		strcat   ( szReplace, " = DefOut.vTensor2.g;\n" );
		iComp++; 

		// g
		if( iComp == 4 ) { iComp = 0; iRT++; };
		_snprintf( szTemp, iTempSz, "  gl_FragData[%d].", iRT );
		strcat   ( szReplace, szTemp );
		strncat  ( szReplace, &chComp[iComp], 1 );

		strcat   ( szReplace, " = DefOut.vTensor2.b;\n" );
		iComp++; 
	}

	// ATTTYPE_TENSOR3
	if( m_pManager->m_bAttTensor3 )
	{
		// Components must fit in the same render pass
		if( 4 > ( m_pManager->m_iNumMRTs - 
			( iRT % m_pManager->m_iNumMRTs ) ) * 4 - iComp ||
			iVtSamplers+1 > 4 )
		{ 
			if( !CreatePSEncoder( iRP, iRT, iComp, &pVS, &pPS2, szReplace ) )
				return -1;

			// Make a copy of pPS for the next render pass
			strcpy( pPS2, pPS );

			iComp = 0; iRT = 0; iRP++; 
			iVtSamplers = 1;

			g_bAttNormal = g_bAttTBN = g_bAttCurv =	false;
			g_bAttTensor2 = g_bAttTensor3 = false;
		}

		iVtSamplers++;
		g_bAttTensor3 = true;

		// a
		if( iComp == 4 ) { iComp = 0; iRT++; };
		_snprintf( szTemp, iTempSz, "  gl_FragData[%d].", iRT );
		strcat   ( szReplace, szTemp );
		strncat  ( szReplace, &chComp[iComp], 1 );

		strcat   ( szReplace, " = DefOut.vTensor3.r;\n" );
		iComp++; 

		// b
		if( iComp == 4 ) { iComp = 0; iRT++; };
		_snprintf( szTemp, iTempSz, "  gl_FragData[%d].", iRT );
		strcat   ( szReplace, szTemp );
		strncat  ( szReplace, &chComp[iComp], 1 );

		strcat   ( szReplace, " = DefOut.vTensor3.g;\n" );
		iComp++; 

		// c
		if( iComp == 4 ) { iComp = 0; iRT++; };
		_snprintf( szTemp, iTempSz, "  gl_FragData[%d].", iRT );
		strcat   ( szReplace, szTemp );
		strncat  ( szReplace, &chComp[iComp], 1 );

		strcat   ( szReplace, " = DefOut.vTensor3.b;\n" );
		iComp++; 

		// d
		if( iComp == 4 ) { iComp = 0; iRT++; };
		_snprintf( szTemp, iTempSz, "  gl_FragData[%d].", iRT );
		strcat   ( szReplace, szTemp );
		strncat  ( szReplace, &chComp[iComp], 1 );

		strcat   ( szReplace, " = DefOut.vTensor3.a;\n" );
		iComp++; 
	}

	if( iComp == 0 && iRT )
		iRT--;
	else
	{
		// Fill color components not used
		while( iComp < 4 )
		{
			_snprintf( szTemp, iTempSz, "  gl_FragData[%d].", iRT );
			strcat   ( szReplace, szTemp );
			strncat  ( szReplace, &chComp[iComp], 1 ) ;	
			strcat   ( szReplace, " = 1.0f;\n" );

			iComp++;
		}
	}

	szOut = NULL;
	ReplaceString( 1, pPS2, "[@7]", szReplace, &szOut );
	SAFE_DELETE_ARRAY( pPS2 );
	pPS2 = szOut;

	szReplace[0] = 0x00;

	// Set DefOut Input
	szOut = NULL;
	ReplaceString( 1, pPS2, "[@16]", m_pManager->m_szInputPSDeform, &szOut );
	SAFE_DELETE_ARRAY( pPS2 );
	pPS2 = szOut;

	// Using normal?
	if( g_bAttNormal || g_bAttTBN )
	{
		strcat( szReplace, 
			"  vec3 vN = texture2D( s1, vVtUV ).xyz;\n"
			//			"  Out.vTN = mul( float4(vN, 0), g_mITW );\n\n" );
			"  Out.vTN = vN;\n\n" );
	}
	else
	{
		if( m_pManager->m_bAttNormal || m_pManager->m_bAttTBN )
		{
			strcat( szReplace, "  Out.vTN = 0;\n\n" );
		}
	}

	// Using TBN frame?
	if( g_bAttTBN )
	{
		strcat( szReplace, 
			"  vec4 vT = texture2D( s3, vVtUV );\n"
			"  vec3 vB = cross( vT.xyz, vN ) * vT.w;\n"
			//			"  Out.vTT = mul( float4(vT.xyz,0), g_mITW );\n"
			//			"  Out.vTB = mul( float4(vB, 0), g_mITW );\n\n" );
			"  Out.vTT = vT.xyz;\n"
			"  Out.vTB = vB;\n\n" );
	}
	else if( m_pManager->m_bAttTBN )
	{
			strcat( szReplace, "  Out.vTT = 0;\n"
							   "  Out.vTB = 0;\n\n" );
	}

	// Using curvature?
	if( g_bAttCurv )
	{
		strcat( szReplace,
			"  vec4 vP1K1 = texture2D( s4, vVtUV );\n"
			"  vec4 vP2K2 = texture2D( s5, vVtUV );\n"
			"  Out.vP1K1  = vP1K1;\n"
			"  Out.vP2K2  = vP2K2;\n"
//			"  Out.vP1K1  = float4( mul( float4(vP1K1.xyz,0), g_mITW ).xyz, vP1K1.w );\n"
//			"  Out.vP2K2  = float4( mul( float4(vP2K2.xyz,0), g_mITW ).xyz, vP2K2.w );\n"
			);
	}
	else if( m_pManager->m_bAttCurv )
	{
		strcat( szReplace, "  Out.vP1K1 = 0;\n"
			               "  Out.vP2K2 = 0;\n" );
	}

	// Using 2nd fundamental form?
	if( g_bAttTensor2 )
	{
		strcat( szReplace,
			"  Out.vTensor2 = texture2D( s6, vVtUV );\n"	);
	}
	else if( m_pManager->m_bAttTensor2 )
	{
		strcat( szReplace, "  Out.vTensor2 = 0;\n" );
	}

	// Using tensor of curvature derivative?
	if( g_bAttTensor3 )
	{
		strcat( szReplace,
			"  Out.vTensor3 = texture2D( s7, vVtUV );\n" );
	}
	else if( m_pManager->m_bAttTensor3 )
	{
		strcat( szReplace, "  Out.vTensor3 = 0;\n" );
	}

	szOut = NULL;
	ReplaceString( 1, pVS, "[@5]", szReplace, &szOut );
	SAFE_DELETE_ARRAY( pVS );
	pVS = szOut;


	// DEBUG: Write vertex shader to file
	fHandleVS = fopen( "out_encodevs.glsl", "wb" );
	if( fHandleVS == NULL )
		return -1;
	fwrite( pVS, sizeof( char ), strlen( pVS ), fHandleVS );
	fclose( fHandleVS );

	// DEBUG: Write pixel shader to file
	fHandlePS = fopen( "out_encodeps.glsl", "wb" );
	if( fHandlePS == NULL )
		return -1;
	fwrite( pPS2, sizeof( char ), strlen( pPS2 ), fHandlePS );
	fclose( fHandlePS );
/*
	//
	// Debug (read shaders again)
	//

	fHandleVS = fopen( "out_encodevs.glsl", "rb" );
	fseek( fHandleVS, 0, SEEK_END );
	iFileSizeVS = ftell( fHandleVS );
	fseek( fHandleVS, 0, SEEK_SET );
	pVS[0] = NULL;
	fread( pVS, sizeof( char ), iFileSizeVS, fHandleVS );
	pVS[iFileSizeVS] = NULL;
	fclose( fHandleVS );

	fHandlePS = fopen( "out_encodeps.glsl", "rb" );
	fseek( fHandlePS, 0, SEEK_END );
	iFileSizePS = ftell( fHandlePS );
	fseek( fHandlePS, 0, SEEK_SET );
	pPS2[0] = NULL;
	fread( pPS2, sizeof( char ), iFileSizePS, fHandlePS );
	pPS2[iFileSizePS] = NULL;
	fclose( fHandlePS );
*/
	//
	// Create encoding (indexed geometry) effect from string
	//

	// Create program object
	m_hEncodeIPO[iRP] = glCreateProgramObjectARB( );

	// Create shader objects
	GLhandleARB hVSSO = glCreateShaderObjectARB( GL_VERTEX_SHADER_ARB   );
	GLhandleARB hPSSO = glCreateShaderObjectARB( GL_FRAGMENT_SHADER_ARB );
	glShaderSourceARB( hVSSO, 1, (const GLcharARB**) &pVS, NULL );
	glShaderSourceARB( hPSSO, 1, (const GLcharARB**) &pPS2, NULL );
	glCompileShaderARB( hVSSO );
	glCompileShaderARB( hPSSO );

	// Check compilation status
	GLint iResult = 0;
	glGetObjectParameterivARB( hVSSO, GL_OBJECT_COMPILE_STATUS_ARB, &iResult );
	if( !iResult )
	{
		//		glGetInfoLogARB( hVSS0 );
		return false;
	}
	glGetObjectParameterivARB( hPSSO, GL_OBJECT_COMPILE_STATUS_ARB, &iResult );
	if( !iResult )
	{
		//		glGetInfoLogARB( hPSS0 );
		return false;
	}

	// Attach shaders to program
	glAttachObjectARB( m_hEncodeIPO[iRP], hVSSO );
	glAttachObjectARB( m_hEncodeIPO[iRP], hPSSO );
	// Flag shaders for deletion when they are no longer referenced
	if( hVSSO ) { glDeleteObjectARB( hVSSO ); hVSSO = 0; }
	if( hPSSO ) { glDeleteObjectARB( hPSSO ); hPSSO = 0; }

	glLinkProgramARB( m_hEncodeIPO[iRP] );

	// Check linkage status
	glGetObjectParameterivARB( m_hEncodeIPO[iRP], GL_OBJECT_LINK_STATUS_ARB, 
							   &iResult );

	if( !iResult )
	{
//		char infobuffer[1000];
//		int infobufferlen = 0;
//		glGetInfoLogARB(m_hEncodeIPO[iRP], 999, &infobufferlen, infobuffer);
//		infobuffer[infobufferlen] = 0;

		//		glGetInfoLogARB( m_hEncodeIPO[iRP] );
		return false;
	}

	// Use vertex-level deformation shader
	glUseProgramObjectARB( m_hEncodeIPO[iRP] );

	// Get handlers to uniform parameters
	m_iEncodeIPOVtMapSize[iRP] = glGetUniformLocationARB( m_hEncodeIPO[iRP], 
														  "g_iVtMapSize" );

	// Disable program object
	glUseProgramObjectARB( 0 );

	SAFE_DELETE_ARRAY( pPS2 );

	// TODO: New render pass here

	SAFE_DELETE_ARRAY( pVS );
	SAFE_DELETE_ARRAY( pPS );
	SAFE_DELETE_ARRAY( szReplace );

	{
		//
		// Load render shader
		//

		// First we read the rendervs.glsl file
		FILE *fHandle = fopen( "rendervs.glsl", "rb" );
		if( fHandle == NULL )
			return -1;

		// Get file size
		fseek( fHandle, 0, SEEK_END );
		int iFileSize = ftell( fHandle );
		fseek( fHandle, 0, SEEK_SET );

		// Allocate memory for render shader
		char *pVS = new char[ iFileSize + 128 ];
		if( pVS == NULL )
			return -1;
		pVS[0] = 0x00;

		// Read file
		fread( pVS, sizeof( char ), iFileSize, fHandle );
		pVS[ iFileSize ] = 0x00;

		// Look for INTSEMANTIC_VERTEXID vertex semantic
		int iCt = 0;
		VSSEMANTIC iVtIDSemantic;
		if( !GetSemantic( INTSEMANTIC_VERTEXID, iVtIDSemantic ) )
			return -2;

		// Replace occurrence of [@1] with string from GetParamStrGLSL
		char *szReplace = new char[32];
		if( szReplace == NULL )
			return -1;

		GetParamStrGLSL( iVtIDSemantic, szReplace );

		char *szOut;
		ReplaceString( 1, pVS, "[@1]", szReplace, &szOut );
		SAFE_DELETE_ARRAY( pVS );
		pVS = szOut;

		// Create program object
		m_hRenderPO = glCreateProgramObjectARB( );

		// Create shader objects
		GLhandleARB hSO = glCreateShaderObjectARB( GL_VERTEX_SHADER_ARB );
		glShaderSourceARB( hSO, 1, (const GLcharARB**) &pVS, NULL );
		glCompileShaderARB( hSO );

		// Check compilation status
		GLint iResult = 0;
		glGetObjectParameterivARB( hSO, GL_OBJECT_COMPILE_STATUS_ARB, &iResult );
		if( !iResult )
		{
			//		glGetInfoLogARB( hPSS0 );
			return -1;
		}

		// Attach shaders to program
		glAttachObjectARB( m_hRenderPO, hSO );
		// Flag shaders for deletion when they are no longer referenced
		if( hSO ) { glDeleteObjectARB( hSO ); hSO = 0; }

		glLinkProgramARB( m_hRenderPO );

		// Check linkage status
		glGetObjectParameterivARB( m_hRenderPO, GL_OBJECT_LINK_STATUS_ARB, &iResult );
		if( !iResult )
		{
			//		glGetInfoLogARB( m_hVtDefPO );
			return -1;
		}

		// Use face TBN shader
		glUseProgramObjectARB( m_hRenderPO );

		// Get handlers to uniform parameters
		m_iRenderPOVtMapSize = glGetUniformLocationARB( m_hRenderPO, "g_iVtMapSize" );

		// Disable program object
		glUseProgramObjectARB( 0 );

		SAFE_DELETE_ARRAY( szReplace );
		SAFE_DELETE_ARRAY( pVS );
	}
#endif

	return 0;
} // CIntObj::CreateShaders

////////////////////////////////////////////////////////////////////////////////
/// \brief Release the shaders created with CIntObj::CreateShaders.
///
/// \return 0 on success.
///         -1 on error.
////////////////////////////////////////////////////////////////////////////////
int CIntObj::DestroyShaders( )
{
#if defined(D3D9)
	SAFE_RELEASE( m_pVtDefFx );

	for( int iCt = 0; iCt < MAX_RENDER_PASSES; iCt++ )
		SAFE_RELEASE( m_pEncodeIFx[iCt] );

	SAFE_RELEASE( m_pEncodeNIFx );

	SAFE_RELEASE( m_pRenderFx );
#elif defined(OGL2)
	glUseProgramObjectARB( 0 );

	if( m_hVtDefPO )
	{
		glDeleteObjectARB( m_hVtDefPO );
		m_hVtDefPO = 0;
	}

	for( int iCt = 0; iCt < MAX_RENDER_PASSES; iCt++ )
	{
		if( m_hEncodeIPO[iCt] ) 
		{
			glDeleteObjectARB( m_hEncodeIPO[iCt] );
			m_hEncodeIPO[iCt] = 0;
		}
	}

	if( m_hEncodeNIPO )
	{
		glDeleteObjectARB( m_hEncodeNIPO );
		m_hEncodeNIPO = 0;
	}
	if( m_hRenderPO )
	{
		glDeleteObjectARB( m_hRenderPO   );
		m_hRenderPO = 0;
	}
#endif

	return 0;
} // CIntObj::DestroyShaders

////////////////////////////////////////////////////////////////////////////////
/// \brief Update the geometry of the model according to the vertex-level 
///        deformations, and re-compute the geometry attributes associated.
///
/// \return 0 on success.
///         -1 on error.
////////////////////////////////////////////////////////////////////////////////
int CIntObj::Update( )
{
	if( !m_bUsePreDefVS )
		return 0;

#if defined(D3D9)
	HRESULT hrResult = S_OK;

	// Set viewport for 1:1 pixel=texel=geometry mapping
	D3DVIEWPORT9 d3dPrevVp, d3dCurVp;
	m_pd3dDevice->GetViewport( &d3dPrevVp );
	d3dCurVp = d3dPrevVp;
	d3dCurVp.X = d3dCurVp.Y = 0;
	d3dCurVp.Width = d3dCurVp.Height = m_iVtAttTexSize;
	m_pd3dDevice->SetViewport( &d3dCurVp );

	// Set shader uniform variables
	D3DXMATRIX mProj;
	D3DXMatrixOrthoOffCenterLH( &mProj, 0.0f, (float) m_iVtAttTexSize, 
								(float) m_iVtAttTexSize, 0.0f, 0.0f, 1.0f );
	hrResult = m_pVtDefFx->SetMatrix( m_hVtDefFxProj, &mProj );
	if( FAILED( hrResult ) )
		return -1;

	hrResult = m_pVtDefFx->SetInt( m_hVtDefFxVtMapSz, m_iVtAttTexSize );
	if( FAILED( hrResult ) )
		return -1;

	//
	// Update vertex attributes
	//

	// Set render targets (render-to-texture)
	LPDIRECT3DSURFACE9 pd3dVtPosSur;
	hrResult = m_pd3dVtPosTex->GetSurfaceLevel( 0, &pd3dVtPosSur );
	if( FAILED( hrResult ) )
		return -1;

	// Vertex position texture
	hrResult = m_pd3dDevice->SetRenderTarget( 0, pd3dVtPosSur );
	if( FAILED( hrResult ) )
		return -1;

	LPDIRECT3DSURFACE9 pd3dVtTexCoordSur = NULL;

	if( m_bDefTex )
	{
		hrResult = m_pd3dVtTexCoordTex->GetSurfaceLevel( 0, 
			&pd3dVtTexCoordSur );
		if( FAILED( hrResult ) )
			return -1;

		// Texture coordinates texture
		hrResult = m_pd3dDevice->SetRenderTarget( 1, pd3dVtTexCoordSur );
		if( FAILED( hrResult ) )
			return -1;
	}

	// Disable z-buffering (testing and writing) 
	m_pd3dDevice->SetRenderState( D3DRS_ZENABLE,      FALSE );
	m_pd3dDevice->SetRenderState( D3DRS_ZWRITEENABLE, FALSE );

	// Render points only
	m_pd3dDevice->SetRenderState( D3DRS_FILLMODE, D3DFILL_POINT );

	// Set effect
	hrResult = m_pVtDefFx->SetTechnique( "VSDeform" );
	if( FAILED( hrResult ) )
		return -1;

	UINT uiPasses;
	hrResult = m_pVtDefFx->Begin( &uiPasses, 0 );
	if( FAILED( hrResult ) )
		return -1;

	for( UINT iPass = 0; iPass < uiPasses; iPass++ )
	{
		hrResult = m_pVtDefFx->BeginPass( iPass );
		if( FAILED( hrResult ) )
			return -1;

		// Call callback function to render indexed geometry with 
		// point rendering only
		if( m_fUpdate )
			(*m_fUpdate)( this, m_pUserData );

		m_pVtDefFx->EndPass( );
	}
	m_pVtDefFx->End( );

	// Render solids again
	m_pd3dDevice->SetRenderState( D3DRS_FILLMODE, D3DFILL_SOLID );

	// Reset render targets
	m_pd3dDevice->SetRenderTarget( 0, m_pManager->m_pd3dBackBuffer );
	m_pd3dDevice->SetRenderTarget( 1, NULL );

	SAFE_RELEASE( pd3dVtPosSur      );
	SAFE_RELEASE( pd3dVtTexCoordSur );

	//
	// Compute geometry attributes
	//

	// Compute TBN?
	if( m_pManager->m_bAttTBN )
	{
		LPDIRECT3DSURFACE9 pd3dVtCacheSur[3] = {NULL};
		LPDIRECT3DSURFACE9 pd3dVtNormalSur = NULL;

		//
		// Compute per-face TBN
		//

		// Set viewport
		d3dCurVp.Width = d3dCurVp.Height = m_iFaceIdxTexSize;
		m_pd3dDevice->SetViewport( &d3dCurVp );

		// Render target 0: face normal
		m_pd3dVtCacheTex[0]->GetSurfaceLevel( 0, &pd3dVtCacheSur[0] );
		m_pd3dDevice->SetRenderTarget( 0, pd3dVtCacheSur[0] );

		// Render target 1: face tangent S
		m_pd3dVtCacheTex[1]->GetSurfaceLevel( 0, &pd3dVtCacheSur[1] );
		m_pd3dDevice->SetRenderTarget( 1, pd3dVtCacheSur[1] );

		// Render target 2: face tangent T
		m_pd3dVtCacheTex[2]->GetSurfaceLevel( 0, &pd3dVtCacheSur[2] );
		m_pd3dDevice->SetRenderTarget( 2, pd3dVtCacheSur[2] );

		m_pd3dDevice->SetRenderTarget( 3, NULL );

		// Texture 0: face ID
		m_pManager->m_pTBNFx->SetTexture( "Tex0", m_pd3dFaceIdxTex    );
		// Texture 1: vertex position
		m_pManager->m_pTBNFx->SetTexture( "Tex1", m_pd3dVtPosTex      );
		// Texture 2: vertex texture coordinates
		m_pManager->m_pTBNFx->SetTexture( "Tex2", m_pd3dVtTexCoordTex );

		m_pd3dDevice->SetFVF( FVF_TLVERTEX );
		m_pd3dDevice->SetStreamSource( 0, NULL, 0, sizeof( TLVERTEX ) );

		// Set uniform parameters
		float pFloat[2];

		// 1D-2D constant of vertex attributes texture
		pFloat[0] = 1.0f / (float) m_iVtAttTexSize;
		pFloat[1] = 1.0f / (float)( m_iVtAttTexSize * m_iVtAttTexSize );
		m_pManager->m_pTBNFx->SetFloatArray( m_pManager->m_hTBNVtIdxC, 
											 pFloat, 2 );

		// Set effect
		hrResult = m_pManager->m_pTBNFx->SetTechnique( "FaceTBN" );
		if( FAILED( hrResult ) )
			return -1;

		UINT uiPasses;
		hrResult = m_pManager->m_pTBNFx->Begin( &uiPasses, 0 );
		if( FAILED( hrResult ) )
			return -1;

		for( UINT iPass = 0; iPass < uiPasses; iPass++ )
		{
			hrResult = m_pManager->m_pTBNFx->BeginPass( iPass );
			if( FAILED( hrResult ) )
				return -1;

			// Render filled quad to hit every pixel/texel
			m_pd3dDevice->DrawPrimitiveUP( D3DPT_TRIANGLESTRIP, 2, 
				m_pScreenQuadFace, sizeof( TLVERTEX ) );

			m_pManager->m_pTBNFx->EndPass( );
		}
		m_pManager->m_pTBNFx->End( );

		SAFE_RELEASE( pd3dVtCacheSur[0] );
		SAFE_RELEASE( pd3dVtCacheSur[1] );
		SAFE_RELEASE( pd3dVtCacheSur[2] );

		//
		// Compute per-vertex TBN
		//

		// Set viewport
		d3dCurVp.Width = d3dCurVp.Height = m_iVtAttTexSize;
		m_pd3dDevice->SetViewport( &d3dCurVp );

		// Render target 0: vertex normal
		m_pd3dVtNormalTex->GetSurfaceLevel( 0, &pd3dVtNormalSur );
		m_pd3dDevice->SetRenderTarget( 0, pd3dVtNormalSur );

		// Render target 1: tangent and handedness
		m_pd3dVtTanHTex->GetSurfaceLevel( 0, &pd3dVtCacheSur[0] );
		m_pd3dDevice->SetRenderTarget( 1, pd3dVtCacheSur[0] );

		m_pd3dDevice->SetRenderTarget( 2, NULL );

		// Texture 3: adjacency addresses
		m_pManager->m_pTBNFx->SetTexture( "Tex3", m_pd3dAdjAddTex     );
		// Texture 4: face adjacency texture
		m_pManager->m_pTBNFx->SetTexture( "Tex4", m_pd3dAdjFaceTex    );
		// Texture 5: face normal
		m_pManager->m_pTBNFx->SetTexture( "Tex5", m_pd3dVtCacheTex[0] );
		// Texture 6: face tangent s
		m_pManager->m_pTBNFx->SetTexture( "Tex6", m_pd3dVtCacheTex[1] );
		// Texture 7: face tangent t
		m_pManager->m_pTBNFx->SetTexture( "Tex7", m_pd3dVtCacheTex[2] );

		m_pd3dDevice->SetFVF( FVF_TLVERTEX );
		m_pd3dDevice->SetStreamSource( 0, NULL, 0, sizeof( TLVERTEX ) );

		// Set uniform parameters

		// 1D-2D constant of face adjacency texture
		pFloat[0] = 1.0f / (float) m_iAdjFaceTexSize; 
		pFloat[1] = 1.0f / (float)( m_iAdjFaceTexSize * m_iAdjFaceTexSize );
		m_pManager->m_pTBNFx->SetFloatArray( m_pManager->m_hTBNAdjFaceIdxC, 
											 pFloat, 2 );

		// 1D-2D constant of per-face vertex indices texture
		pFloat[0] = 1.0f / (float) m_iFaceIdxTexSize; 
		pFloat[1] = 1.0f / (float)( m_iFaceIdxTexSize * m_iFaceIdxTexSize );
		m_pManager->m_pTBNFx->SetFloatArray( m_pManager->m_hTBNFaceIdxC, 
											 pFloat, 2 );

		// Set effect
		hrResult = m_pManager->m_pTBNFx->SetTechnique( "VertexTBN" );
		if( FAILED( hrResult ) )
			return -1;

		hrResult = m_pManager->m_pTBNFx->Begin( &uiPasses, 0 );
		if( FAILED( hrResult ) )
			return -1;

		for( UINT iPass = 0; iPass < uiPasses; iPass++ )
		{
			hrResult = m_pManager->m_pTBNFx->BeginPass( iPass );
			if( FAILED( hrResult ) )
				return -1;

			// Render filled quad to hit every pixel/texel
			m_pd3dDevice->DrawPrimitiveUP( D3DPT_TRIANGLESTRIP, 2, 
				m_pScreenQuadVt, sizeof( TLVERTEX ) );

			m_pManager->m_pTBNFx->EndPass( );
		}
		m_pManager->m_pTBNFx->End( );

		// Reset render targets
		m_pd3dDevice->SetRenderTarget( 0, m_pManager->m_pd3dBackBuffer );
		m_pd3dDevice->SetRenderTarget( 1, NULL );

		SAFE_RELEASE( pd3dVtNormalSur   );
		SAFE_RELEASE( pd3dVtCacheSur[0] );

		m_pd3dDevice->SetVertexShader( NULL );
		m_pd3dDevice->SetPixelShader ( NULL );

		m_pd3dDevice->SetTexture( 0, NULL );
		m_pd3dDevice->SetTexture( 1, NULL );
		m_pd3dDevice->SetTexture( 2, NULL );
		m_pd3dDevice->SetTexture( 3, NULL );
		m_pd3dDevice->SetTexture( 4, NULL );
		m_pd3dDevice->SetTexture( 5, NULL );
		m_pd3dDevice->SetTexture( 6, NULL );
		m_pd3dDevice->SetTexture( 7, NULL );
	}

	// Compute normal?
	if( !m_pManager->m_bAttTBN && ( m_pManager->m_bAttNormal ||
		( m_pManager->m_bAttTensor2 || m_pManager->m_bAttTensor3 ||
		  m_pManager->m_bAttCurv ) ) )
	{
		LPDIRECT3DSURFACE9 pd3dVtCacheSur  = NULL;
		LPDIRECT3DSURFACE9 pd3dVtNormalSur = NULL;

		//
		// Compute per-face normal
		//

		// Set viewport
		d3dCurVp.Width = d3dCurVp.Height = m_iFaceIdxTexSize;
		m_pd3dDevice->SetViewport( &d3dCurVp );

		// Render target 0: face normal
		m_pd3dVtCacheTex[0]->GetSurfaceLevel( 0, &pd3dVtCacheSur );
		m_pd3dDevice->SetRenderTarget( 0, pd3dVtCacheSur );

		// Texture 0: face ID
		m_pManager->m_pNFx->SetTexture( "Tex0", m_pd3dFaceIdxTex    );
		// Texture 1: vertex position
		m_pManager->m_pNFx->SetTexture( "Tex1", m_pd3dVtPosTex      );

		m_pd3dDevice->SetFVF( FVF_TLVERTEX );
		m_pd3dDevice->SetStreamSource( 0, NULL, 0, sizeof( TLVERTEX ) );

		// Set uniform parameters
		float pFloat[2];

		// 1D-2D constant of vertex attributes texture
		pFloat[0] = 1.0f / (float) m_iVtAttTexSize;
		pFloat[1] = 1.0f / (float)( m_iVtAttTexSize * m_iVtAttTexSize );
		m_pManager->m_pNFx->SetFloatArray( m_pManager->m_hNVtIdxC, 
										   pFloat, 2 );

		// Set effect
		hrResult = m_pManager->m_pNFx->SetTechnique( "FaceN" );
		if( FAILED( hrResult ) )
			return -1;

		UINT uiPasses;
		hrResult = m_pManager->m_pNFx->Begin( &uiPasses, 0 );
		if( FAILED( hrResult ) )
			return -1;

		for( UINT iPass = 0; iPass < uiPasses; iPass++ )
		{
			hrResult = m_pManager->m_pNFx->BeginPass( iPass );
			if( FAILED( hrResult ) )
				return -1;

			// Render filled quad to hit every pixel/texel
			m_pd3dDevice->DrawPrimitiveUP( D3DPT_TRIANGLESTRIP, 2, 
				m_pScreenQuadFace, sizeof( TLVERTEX ) );

			m_pManager->m_pNFx->EndPass( );
		}
		m_pManager->m_pNFx->End( );

		SAFE_RELEASE( pd3dVtCacheSur );

		//
		// Compute per-vertex normal
		//

		// Set viewport
		d3dCurVp.Width = d3dCurVp.Height = m_iVtAttTexSize;
		m_pd3dDevice->SetViewport( &d3dCurVp );

		// Render target 0: vertex normal
		m_pd3dVtNormalTex->GetSurfaceLevel( 0, &pd3dVtNormalSur );
		m_pd3dDevice->SetRenderTarget( 0, pd3dVtNormalSur );

		// Texture 3: adjacency addresses
		m_pManager->m_pNFx->SetTexture( "Tex3", m_pd3dAdjAddTex     );
		// Texture 4: face adjacency texture
		m_pManager->m_pNFx->SetTexture( "Tex4", m_pd3dAdjFaceTex    );
		// Texture 5: face normal
		m_pManager->m_pNFx->SetTexture( "Tex5", m_pd3dVtCacheTex[0] );

		m_pd3dDevice->SetFVF( FVF_TLVERTEX );
		m_pd3dDevice->SetStreamSource( 0, NULL, 0, sizeof( TLVERTEX ) );

		// Set uniform parameters

		// 1D-2D constant of face adjacency texture
		pFloat[0] = 1.0f / (float) m_iAdjFaceTexSize; 
		pFloat[1] = 1.0f / (float)( m_iAdjFaceTexSize * m_iAdjFaceTexSize );
		m_pManager->m_pNFx->SetFloatArray( m_pManager->m_hNAdjFaceIdxC, 
										   pFloat, 2 );

		// 1D-2D constant of per-face vertex indices texture
		pFloat[0] = 1.0f / (float) m_iFaceIdxTexSize; 
		pFloat[1] = 1.0f / (float)( m_iFaceIdxTexSize * m_iFaceIdxTexSize );
		m_pManager->m_pNFx->SetFloatArray( m_pManager->m_hNFaceIdxC, 
										   pFloat, 2 );

		// Set effect
		hrResult = m_pManager->m_pNFx->SetTechnique( "VertexN" );
		if( FAILED( hrResult ) )
			return -1;

		hrResult = m_pManager->m_pNFx->Begin( &uiPasses, 0 );
		if( FAILED( hrResult ) )
			return -1;

		for( UINT iPass = 0; iPass < uiPasses; iPass++ )
		{
			hrResult = m_pManager->m_pNFx->BeginPass( iPass );
			if( FAILED( hrResult ) )
				return -1;

			// Render filled quad to hit every pixel/texel
			m_pd3dDevice->DrawPrimitiveUP( D3DPT_TRIANGLESTRIP, 2, 
				m_pScreenQuadVt, sizeof( TLVERTEX ) );

			m_pManager->m_pNFx->EndPass( );
		}
		m_pManager->m_pNFx->End( );

		// Reset render targets
		m_pd3dDevice->SetRenderTarget( 0, m_pManager->m_pd3dBackBuffer );

		SAFE_RELEASE( pd3dVtNormalSur );

		m_pd3dDevice->SetVertexShader( NULL );
		m_pd3dDevice->SetPixelShader ( NULL );

		m_pd3dDevice->SetTexture( 0, NULL );
		m_pd3dDevice->SetTexture( 1, NULL );
		m_pd3dDevice->SetTexture( 3, NULL );
		m_pd3dDevice->SetTexture( 4, NULL );
		m_pd3dDevice->SetTexture( 5, NULL );
	}

	// Compute 2nd fundamental form?
	if( m_pManager->m_bAttTensor2 || m_pManager->m_bAttTensor3 ||
		m_pManager->m_bAttCurv )
	{
		LPDIRECT3DSURFACE9 pd3dVtCacheSur[3] = {NULL};

		//
		// Compute per-vertex 2nd fundamental form and 
		// principle curvatures/directions
		//

		// Set viewport
		d3dCurVp.Width = d3dCurVp.Height = m_iVtAttTexSize;
		m_pd3dDevice->SetViewport( &d3dCurVp );

		// Render target 0: 2nd fundamental form
		m_pd3dVtTensor2Tex->GetSurfaceLevel( 0, &pd3dVtCacheSur[0] );
		m_pd3dDevice->SetRenderTarget( 0, pd3dVtCacheSur[0] );

		// Render target 1: #1 principle direction and curvature
		m_pd3dVtP1K1Tex->GetSurfaceLevel( 0, &pd3dVtCacheSur[1] );
		m_pd3dDevice->SetRenderTarget( 1, pd3dVtCacheSur[1] );

		// Render target 2: #2 principle direction and curvature
		m_pd3dVtP2K2Tex->GetSurfaceLevel( 0, &pd3dVtCacheSur[2] );
		m_pd3dDevice->SetRenderTarget( 2, pd3dVtCacheSur[2] );

		m_pd3dDevice->SetRenderTarget( 3, NULL );

		// Texture 0: vertex position
		m_pManager->m_pTensorFx->SetTexture( "Tex0", m_pd3dVtPosTex    );
		// Texture 1: normal vertex
		m_pManager->m_pTensorFx->SetTexture( "Tex1", m_pd3dVtNormalTex );
		// Texture 2: adjacency addresses
		m_pManager->m_pTensorFx->SetTexture( "Tex2", m_pd3dAdjAddTex   );
		// Texture 3: vertex adjacency addresses
		m_pManager->m_pTensorFx->SetTexture( "Tex3", m_pd3dAdjVtTex    );

		m_pd3dDevice->SetFVF( FVF_TLVERTEX );
		m_pd3dDevice->SetStreamSource( 0, NULL, 0, sizeof( TLVERTEX ) );

		// Set uniform parameters
		float pFloat[2];

		// 1D-2D constant of vertex attributes texture
		pFloat[0] = 1.0f / (float) m_iVtAttTexSize;
		pFloat[1] = 1.0f / (float)( m_iVtAttTexSize * m_iVtAttTexSize );
		m_pManager->m_pTensorFx->SetFloatArray( m_pManager->m_hTensorVtIdxC, 
											    pFloat, 2 );

		// 1D-2D constant of vertex adjacency texture
		pFloat[0] = 1.0f / (float) m_iAdjVtTexSize; 
		pFloat[1] = 1.0f / (float)( m_iAdjVtTexSize * m_iAdjVtTexSize );
		m_pManager->m_pTensorFx->SetFloatArray( m_pManager->m_hTensorAdjVtIdxC, 
											    pFloat, 2 );

		// Set effect
		hrResult = m_pManager->m_pTensorFx->SetTechnique( "Tensor2" );
		if( FAILED( hrResult ) )
			return -1;

		UINT uiPasses;
		hrResult = m_pManager->m_pTensorFx->Begin( &uiPasses, 0 );
		if( FAILED( hrResult ) )
			return -1;

		for( UINT iPass = 0; iPass < uiPasses; iPass++ )
		{
			hrResult = m_pManager->m_pTensorFx->BeginPass( iPass );
			if( FAILED( hrResult ) )
				return -1;

			// Render filled quad to hit every pixel/texel
			m_pd3dDevice->DrawPrimitiveUP( D3DPT_TRIANGLESTRIP, 2, 
				m_pScreenQuadVt, sizeof( TLVERTEX ) );

			m_pManager->m_pTensorFx->EndPass( );
		}
		m_pManager->m_pTensorFx->End( );

		// Reset render targets
		m_pd3dDevice->SetRenderTarget( 0, m_pManager->m_pd3dBackBuffer );
		m_pd3dDevice->SetRenderTarget( 1, NULL );
		m_pd3dDevice->SetRenderTarget( 2, NULL );

		SAFE_RELEASE( pd3dVtCacheSur[0] );
		SAFE_RELEASE( pd3dVtCacheSur[1] );
		SAFE_RELEASE( pd3dVtCacheSur[2] );

		m_pd3dDevice->SetVertexShader( NULL );
		m_pd3dDevice->SetPixelShader ( NULL );

		m_pd3dDevice->SetTexture( 0, NULL );
		m_pd3dDevice->SetTexture( 1, NULL );
		m_pd3dDevice->SetTexture( 2, NULL );
		m_pd3dDevice->SetTexture( 3, NULL );
	}

	// Compute tensor of curvature derivative?
	if( m_pManager->m_bAttTensor3 )
	{
		LPDIRECT3DSURFACE9 pd3dVtCacheSur = NULL;

		//
		// Compute per-vertex tensor of curvature derivative
		//

		// Set viewport
		d3dCurVp.Width = d3dCurVp.Height = m_iVtAttTexSize;
		m_pd3dDevice->SetViewport( &d3dCurVp );

		// Render target 0: tensor of curvature derivative
		m_pd3dVtTensor3Tex->GetSurfaceLevel( 0, &pd3dVtCacheSur );
		m_pd3dDevice->SetRenderTarget( 0, pd3dVtCacheSur );

		// Texture 0: vertex position
		m_pManager->m_pTensorFx->SetTexture( "Tex0", m_pd3dVtPosTex    );
		// Texture 1: normal vertex
		m_pManager->m_pTensorFx->SetTexture( "Tex1", m_pd3dVtNormalTex );
		// Texture 2: adjacency addresses
		m_pManager->m_pTensorFx->SetTexture( "Tex2", m_pd3dAdjAddTex   );
		// Texture 3: vertex adjacency addresses
		m_pManager->m_pTensorFx->SetTexture( "Tex3", m_pd3dAdjVtTex    );
		// Texture 4: Principal direction and curvature #1
		m_pManager->m_pTensorFx->SetTexture( "Tex4", m_pd3dVtP1K1Tex   );
		// Texture 5: Principal direction and curvature #2
		m_pManager->m_pTensorFx->SetTexture( "Tex5", m_pd3dVtP2K2Tex   );

		m_pd3dDevice->SetFVF( FVF_TLVERTEX );
		m_pd3dDevice->SetStreamSource( 0, NULL, 0, sizeof( TLVERTEX ) );

		// Set uniform parameters
		float pFloat[2];

		// 1D-2D constant of vertex attributes texture
		pFloat[0] = 1.0f / (float) m_iVtAttTexSize;
		pFloat[1] = 1.0f / (float)( m_iVtAttTexSize * m_iVtAttTexSize );
		m_pManager->m_pTensorFx->SetFloatArray( m_pManager->m_hTensorVtIdxC, 
											    pFloat, 2 );

		// 1D-2D constant of vertex adjacency texture
		pFloat[0] = 1.0f / (float) m_iAdjVtTexSize; 
		pFloat[1] = 1.0f / (float)( m_iAdjVtTexSize * m_iAdjVtTexSize );
		m_pManager->m_pTensorFx->SetFloatArray( m_pManager->m_hTensorAdjVtIdxC, 
											    pFloat, 2 );

		// Set effect
		hrResult = m_pManager->m_pTensorFx->SetTechnique( "Tensor3" );
		if( FAILED( hrResult ) )
			return -1;

		UINT uiPasses;
		hrResult = m_pManager->m_pTensorFx->Begin( &uiPasses, 0 );
		if( FAILED( hrResult ) )
			return -1;

		for( UINT iPass = 0; iPass < uiPasses; iPass++ )
		{
			hrResult = m_pManager->m_pTensorFx->BeginPass( iPass );
			if( FAILED( hrResult ) )
				return -1;

			// Render filled quad to hit every pixel/texel
			m_pd3dDevice->DrawPrimitiveUP( D3DPT_TRIANGLESTRIP, 2, 
				m_pScreenQuadVt, sizeof( TLVERTEX ) );

			m_pManager->m_pTensorFx->EndPass( );
		}
		m_pManager->m_pTensorFx->End( );

		// Reset render targets
		m_pd3dDevice->SetRenderTarget( 0, m_pManager->m_pd3dBackBuffer );

		SAFE_RELEASE( pd3dVtCacheSur );

		m_pd3dDevice->SetVertexShader( NULL );
		m_pd3dDevice->SetPixelShader ( NULL );

		m_pd3dDevice->SetTexture( 0, NULL );
		m_pd3dDevice->SetTexture( 1, NULL );
		m_pd3dDevice->SetTexture( 2, NULL );
		m_pd3dDevice->SetTexture( 3, NULL );
		m_pd3dDevice->SetTexture( 4, NULL );
		m_pd3dDevice->SetTexture( 5, NULL );
	}

	// Re-enable z-buffering (testing and writing)
	m_pd3dDevice->SetRenderState( D3DRS_ZWRITEENABLE, TRUE );
	m_pd3dDevice->SetRenderState( D3DRS_ZENABLE,      TRUE );

	// Restore viewport
	m_pd3dDevice->SetViewport( &d3dPrevVp );

#elif defined(OGL2)
	// Save current viewport settings
	GLint vPrevVP[4];
	glGetIntegerv( GL_VIEWPORT, vPrevVP );

	// Save current projection and modelview matrices
	GLfloat vPrevMV[16], vPrevP[16];
	glGetFloatv( GL_MODELVIEW_MATRIX, vPrevMV );
	glGetFloatv( GL_PROJECTION_MATRIX, vPrevP );

	// Set viewport for 1:1 pixel=texel=geometry mapping
	glViewport( 0, 0, m_iVtAttTexSize, m_iVtAttTexSize );
	glMatrixMode( GL_PROJECTION );
	glLoadIdentity( );
	gluOrtho2D( 0.0, m_iVtAttTexSize, 0.0, m_iVtAttTexSize );
	glMatrixMode( GL_MODELVIEW );
	glLoadIdentity( );

	// Use shader program for vertex-level deformations
	glUseProgramObjectARB( m_hVtDefPO );

	// Set shader uniform parameters
	glUniform1iARB( m_iVtDefPOVtMapSize, m_iVtAttTexSize );

	//
	// Update vertex attributes
	//

	// Bind framebuffer object
	glBindFramebufferEXT( GL_FRAMEBUFFER_EXT, m_uiVtTexFBO );

	// Attach render targets (render-to-textures)
	glFramebufferTexture2DEXT( GL_FRAMEBUFFER_EXT, 
							   GL_COLOR_ATTACHMENT0_EXT, 
							   GL_TEXTURE_2D, m_uiVtPosTexRT, 0 );
	if( m_bDefTex )
	{
		glFramebufferTexture2DEXT( GL_FRAMEBUFFER_EXT, 
								   GL_COLOR_ATTACHMENT1_EXT, 
								   GL_TEXTURE_2D, m_uiVtTexCoordTexRT, 0 );

		// Set the texture as render target
		static const GLenum eRTs1[] = { GL_COLOR_ATTACHMENT0_EXT, 
										GL_COLOR_ATTACHMENT1_EXT };
		glDrawBuffers( 2, eRTs1 );
	}
	else
	{
		// Set the texture as render target
		static const GLenum eRTs1[] = { GL_COLOR_ATTACHMENT0_EXT };
		glDrawBuffers( 1, eRTs1 );
	}

	// Disable z-buffering (testing and writing) 
	glDisable( GL_DEPTH_TEST );

	// Render points only
	glPolygonMode( GL_FRONT_AND_BACK, GL_POINT );

	// Call callback function to render indexed geometry with 
	// point rendering only
	if( m_fUpdate )
		(*m_fUpdate)( this, m_pUserData );

	// Render solid again
	glPolygonMode( GL_FRONT_AND_BACK, GL_FILL );

	// Re-enable rendering to the back buffer
    glBindFramebufferEXT( GL_FRAMEBUFFER_EXT, 0 );

	//
	// Compute geometry attributes
	//

	GLint is0, is1, is2, is3, is4, is5, is6, is7;

	// Compute TBN?
	if( m_pManager->m_bAttTBN )
	{
		//
		// Compute per-face TBN
		//

		// Set viewport
		glViewport( 0, 0, m_iFaceIdxTexSize, m_iFaceIdxTexSize );

		GLenum eRTs2[3];

		// Render target 0: face normal
		glFramebufferTexture2DEXT( GL_FRAMEBUFFER_EXT, 
								   GL_COLOR_ATTACHMENT0_EXT + 0, 
								   GL_TEXTURE_2D, 
								   m_uiVtCacheTex[0], 0 );
		eRTs2[0] = GL_COLOR_ATTACHMENT0_EXT + 0;

		// Render target 1: face tangent s
		glFramebufferTexture2DEXT( GL_FRAMEBUFFER_EXT, 
								   GL_COLOR_ATTACHMENT0_EXT + 1, 
								   GL_TEXTURE_2D, 
								   m_uiVtCacheTex[1], 0 );
		eRTs2[1] = GL_COLOR_ATTACHMENT0_EXT + 1;

		// Render target 2: face tangent t
		glFramebufferTexture2DEXT( GL_FRAMEBUFFER_EXT, 
								   GL_COLOR_ATTACHMENT0_EXT + 2, 
								   GL_TEXTURE_2D, 
								   m_uiVtCacheTex[2], 0 );
		eRTs2[2] = GL_COLOR_ATTACHMENT0_EXT + 2;

		glDrawBuffers( 3, eRTs2 );

		// Set shader
		glUseProgramObjectARB( m_pManager->m_hFaceTBNPO );

		// Size of a dimension of the vertex attributes texture
		glUniform1iARB( m_pManager->m_iFaceTBNVtMapSize, m_iVtAttTexSize );

		// Texture 0: face ID
		glActiveTexture( GL_TEXTURE0 );
		glBindTexture( GL_TEXTURE_2D, m_uiFaceIdxTex );
		is0 = glGetUniformLocationARB( m_pManager->m_hFaceTBNPO, "s0" );
		glUniform1iARB( is0, 0 );

		// Texture 1: vertex position
		glActiveTexture( GL_TEXTURE1 );
		glBindTexture( GL_TEXTURE_2D, m_uiVtPosTexRT );
		is1 = glGetUniformLocationARB( m_pManager->m_hFaceTBNPO, "s1" );
		glUniform1iARB( is1, 1 );

		// Texture 2: vertex texture coordinates
		glActiveTexture( GL_TEXTURE2 );
		glBindTexture( GL_TEXTURE_2D, m_uiVtTexCoordTexRT );
		is2 = glGetUniformLocationARB( m_pManager->m_hFaceTBNPO, "s2" );
		glUniform1iARB( is2, 2 );

		// Render filled quad to hit every pixel/texel
		glPolygonMode( GL_FRONT, GL_FILL );
		glBegin( GL_QUADS );
			glTexCoord2f( 0.0f, 0.0f ); 
			glVertex2f  ( 0.0f, (float) m_iFaceIdxTexSize - 0.0f );
			glTexCoord2f( 1.0f, 0.0f ); 
			glVertex2f  ( (float) m_iFaceIdxTexSize, (float) m_iFaceIdxTexSize - 0.0f );
			glTexCoord2f( 1.0f, 1.0f );
			glVertex2f  ( (float) m_iFaceIdxTexSize, (float) m_iFaceIdxTexSize - (float) m_iFaceIdxTexSize );
			glTexCoord2f( 0.0f, 1.0f ); 
			glVertex2f  ( 0.0f, (float) m_iFaceIdxTexSize - (float) m_iFaceIdxTexSize );
		glEnd( );

		//
		// Compute per-vertex TBN
		//

		// Set viewport
		glViewport( 0, 0, m_iVtAttTexSize, m_iVtAttTexSize );

		// Render target 0: vertex normal
		glFramebufferTexture2DEXT( GL_FRAMEBUFFER_EXT, 
								   GL_COLOR_ATTACHMENT0_EXT + 0, 
								   GL_TEXTURE_2D, 
								   m_uiVtNormalTexRT, 0 );
		eRTs2[0] = GL_COLOR_ATTACHMENT0_EXT + 0;

		// Render target 1: tangent and handedness
		glFramebufferTexture2DEXT( GL_FRAMEBUFFER_EXT, 
								   GL_COLOR_ATTACHMENT0_EXT + 1, 
								   GL_TEXTURE_2D, 
								   m_uiVtTanHTexRT, 0 );
		eRTs2[1] = GL_COLOR_ATTACHMENT0_EXT + 1;

		glDrawBuffers( 2, eRTs2 );

		// Set shader
		glUseProgramObjectARB( m_pManager->m_hVertexTBNPO );

		// Size of a dimension of the face adjacency texture
		glUniform1iARB( m_pManager->m_iVertexTBNAdjFaceMapSize, m_iAdjFaceTexSize );

		// Size of a dimension of the per-face vertex indices texture
		glUniform1iARB( m_pManager->m_iVertexTBNFaceMapSize, m_iFaceIdxTexSize );

		// Texture 0: face ID
		glActiveTexture( GL_TEXTURE0 );
		glBindTexture( GL_TEXTURE_2D, m_uiFaceIdxTex );
		is0 = glGetUniformLocationARB( m_pManager->m_hVertexTBNPO, "s0" );
		glUniform1iARB( is0, 0 );

		// Texture 1: vertex position
		glActiveTexture( GL_TEXTURE1 );
		glBindTexture( GL_TEXTURE_2D, m_uiVtPosTexRT );
		is1 = glGetUniformLocationARB( m_pManager->m_hVertexTBNPO, "s1" );
		glUniform1iARB( is1, 1 );

		// Texture 2: vertex texture coordinates
		glActiveTexture( GL_TEXTURE2 );
		glBindTexture( GL_TEXTURE_2D, m_uiVtTexCoordTexRT );
		is2 = glGetUniformLocationARB( m_pManager->m_hVertexTBNPO, "s2" );
		glUniform1iARB( is2, 2 );

		// Texture 3: adjacency addresses
		glActiveTexture( GL_TEXTURE3 );
		glBindTexture( GL_TEXTURE_2D, m_uiAdjAddTex );
		is3 = glGetUniformLocationARB( m_pManager->m_hVertexTBNPO, "s3" );
		glUniform1iARB( is3, 3 );

		// Texture 4: face adjacency texture
		glActiveTexture( GL_TEXTURE4 );
		glBindTexture( GL_TEXTURE_2D, m_uiAdjFaceTex );
		is4 = glGetUniformLocationARB( m_pManager->m_hVertexTBNPO, "s4" );
		glUniform1iARB( is4, 4 );

		// Texture 5: face normal
		glActiveTexture( GL_TEXTURE5 );
		glBindTexture( GL_TEXTURE_2D, m_uiVtCacheTex[0] );
		is5 = glGetUniformLocationARB( m_pManager->m_hVertexTBNPO, "s5" );
		glUniform1iARB( is5, 5 );

		// Texture 6: face tangent s
		glActiveTexture( GL_TEXTURE6 );
		glBindTexture( GL_TEXTURE_2D, m_uiVtCacheTex[1] );
		is6 = glGetUniformLocationARB( m_pManager->m_hVertexTBNPO, "s6" );
		glUniform1iARB( is6, 6 );

		// Texture 7: face tangent t
		glActiveTexture( GL_TEXTURE7 );
		glBindTexture( GL_TEXTURE_2D, m_uiVtCacheTex[2] );
		is7 = glGetUniformLocationARB( m_pManager->m_hVertexTBNPO, "s7" );
		glUniform1iARB( is7, 7 );

		// Render filled quad to hit every pixel/texel
		glPolygonMode( GL_FRONT, GL_FILL );
		glBegin( GL_QUADS );
			glTexCoord2f( 0.0f, 0.0f ); 
			glVertex2f  ( 0.0f, (float) m_iVtAttTexSize - 0.0f );
			glTexCoord2f( 1.0f, 0.0f ); 
			glVertex2f  ( (float) m_iVtAttTexSize, (float) m_iVtAttTexSize - 0.0f );
			glTexCoord2f( 1.0f, 1.0f );
			glVertex2f  ( (float) m_iVtAttTexSize, (float) m_iVtAttTexSize - (float) m_iVtAttTexSize );
			glTexCoord2f( 0.0f, 1.0f ); 
			glVertex2f  ( 0.0f, (float) m_iVtAttTexSize - (float) m_iVtAttTexSize );
		glEnd( );

		// Re-enable rendering to the back buffer
		glBindFramebufferEXT( GL_FRAMEBUFFER_EXT, 0 );

		// Disable shaders
		glUseProgramObjectARB( 0 );
	}

	// Compute normal?
	if( !m_pManager->m_bAttTBN && ( m_pManager->m_bAttNormal ||
		( m_pManager->m_bAttTensor2 || m_pManager->m_bAttTensor3 ||
		  m_pManager->m_bAttCurv ) ) )
	{
		//
		// Compute per-face N
		//

		// Set viewport
		glViewport( 0, 0, m_iFaceIdxTexSize, m_iFaceIdxTexSize );

		// Render target 0: face normal
		glFramebufferTexture2DEXT( GL_FRAMEBUFFER_EXT, 
								   GL_COLOR_ATTACHMENT0_EXT, 
								   GL_TEXTURE_2D, 
								   m_uiVtCacheTex[0], 0 );

		glDrawBuffer( GL_COLOR_ATTACHMENT0_EXT );

		// Set shader
		glUseProgramObjectARB( m_pManager->m_hFaceNPO );

		// Size of a dimension of the vertex attributes texture
		glUniform1iARB( m_pManager->m_iFaceNVtMapSize, m_iVtAttTexSize );

		// Texture 0: face ID
		glActiveTexture( GL_TEXTURE0 );
		glBindTexture( GL_TEXTURE_2D, m_uiFaceIdxTex );
		GLint is0 = glGetUniformLocationARB( m_pManager->m_hFaceNPO, "s0" );
		glUniform1iARB( is0, 0 );

		// Texture 1: vertex position
		glActiveTexture( GL_TEXTURE1 );
		glBindTexture( GL_TEXTURE_2D, m_uiVtPosTexRT );
		GLint is1 = glGetUniformLocationARB( m_pManager->m_hFaceNPO, "s1" );
		glUniform1iARB( is1, 1 );

		// Render filled quad to hit every pixel/texel
		glPolygonMode( GL_FRONT, GL_FILL );
		glBegin( GL_QUADS );
			glTexCoord2f( 0.0f, 0.0f ); 
			glVertex2f  ( 0.0f, (float) m_iFaceIdxTexSize - 0.0f );
			glTexCoord2f( 1.0f, 0.0f ); 
			glVertex2f  ( (float) m_iFaceIdxTexSize, (float) m_iFaceIdxTexSize - 0.0f );
			glTexCoord2f( 1.0f, 1.0f );
			glVertex2f  ( (float) m_iFaceIdxTexSize, (float) m_iFaceIdxTexSize - (float) m_iFaceIdxTexSize );
			glTexCoord2f( 0.0f, 1.0f ); 
			glVertex2f  ( 0.0f, (float) m_iFaceIdxTexSize - (float) m_iFaceIdxTexSize );
		glEnd( );

		//
		// Compute per-vertex N
		//

		// Set viewport
		glViewport( 0, 0, m_iVtAttTexSize, m_iVtAttTexSize );

		// Render target 0: vertex normal
		glFramebufferTexture2DEXT( GL_FRAMEBUFFER_EXT, 
								   GL_COLOR_ATTACHMENT0_EXT, 
								   GL_TEXTURE_2D, 
								   m_uiVtNormalTexRT, 0 );

		glDrawBuffer( GL_COLOR_ATTACHMENT0_EXT );

		// Set shader
		glUseProgramObjectARB( m_pManager->m_hVertexNPO );

		// Size of a dimension of the face adjacency texture
		glUniform1iARB( m_pManager->m_iVertexNAdjFaceMapSize, m_iAdjFaceTexSize );

		// Size of a dimension of the per-face vertex indices texture
		glUniform1iARB( m_pManager->m_iVertexNFaceMapSize, m_iFaceIdxTexSize );

		// Texture 0: face ID
		glActiveTexture( GL_TEXTURE0 );
		glBindTexture( GL_TEXTURE_2D, m_uiFaceIdxTex );
		is0 = glGetUniformLocationARB( m_pManager->m_hVertexNPO, "s0" );
		glUniform1iARB( is0, 0 );

		// Texture 1: vertex position
		glActiveTexture( GL_TEXTURE1 );
		glBindTexture( GL_TEXTURE_2D, m_uiVtPosTexRT );
		is1 = glGetUniformLocationARB( m_pManager->m_hVertexNPO, "s1" );
		glUniform1iARB( is1, 1 );

		// Texture 3: adjacency addresses
		glActiveTexture( GL_TEXTURE3 );
		glBindTexture( GL_TEXTURE_2D, m_uiAdjAddTex );
		is3 = glGetUniformLocationARB( m_pManager->m_hVertexNPO, "s3" );
		glUniform1iARB( is3, 3 );

		// Texture 4: face adjacency texture
		glActiveTexture( GL_TEXTURE4 );
		glBindTexture( GL_TEXTURE_2D, m_uiAdjFaceTex );
		is4 = glGetUniformLocationARB( m_pManager->m_hVertexNPO, "s4" );
		glUniform1iARB( is4, 4 );

		// Texture 5: face normal
		glActiveTexture( GL_TEXTURE5 );
		glBindTexture( GL_TEXTURE_2D, m_uiVtCacheTex[0] );
		is5 = glGetUniformLocationARB( m_pManager->m_hVertexNPO, "s5" );
		glUniform1iARB( is5, 5 );

		// Render filled quad to hit every pixel/texel
		glPolygonMode( GL_FRONT, GL_FILL );
		glBegin( GL_QUADS );
			glTexCoord2f( 0.0f, 0.0f ); 
			glVertex2f  ( 0.0f, (float) m_iVtAttTexSize - 0.0f );
			glTexCoord2f( 1.0f, 0.0f ); 
			glVertex2f  ( (float) m_iVtAttTexSize, (float) m_iVtAttTexSize - 0.0f );
			glTexCoord2f( 1.0f, 1.0f );
			glVertex2f  ( (float) m_iVtAttTexSize, (float) m_iVtAttTexSize - (float) m_iVtAttTexSize );
			glTexCoord2f( 0.0f, 1.0f ); 
			glVertex2f  ( 0.0f, (float) m_iVtAttTexSize - (float) m_iVtAttTexSize );
		glEnd( );

		// Re-enable rendering to the back buffer
		glBindFramebufferEXT( GL_FRAMEBUFFER_EXT, 0 );

		// Disable shaders
		glUseProgramObjectARB( 0 );
	}

	// Compute 2nd fundamental form?
	if( m_pManager->m_bAttTensor2 || m_pManager->m_bAttTensor3 ||
		m_pManager->m_bAttCurv )
	{
		//
		// Compute per-vertex 2nd fundamental form and 
		// principle curvatures/directions
		//

		// Set viewport
		glViewport( 0, 0, m_iVtAttTexSize, m_iVtAttTexSize );

		GLenum eRTs2[3];

		// Render target 0: 2nd fundamental form
		glFramebufferTexture2DEXT( GL_FRAMEBUFFER_EXT, 
								   GL_COLOR_ATTACHMENT0_EXT + 0, 
								   GL_TEXTURE_2D, 
								   m_uiVtTensor2TexRT, 0 );
		eRTs2[0] = GL_COLOR_ATTACHMENT0_EXT + 0;

		// Render target 1: #1 principle direction and curvature
		glFramebufferTexture2DEXT( GL_FRAMEBUFFER_EXT, 
								   GL_COLOR_ATTACHMENT0_EXT + 1, 
								   GL_TEXTURE_2D, 
								   m_uiVtP1K1TexRT, 0 );
		eRTs2[1] = GL_COLOR_ATTACHMENT0_EXT + 1;

		// Render target 2: #2 principle direction and curvature
		glFramebufferTexture2DEXT( GL_FRAMEBUFFER_EXT, 
								   GL_COLOR_ATTACHMENT0_EXT + 2, 
								   GL_TEXTURE_2D, 
								   m_uiVtP2K2TexRT, 0 );
		eRTs2[2] = GL_COLOR_ATTACHMENT0_EXT + 2;

		glDrawBuffers( 3, eRTs2 );

		// Set shader
		glUseProgramObjectARB( m_pManager->m_h2ndTensorPO );

		// Size of a dimension of the vertex attributes texture
		glUniform1iARB( m_pManager->m_i2ndTensorVtMapSize, m_iVtAttTexSize );

		// Size of a dimension of the vertex adjacency texture
		glUniform1iARB( m_pManager->m_i2ndTensorAdjVtMapSize, m_iAdjVtTexSize );

		// Texture 0: vertex position
		glActiveTexture( GL_TEXTURE0 );
		glBindTexture( GL_TEXTURE_2D, m_uiVtPosTexRT );
		is1 = glGetUniformLocationARB( m_pManager->m_h2ndTensorPO, "s0" );
		glUniform1iARB( is0, 0 );

		// Texture 1: vertex normal
		glActiveTexture( GL_TEXTURE1 );
		glBindTexture( GL_TEXTURE_2D, m_uiVtNormalTexRT );
		is1 = glGetUniformLocationARB( m_pManager->m_h2ndTensorPO, "s1" );
		glUniform1iARB( is1, 1 );

		// Texture 2: adjacency addresses
		glActiveTexture( GL_TEXTURE2 );
		glBindTexture( GL_TEXTURE_2D, m_uiAdjAddTex );
		is2 = glGetUniformLocationARB( m_pManager->m_h2ndTensorPO, "s2" );
		glUniform1iARB( is2, 2 );

		// Texture 3: vertex adjacency texture
		glActiveTexture( GL_TEXTURE3 );
		glBindTexture( GL_TEXTURE_2D, m_uiAdjVtTex );
		is3 = glGetUniformLocationARB( m_pManager->m_h2ndTensorPO, "s3" );
		glUniform1iARB( is3, 3 );

		// Render filled quad to hit every pixel/texel
		glPolygonMode( GL_FRONT, GL_FILL );
		glBegin( GL_QUADS );
			glTexCoord2f( 0.0f, 0.0f ); 
			glVertex2f  ( 0.0f, (float) m_iVtAttTexSize - 0.0f );
			glTexCoord2f( 1.0f, 0.0f ); 
			glVertex2f  ( (float) m_iVtAttTexSize, (float) m_iVtAttTexSize - 0.0f );
			glTexCoord2f( 1.0f, 1.0f );
			glVertex2f  ( (float) m_iVtAttTexSize, (float) m_iVtAttTexSize - (float) m_iVtAttTexSize );
			glTexCoord2f( 0.0f, 1.0f ); 
			glVertex2f  ( 0.0f, (float) m_iVtAttTexSize - (float) m_iVtAttTexSize );
		glEnd( );

		// Re-enable rendering to the back buffer
		glBindFramebufferEXT( GL_FRAMEBUFFER_EXT, 0 );

		// Disable shaders
		glUseProgramObjectARB( 0 );
	}

	// Compute tensor of curvature derivative?
	if( m_pManager->m_bAttTensor3 )
	{
		//
		// Compute per-vertex tensor of curvature derivative
		//

		// Set viewport
		glViewport( 0, 0, m_iVtAttTexSize, m_iVtAttTexSize );

		// Render target 0: 2nd fundamental form
		glFramebufferTexture2DEXT( GL_FRAMEBUFFER_EXT, 
								   GL_COLOR_ATTACHMENT0_EXT, 
								   GL_TEXTURE_2D, 
								   m_uiVtTensor3TexRT, 0 );

		glDrawBuffer( GL_COLOR_ATTACHMENT0_EXT );

		// Set shader
		glUseProgramObjectARB( m_pManager->m_h3rdTensorPO );

		// Size of a dimension of the vertex attributes texture
		glUniform1iARB( m_pManager->m_i3rdTensorVtMapSize, m_iVtAttTexSize );

		// Size of a dimension of the vertex adjacency texture
		glUniform1iARB( m_pManager->m_i3rdTensorAdjVtMapSize, m_iAdjVtTexSize );

		// Texture 0: vertex position
		glActiveTexture( GL_TEXTURE0 );
		glBindTexture( GL_TEXTURE_2D, m_uiVtPosTexRT );
		is1 = glGetUniformLocationARB( m_pManager->m_h3rdTensorPO, "s0" );
		glUniform1iARB( is0, 0 );

		// Texture 1: vertex normal
		glActiveTexture( GL_TEXTURE1 );
		glBindTexture( GL_TEXTURE_2D, m_uiVtNormalTexRT );
		is1 = glGetUniformLocationARB( m_pManager->m_h3rdTensorPO, "s1" );
		glUniform1iARB( is1, 1 );

		// Texture 2: adjacency addresses
		glActiveTexture( GL_TEXTURE2 );
		glBindTexture( GL_TEXTURE_2D, m_uiAdjAddTex );
		is2 = glGetUniformLocationARB( m_pManager->m_h3rdTensorPO, "s2" );
		glUniform1iARB( is2, 2 );

		// Texture 3: vertex adjacency texture
		glActiveTexture( GL_TEXTURE3 );
		glBindTexture( GL_TEXTURE_2D, m_uiAdjVtTex );
		is3 = glGetUniformLocationARB( m_pManager->m_h3rdTensorPO, "s3" );
		glUniform1iARB( is3, 3 );

		// Texture 4: Principal direction and curvature #1
		glActiveTexture( GL_TEXTURE4 );
		glBindTexture( GL_TEXTURE_2D, m_uiVtP1K1TexRT );
		is4 = glGetUniformLocationARB( m_pManager->m_h3rdTensorPO, "s4" );
		glUniform1iARB( is4, 4 );

		// Texture 5: Principal direction and curvature #2
		glActiveTexture( GL_TEXTURE5 );
		glBindTexture( GL_TEXTURE_2D, m_uiVtP2K2TexRT );
		is5 = glGetUniformLocationARB( m_pManager->m_h3rdTensorPO, "s5" );
		glUniform1iARB( is5, 5 );

		// Render filled quad to hit every pixel/texel
		glPolygonMode( GL_FRONT, GL_FILL );
		glBegin( GL_QUADS );
			glTexCoord2f( 0.0f, 0.0f ); 
			glVertex2f  ( 0.0f, (float) m_iVtAttTexSize - 0.0f );
			glTexCoord2f( 1.0f, 0.0f ); 
			glVertex2f  ( (float) m_iVtAttTexSize, (float) m_iVtAttTexSize - 0.0f );
			glTexCoord2f( 1.0f, 1.0f );
			glVertex2f  ( (float) m_iVtAttTexSize, (float) m_iVtAttTexSize - (float) m_iVtAttTexSize );
			glTexCoord2f( 0.0f, 1.0f ); 
			glVertex2f  ( 0.0f, (float) m_iVtAttTexSize - (float) m_iVtAttTexSize );
		glEnd( );

		// Re-enable rendering to the back buffer
		glBindFramebufferEXT( GL_FRAMEBUFFER_EXT, 0 );

		// Disable shaders
		glUseProgramObjectARB( 0 );
	}

	// Re-enable z-buffering (testing and writing)
	glEnable( GL_DEPTH_TEST );

	// Restore projection and modelview matrices
	glMatrixMode( GL_PROJECTION );
	glLoadIdentity( );
	glMultMatrixf( (const GLfloat*) vPrevP );
	glMatrixMode( GL_MODELVIEW );
	glLoadIdentity( );
	glMultMatrixf( (const GLfloat*) vPrevMV );

	// Restore viewport
	glViewport( vPrevVP[0], vPrevVP[1], vPrevVP[2], vPrevVP[3] );
#endif

	return 0;
} // CIntObj::Update

////////////////////////////////////////////////////////////////////////////////
/// \brief Render the required attributes of this model.
///
/// \return 0 on success.
///         -1 on error.
////////////////////////////////////////////////////////////////////////////////
int CIntObj::Render( )
{
	if( !m_bVisible )
		return 0;

#if defined(D3D9)
	// Set viewport for attribute rendering
	D3DVIEWPORT9 d3dPrevVp, d3dCurVp;
	m_pd3dDevice->GetViewport( &d3dPrevVp );
	d3dCurVp = d3dPrevVp;
	d3dCurVp.X      = 0;
	d3dCurVp.Y      = 0;
	d3dCurVp.Width  = m_pManager->m_iRight  - m_pManager->m_iLeft + 1;
	d3dCurVp.Height = m_pManager->m_iBottom - m_pManager->m_iTop  + 1;
	m_pd3dDevice->SetViewport( &d3dCurVp );

	// Get current world, view, proj matrices
	D3DXMATRIX mProj = m_pManager->m_mROIProj;
	D3DXMATRIX mWorld, mView, mITW;
	m_pd3dDevice->GetTransform( D3DTS_WORLD, &mWorld );
	m_pd3dDevice->GetTransform( D3DTS_VIEW,  &mView  );

	// Apply local transformation
	D3DXMatrixMultiply( &mWorld, &m_mLocal, &mWorld );

	D3DXMATRIX mWorldView     = mWorld     * mView;
	D3DXMATRIX mViewProj      = mView      * mProj;
	D3DXMATRIX mWorldViewProj = mWorldView * mProj;

	//	D3DXMatrixIdentity( &mITW );
	float fDet;
	D3DXMatrixInverse  ( &mITW, &fDet, &mWorld );
	D3DXMatrixTranspose( &mITW, &mITW );

	// 1D-2D index conversion constant for sampling vertex 
	// attributes textures
	float pFloat[2];
	pFloat[0] = 1.0f / (float) m_iVtAttTexSize;
	pFloat[1] = 1.0f / (float)( m_iVtAttTexSize * m_iVtAttTexSize );

	// Call callback function to render indexed geometry
	if( m_fRenderIdx && m_pManager->m_bRenderIdx )
	{
		HRESULT hrResult = S_OK;

		if( m_pd3dDevice == NULL )
			return -1;

		// For each render pass
		int iRT = 0;
		for( int iRP = 0; iRP < m_pManager->m_iNumRPs; iRP++ )
		{
			// Set render targets (render-to-texture)
			int iCt;
			for( iCt = 0; iCt < m_pManager->m_pRenderPass[iRP].m_iRTs; iCt++ )
			{
				m_pd3dDevice->SetRenderTarget( iCt, m_pManager->m_pRTS[iRT] );
				iRT++;
			}
			for( int iCt2 = iCt; iCt2 < 4; iCt2++ )
				m_pd3dDevice->SetRenderTarget( iCt2, NULL );

//			m_pd3dDevice->Clear( 0L, NULL, D3DCLEAR_TARGET,
//								 D3DCOLOR_ARGB( 255, 0, 0, 0 ), 1.0f, 0L );

			// Set 1D-2D index conversion constant
			m_pEncodeIFx[iRP]->SetFloatArray( m_hEncodeIFxIdxC[iRP], 
											  pFloat, 2 );

			// Set transformation matrices to the current shader
			m_pEncodeIFx[iRP]->SetMatrix( m_hEncodeIFxWVP[iRP], 
										  &mWorldViewProj );
			m_pEncodeIFx[iRP]->SetMatrix( m_hEncodeIFxITW[iRP], &mITW );

			// Texture 0: vertex position
			m_pEncodeIFx[iRP]->SetTexture( "Tex0", m_pd3dVtPosTex );

			// Texture 1: vertex normal
			if( m_pManager->m_bAttTBN )
			{
				// Vertex normal
				m_pEncodeIFx[iRP]->SetTexture( "Tex1", m_pd3dVtNormalTex );

				// Texture 3: vertex tangent and handedness
				m_pEncodeIFx[iRP]->SetTexture( "Tex3", m_pd3dVtTanHTex );
			}
			else
			if( m_pManager->m_bAttNormal )
				m_pEncodeIFx[iRP]->SetTexture( "Tex1", m_pd3dVtNormalTex );

			// Texture 2: vertex texture coordinates
			m_pEncodeIFx[iRP]->SetTexture( "Tex2", m_pd3dVtTexCoordTex );

			if( m_pManager->m_bAttCurv )
			{
				// Texture 4: vertex principal curvature and direction #1
				m_pEncodeIFx[iRP]->SetTexture( "Tex4", m_pd3dVtP1K1Tex );
				// Texture 5: vertex principal curvature and direction #2
				m_pEncodeIFx[iRP]->SetTexture( "Tex5", m_pd3dVtP2K2Tex );
			}

			if( m_pManager->m_bAttTensor2 )
			{
				// Texture 6: 2nd fundamental form
				m_pEncodeIFx[iRP]->SetTexture( "Tex6", m_pd3dVtTensor2Tex );
			}

			if( m_pManager->m_bAttTensor3 )
			{
				// Texture 7: tensor of curvature derivative
				m_pEncodeIFx[iRP]->SetTexture( "Tex7", m_pd3dVtTensor3Tex );
			}

			// Set effect
			hrResult = m_pEncodeIFx[iRP]->SetTechnique( "Encode" );
			if( FAILED( hrResult ) )
				return -1;

			UINT uiPasses;
			hrResult = m_pEncodeIFx[iRP]->Begin( &uiPasses, 0 );
			if( FAILED( hrResult ) )
				return -1;

			for( UINT iPass = 0; iPass < uiPasses; iPass++ )
			{
				hrResult = m_pEncodeIFx[iRP]->BeginPass( iPass );
				if( FAILED( hrResult ) )
					return -1;

				// Call callback function to render indexed geometry
				(*m_fRenderIdx)( this, iRP, m_pUserData );

				m_pEncodeIFx[iRP]->EndPass( );
			}

			m_pEncodeIFx[iRP]->End( );
		}
	}

	// Call callback function to render non-indexed geometry
	if( m_fRenderNonIdx && m_pManager->m_bAttUserDefNI )
	{
		HRESULT hrResult = S_OK;
		hrResult = m_pd3dDevice->SetRenderTarget( 0, 
			m_pManager->m_pRTS[m_pManager->m_iNumRTs-1] );
		if( FAILED( hrResult ) )
			return -2;
		for( int iCt2 = 1; iCt2 < 4; iCt2++ )
			m_pd3dDevice->SetRenderTarget( iCt2, NULL );

//		m_pd3dDevice->Clear( 0L, NULL, D3DCLEAR_TARGET,
//							 D3DCOLOR_ARGB( 255, 0, 0, 0 ), 1.0f, 0L );

		// Set 1D-2D index conversion constant
		m_pEncodeNIFx->SetFloatArray( m_hEncodeNIFxIdxC, pFloat, 2 );

		// Set transformation matrices to the current shader
		m_pEncodeNIFx->SetMatrix( m_hEncodeNIFxWVP, &mWorldViewProj );
		m_pEncodeNIFx->SetMatrix( m_hEncodeNIFxITW, &mITW );

		// Texture 0: vertex position
		m_pEncodeNIFx->SetTexture( "Tex0", m_pd3dVtPosTex );

		// Texture 1: vertex normal
		if( m_pManager->m_bAttTBN )
		{
			// Vertex normal
			m_pEncodeNIFx->SetTexture( "Tex1", m_pd3dVtNormalTex );

			// Texture 3: vertex tangent and handedness
			m_pEncodeNIFx->SetTexture( "Tex3", m_pd3dVtTanHTex );
		}
		else
			if( m_pManager->m_bAttNormal )
				m_pEncodeNIFx->SetTexture( "Tex1", m_pd3dVtNormalTex );

		// Texture 2: vertex texture coordinates
		m_pEncodeNIFx->SetTexture( "Tex2", m_pd3dVtTexCoordTex );

		if( m_pManager->m_bAttCurv )
		{
			// Texture 4: vertex principal curvature and direction #1
			m_pEncodeNIFx->SetTexture( "Tex4", m_pd3dVtP1K1Tex );
			// Texture 5: vertex principal curvature and direction #2
			m_pEncodeNIFx->SetTexture( "Tex5", m_pd3dVtP2K2Tex );
		}

		if( m_pManager->m_bAttTensor2 )
		{
			// Texture 6: 2nd fundamental form
			m_pEncodeNIFx->SetTexture( "Tex6", m_pd3dVtTensor2Tex );
		}

		if( m_pManager->m_bAttTensor3 )
		{
			// Texture 7: tensor of curvature derivative
			m_pEncodeNIFx->SetTexture( "Tex7", m_pd3dVtTensor3Tex );
		}

		// Set effect
		hrResult = m_pEncodeNIFx->SetTechnique( "Encode" );
		if( FAILED( hrResult ) )
			return -1;

		UINT uiPasses;
		hrResult = m_pEncodeNIFx->Begin( &uiPasses, 0 );
		if( FAILED( hrResult ) )
			return -1;

		for( UINT iPass = 0; iPass < uiPasses; iPass++ )
		{
			hrResult = m_pEncodeNIFx->BeginPass( iPass );
			if( FAILED( hrResult ) )
				return -1;

			// Call callback function to render indexed geometry
			(*m_fRenderNonIdx)( this, m_pUserData );

			m_pEncodeNIFx->EndPass( );
		}

		m_pEncodeNIFx->End( );

		m_pd3dDevice->SetRenderTarget( 0, m_pManager->m_pd3dBackBuffer );
	}

	// Restore viewport
	m_pd3dDevice->SetViewport( &d3dPrevVp );
#elif defined(OGL2)
	// Get current viewport settings
	GLint vPrevVP[4];
	glGetIntegerv( GL_VIEWPORT, vPrevVP );

	// Set viewport for attribute rendering
	glViewport( 0, 0, 
				m_pManager->m_iRight  - m_pManager->m_iLeft + 1,
				m_pManager->m_iBottom - m_pManager->m_iTop  + 1 );

	// Save current projection and modelview matrices
	GLfloat vPrevMV[16], vPrevP[16];
	glGetFloatv( GL_MODELVIEW_MATRIX, vPrevMV );
	glGetFloatv( GL_PROJECTION_MATRIX, vPrevP );

	// Set projection matrix
	glMatrixMode( GL_PROJECTION );
	glLoadIdentity( );
	glMultMatrixf( (const GLfloat*) m_pManager->m_mROIProj );
	glMatrixMode( GL_MODELVIEW );

	GLint is;

	// Call callback function to render indexed geometry
	if( m_fRenderIdx )
	{
		// Bind framebuffer object
		glBindFramebufferEXT( GL_FRAMEBUFFER_EXT, m_pManager->m_uiFBO );

		// For each render pass
		int iRT = 0;
		for( int iRP = 0; iRP < m_pManager->m_iNumRPs; iRP++ )
		{
			// Set render targets (render-to-texture)
			int iCt;
			for( iCt = 0; iCt < m_pManager->m_pRenderPass[iRP].m_iRTs; iCt++ )
			{
				// Set up texture for render-to-texture
				glFramebufferTexture2DEXT( GL_FRAMEBUFFER_EXT,
										   GL_COLOR_ATTACHMENT0_EXT + iRT,
										   GL_TEXTURE_2D, 
										   m_pManager->m_uiRTs[iRT], 0 );
				iRT++;
			}

			// Enable both attachments as draw buffers
			glDrawBuffers( m_pManager->m_iNumRTs, m_pManager->m_eRTs );

			// Use shader program for vertex-level deformations
			glUseProgramObjectARB( m_hEncodeIPO[iRP] );

			// Set shader uniform parameters
			glUniform1iARB( m_iEncodeIPOVtMapSize[iRP], m_iVtAttTexSize );

			// Texture 0: vertex position
			glActiveTexture( GL_TEXTURE0 );
			glBindTexture( GL_TEXTURE_2D, m_uiVtPosTexRT );
			is = glGetUniformLocationARB( m_hEncodeIPO[iRP], "s0" );
			glUniform1iARB( is, 0 );

			// Texture 1: vertex normal
			if( m_pManager->m_bAttTBN )
			{
				// Vertex normal
				glActiveTexture( GL_TEXTURE1 );
				glBindTexture( GL_TEXTURE_2D, m_uiVtNormalTexRT );
				is = glGetUniformLocationARB( m_hEncodeIPO[iRP], "s1" );
				glUniform1iARB( is, 1 );

				// Texture 3: vertex tangent and handedness
				glActiveTexture( GL_TEXTURE3 );
				glBindTexture( GL_TEXTURE_2D, m_uiVtTanHTexRT );
				is = glGetUniformLocationARB( m_hEncodeIPO[iRP], "s3" );
				glUniform1iARB( is, 3 );
			}
			else
			if( m_pManager->m_bAttNormal )
			{
				glActiveTexture( GL_TEXTURE1 );
				glBindTexture( GL_TEXTURE_2D, m_uiVtNormalTexRT );
				is = glGetUniformLocationARB( m_hEncodeIPO[iRP], "s1" );
				glUniform1iARB( is, 1 );
			}

			// Texture 2: vertex texture coordinates
			glActiveTexture( GL_TEXTURE2 );
			glBindTexture( GL_TEXTURE_2D, m_uiVtTexCoordTexRT );
			is = glGetUniformLocationARB( m_hEncodeIPO[iRP], "s2" );
			glUniform1iARB( is, 2 );

			if( m_pManager->m_bAttCurv )
			{
				// Texture 4: vertex principal curvature and direction #1
				glActiveTexture( GL_TEXTURE4 );
				glBindTexture( GL_TEXTURE_2D, m_uiVtP1K1TexRT );
				is = glGetUniformLocationARB( m_hEncodeIPO[iRP], "s4" );
				glUniform1iARB( is, 4 );

				// Texture 5: vertex principal curvature and direction #2
				glActiveTexture( GL_TEXTURE5 );
				glBindTexture( GL_TEXTURE_2D, m_uiVtP2K2TexRT );
				is = glGetUniformLocationARB( m_hEncodeIPO[iRP], "s5" );
				glUniform1iARB( is, 5 );
			}

			if( m_pManager->m_bAttTensor2 )
			{
				// Texture 6: 2nd fundamental form
				glActiveTexture( GL_TEXTURE6 );
				glBindTexture( GL_TEXTURE_2D, m_uiVtTensor2TexRT );
				is = glGetUniformLocationARB( m_hEncodeIPO[iRP], "s6" );
				glUniform1iARB( is, 6 );
			}

			if( m_pManager->m_bAttTensor3 )
			{
				// Texture 7: tensor of curvature derivative
				glActiveTexture( GL_TEXTURE7 );
				glBindTexture( GL_TEXTURE_2D, m_uiVtTensor3TexRT );
				is = glGetUniformLocationARB( m_hEncodeIPO[iRP], "s7" );
				glUniform1iARB( is, 7 );
			}

			// Invoke callback
			(*m_fRenderIdx)( this, 0, m_pUserData );
		}
	}

	// Call callback function to render non-indexed geometry
	if( m_fRenderNonIdx && m_pManager->m_bAttUserDefNI )
	{
		// Set render targets (render-to-texture)
		glFramebufferTexture2DEXT( GL_FRAMEBUFFER_EXT,
								   GL_COLOR_ATTACHMENT0_EXT,
								   GL_TEXTURE_2D, 
								   m_pManager->m_uiRTs[m_pManager->m_iNumRTs-1], 
								   0 );

		// Enable both attachments as draw buffers
		glDrawBuffers( 1, m_pManager->m_eRTs );

		// Use shader program for vertex-level deformations
		glUseProgramObjectARB( m_hEncodeNIPO );

		// Set shader uniform parameters
		glUniform1iARB( m_iEncodeNIPOVtMapSize, m_iVtAttTexSize );

		// Texture 0: vertex position
		glActiveTexture( GL_TEXTURE0 );
		glBindTexture( GL_TEXTURE_2D, m_uiVtPosTexRT );
		is = glGetUniformLocationARB( m_hEncodeNIPO, "s0" );
		glUniform1iARB( is, 0 );

		// Texture 1: vertex normal
		if( m_pManager->m_bAttTBN )
		{
			// Vertex normal
			glActiveTexture( GL_TEXTURE1 );
			glBindTexture( GL_TEXTURE_2D, m_uiVtNormalTexRT );
			is = glGetUniformLocationARB( m_hEncodeNIPO, "s1" );
			glUniform1iARB( is, 1 );

			// Texture 3: vertex tangent and handedness
			glActiveTexture( GL_TEXTURE3 );
			glBindTexture( GL_TEXTURE_2D, m_uiVtTanHTexRT );
			is = glGetUniformLocationARB( m_hEncodeNIPO, "s3" );
			glUniform1iARB( is, 3 );
		}
		else
		if( m_pManager->m_bAttNormal )
		{
			glActiveTexture( GL_TEXTURE1 );
			glBindTexture( GL_TEXTURE_2D, m_uiVtNormalTexRT );
			is = glGetUniformLocationARB( m_hEncodeNIPO, "s1" );
			glUniform1iARB( is, 1 );
		}

		// Texture 2: vertex texture coordinates
		glActiveTexture( GL_TEXTURE2 );
		glBindTexture( GL_TEXTURE_2D, m_uiVtTexCoordTexRT );
		is = glGetUniformLocationARB( m_hEncodeNIPO, "s2" );
		glUniform1iARB( is, 2 );

		if( m_pManager->m_bAttCurv )
		{
			// Texture 4: vertex principal curvature and direction #1
			glActiveTexture( GL_TEXTURE4 );
			glBindTexture( GL_TEXTURE_2D, m_uiVtP1K1TexRT );
			is = glGetUniformLocationARB( m_hEncodeNIPO, "s4" );
			glUniform1iARB( is, 4 );

			// Texture 5: vertex principal curvature and direction #2
			glActiveTexture( GL_TEXTURE5 );
			glBindTexture( GL_TEXTURE_2D, m_uiVtP2K2TexRT );
			is = glGetUniformLocationARB( m_hEncodeNIPO, "s5" );
			glUniform1iARB( is, 5 );
		}

		if( m_pManager->m_bAttTensor2 )
		{
			// Texture 6: 2nd fundamental form
			glActiveTexture( GL_TEXTURE6 );
			glBindTexture( GL_TEXTURE_2D, m_uiVtTensor2TexRT );
			is = glGetUniformLocationARB( m_hEncodeNIPO, "s6" );
			glUniform1iARB( is, 6 );
		}

		if( m_pManager->m_bAttTensor3 )
		{
			// Texture 7: tensor of curvature derivative
			glActiveTexture( GL_TEXTURE7 );
			glBindTexture( GL_TEXTURE_2D, m_uiVtTensor3TexRT );
			is = glGetUniformLocationARB( m_hEncodeNIPO, "s7" );
			glUniform1iARB( is, 7 );
		}

		// Invoke callback
		(*m_fRenderNonIdx)( this, m_pUserData );
	}

	// Re-enable rendering to the back buffer
	glBindFramebufferEXT( GL_FRAMEBUFFER_EXT, 0 );

	// Disable shaders
	glUseProgramObjectARB( 0 );

	// Restore viewport
	glViewport( vPrevVP[0], vPrevVP[1], vPrevVP[2], vPrevVP[3] );

	// Restore projection and modelview matrices
	glMatrixMode( GL_PROJECTION );
	glLoadIdentity( );
	glMultMatrixf( (const GLfloat*) vPrevP );
	glMatrixMode( GL_MODELVIEW );
	glLoadIdentity( );
	glMultMatrixf( (const GLfloat*) vPrevMV );
#endif

	return 0;
} // CIntObj::Render


////////////////////////////////////////////////////////////////////////////////
/// \brief Render the required attributes of this model.
///
/// \param bOvlWire Whether to overlay a wireframe rendering over the
///        shaded one.
/// \param fAspect Aspect ratio of the projection.
/// \param fZNear Distance of the near clipping plane.
/// \param fZFar Distance of the far clipping plane.
/// \param pd3dActualTex Diffuse color texture.
/// \param pd3dReliefTex Relief map.
/// \param pd3dBumpTex Bump map.
/// \param bUseReliefMap Whether relief mapping is being used.
/// \param bUseBumpMap Whether bump mapping is being used.
///
/// \return 0 on success.
///         -1 on error.
////////////////////////////////////////////////////////////////////////////////
int CIntObj::RenderDebug( bool bOvlWire, float fAspect, 
						  float fZNear, float fZFar, 
#if defined(D3D9)
						  LPDIRECT3DTEXTURE9 pd3dActualTex,
						  LPDIRECT3DTEXTURE9 pd3dReliefTex,
						  LPDIRECT3DTEXTURE9 pd3dBumpTex,
#elif defined(OGL2)
						  GLuint uiActualTex, 
						  GLuint uiReliefTex, 
						  GLuint uiBumpTex,
#endif
						  bool bUseReliefMap, bool bUseBumpMap,
						  int iShowAtt )
{
	//
	// Render the geometry
	//

#if defined(D3D9)
	if( m_pRenderFx && m_fRenderIdx && m_pManager->m_bRenderIdx )
	{
		HRESULT hrResult = S_OK;

		if( m_pd3dDevice == NULL )
			return -1;

		m_pd3dDevice->SetRenderTarget( 0, m_pManager->m_pd3dBackBuffer );

		// Set uniform parameters
		float pFloat[2];

		// 1D-2D constant of vertex attributes texture
		pFloat[0] = 1.0f / (float) m_iVtAttTexSize;
		pFloat[1] = 1.0f / (float)( m_iVtAttTexSize * m_iVtAttTexSize );
		m_pRenderFx->SetFloatArray( m_hRenderFxIdxC, pFloat, 2 );

		// Get current world, view, proj matrices
		D3DXMATRIX mWorld, mView, mProj;
		m_pd3dDevice->GetTransform( D3DTS_WORLD,      &mWorld );
		m_pd3dDevice->GetTransform( D3DTS_PROJECTION, &mProj  );
		m_pd3dDevice->GetTransform( D3DTS_VIEW,       &mView  );

		// Apply local transformation
		D3DXMatrixMultiply( &mWorld, &m_mLocal, &mWorld );

		D3DXMATRIX mWorldView     = mWorld     * mView;
		D3DXMATRIX mViewProj      = mView      * mProj;
		D3DXMATRIX mWorldViewProj = mWorldView * mProj;

		// Set them to the current shader
		m_pRenderFx->SetMatrix( m_hRenderFxW,   &mWorld         );
		m_pRenderFx->SetMatrix( m_hRenderFxV,   &mView          );
		m_pRenderFx->SetMatrix( m_hRenderFxWVP, &mWorldViewProj );

		D3DXMATRIX mITW;
		float fDet;
		D3DXMatrixInverse( &mITW, &fDet, &mWorld );
		D3DXMatrixTranspose( &mITW, &mITW );
		m_pRenderFx->SetMatrix( m_hRenderFxITW, &mITW );

		// Texture 0: vertex position
		m_pRenderFx->SetTexture( "Tex0", m_pd3dVtPosTex );

		// Texture 1: vertex normal
		m_pRenderFx->SetTexture( "Tex1", m_pd3dVtNormalTex );

		// Texture 2: vertex texture coordinates
		m_pRenderFx->SetTexture( "Tex2", m_pd3dVtTexCoordTex );

		// Texture 3: vertex tangent and handedness
		m_pRenderFx->SetTexture( "Tex3", m_pd3dVtTanHTex );

		// Texture 4: vertex principal curvature and direction #1
		m_pRenderFx->SetTexture( "Tex4", m_pd3dVtP1K1Tex );
		
		// Texture 5: vertex principal curvature and direction #2
		m_pRenderFx->SetTexture( "Tex5", m_pd3dVtP2K2Tex );

		// Texture 6: 2nd fundamental form
		m_pRenderFx->SetTexture( "Tex6", m_pd3dVtTensor2Tex );

		// Texture 7: tensor of curvature derivative
		m_pRenderFx->SetTexture( "Tex7", m_pd3dVtTensor3Tex );

		// Texture 8: actual texture
		m_pRenderFx->SetTexture( "Tex8", pd3dActualTex );

		m_pRenderFx->SetFloat( "g_fShowAtt", (float) iShowAtt );

		// Select effect
		if( bUseReliefMap )
		{
			m_pRenderFx->SetTexture( "Tex9", pd3dReliefTex );
			hrResult = m_pRenderFx->SetTechnique( "Relief" );
			if( FAILED( hrResult ) )
				return -1;
		}
		else if( bUseBumpMap )
		{
			m_pRenderFx->SetTexture( "Tex9", pd3dBumpTex );

			if( iShowAtt < 5 )
				hrResult = m_pRenderFx->SetTechnique( "BlinnBump1" );
			else
				hrResult = m_pRenderFx->SetTechnique( "BlinnBump2" );
			if( FAILED( hrResult ) )
				return -1;
		}
		else
		{
			if( iShowAtt < 5 )
				hrResult = m_pRenderFx->SetTechnique( "Blinn1" );
			else
				hrResult = m_pRenderFx->SetTechnique( "Blinn2" );
			if( FAILED( hrResult ) )
				return -1;
		}

		UINT uiPasses;
		hrResult = m_pRenderFx->Begin( &uiPasses, 0 );
		if( FAILED( hrResult ) )
			return -1;

		for( UINT iPass = 0; iPass < uiPasses; iPass++ )
		{
			hrResult = m_pRenderFx->BeginPass( iPass );
			if( FAILED( hrResult ) )
				return -1;

			// Call callback function to render indexed geometry
			(*m_fRenderIdx)( this, 0, (void*) 1 );

			m_pRenderFx->EndPass( );
		}

		m_pRenderFx->End( );

		if( bOvlWire )
		{
			D3DXMATRIX mWorld;
			m_pd3dDevice->GetTransform( D3DTS_WORLD, &mWorld );

			// Apply local transformation
			D3DXMATRIX mW;
			D3DXMatrixMultiply( &mW, &m_mLocal, &mWorld );
			m_pd3dDevice->SetTransform( D3DTS_WORLD, &mW );

			m_pd3dDevice->SetRenderState( D3DRS_LIGHTING, FALSE );
			m_pd3dDevice->SetRenderState( D3DRS_FILLMODE, D3DFILL_WIREFRAME );

			D3DXMatrixPerspectiveFovLH( &mProj, D3DX_PI / 4, fAspect,
				fZNear + 0.0001f, fZFar + 0.0001f );
			m_pd3dDevice->SetTransform( D3DTS_PROJECTION, &mProj );

			// Call callback function to render indexed geometry
			(*m_fRenderIdx)( this, 0, m_pUserData );

			D3DXMatrixPerspectiveFovLH( &mProj, D3DX_PI / 4, fAspect,
				fZNear - 0.0001f, fZFar - 0.0001f );
			m_pd3dDevice->SetTransform( D3DTS_PROJECTION, &mProj );

			m_pd3dDevice->SetRenderState( D3DRS_FILLMODE, D3DFILL_SOLID );
			m_pd3dDevice->SetRenderState( D3DRS_LIGHTING, TRUE );

			m_pd3dDevice->SetTransform( D3DTS_WORLD, &mWorld );
		}
	}

	// Call callback function to render indexed geometry
/*	if( m_fRenderIdx && m_pManager->m_bRenderIdx )
	{
		HRESULT hrResult = S_OK;

		if( m_pd3dDevice == NULL )
			return -1;

		m_pd3dDevice->SetRenderTarget( 0, m_pManager->m_pd3dBackBuffer );

		// For each render pass
		int iRT = 0;
		for( int iRP = 0; iRP < m_pManager->m_iNumRPs; iRP++ )
		{
			// Set uniform parameters
			float pFloat[2];

			// 1D-2D constant of vertex attributes texture
			pFloat[0] = 1.0f / (float) m_iVtAttTexSize;
			pFloat[1] = 1.0f / (float)( m_iVtAttTexSize * m_iVtAttTexSize );
			m_pEncodeIFx[iRP]->SetFloatArray( m_hEncodeIFxIdxC[iRP], 
				pFloat, 2 );

			D3DXMATRIX mWorld, mView, mProj;

			// Get current world, view, proj matrices
			m_pd3dDevice->GetTransform( D3DTS_WORLD,      &mWorld );
			m_pd3dDevice->GetTransform( D3DTS_PROJECTION, &mProj  );
			m_pd3dDevice->GetTransform( D3DTS_VIEW,       &mView  );

			// Apply local transformation
			D3DXMatrixMultiply( &mWorld, &m_mLocal, &mWorld );

			D3DXMATRIX mWorldView     = mWorld * mView;
			D3DXMATRIX mViewProj      = mView * mProj;
			D3DXMATRIX mWorldViewProj = mWorldView * mProj;

			// Set them to the current shader
			m_pEncodeIFx[iRP]->SetMatrix( m_hEncodeIFxWVP[iRP], 
										  &mWorldViewProj );

			D3DXMATRIX mITW;
			//	D3DXMatrixIdentity( &mITW );
			float fDet;
			D3DXMatrixInverse( &mITW, &fDet, &mWorld );
			D3DXMatrixTranspose( &mITW, &mITW );
			m_pEncodeIFx[iRP]->SetMatrix( m_hEncodeIFxITW[iRP], &mITW );

			// Texture 0: vertex position
			m_pEncodeIFx[iRP]->SetTexture( "Tex0", m_pd3dVtPosTex );

			// Texture 1: vertex normal
			if( m_pManager->m_bAttTBN )
			{
				// Vertex normal
				m_pEncodeIFx[iRP]->SetTexture( "Tex1", m_pd3dVtNormalTex );

				// Texture 3: vertex tangent and handedness
				m_pEncodeIFx[iRP]->SetTexture( "Tex3", m_pd3dVtTanHTex );
			}
			else
				if( m_pManager->m_bAttNormal )
					m_pEncodeIFx[iRP]->SetTexture( "Tex1", m_pd3dVtNormalTex );

			// Texture 2: vertex texture coordinates
			m_pEncodeIFx[iRP]->SetTexture( "Tex2", m_pd3dVtTexCoordTex );

			if( m_pManager->m_bAttCurv )
			{
				// Texture 4: vertex principal curvature and direction #1
				m_pEncodeIFx[iRP]->SetTexture( "Tex4", m_pd3dVtP1K1Tex );
				// Texture 5: vertex principal curvature and direction #2
				m_pEncodeIFx[iRP]->SetTexture( "Tex5", m_pd3dVtP2K2Tex );
			}

			if( m_pManager->m_bAttTensor2 )
			{
				// Texture 6: 2nd fundamental form
				m_pEncodeIFx[iRP]->SetTexture( "Tex6", m_pd3dVtTensor2Tex );
			}

			if( m_pManager->m_bAttTensor3 )
			{
				// Texture 7: tensor of curvature derivative
				m_pEncodeIFx[iRP]->SetTexture( "Tex7", m_pd3dVtTensor3Tex );
			}

			// Set effect
			hrResult = m_pEncodeIFx[iRP]->SetTechnique( "Encode" );
			if( FAILED( hrResult ) )
				return -1;

			UINT uiPasses;
			hrResult = m_pEncodeIFx[iRP]->Begin( &uiPasses, 0 );
			if( FAILED( hrResult ) )
				return -1;

			for( UINT iPass = 0; iPass < uiPasses; iPass++ )
			{
				hrResult = m_pEncodeIFx[iRP]->BeginPass( iPass );
				if( FAILED( hrResult ) )
					return -1;

				// Call callback function to render indexed geometry
				(*m_fRenderIdx)( this, 0, m_pUserData );

				m_pEncodeIFx[iRP]->EndPass( );
			}

			m_pEncodeIFx[iRP]->End( );
		}
	}
*/	
	return 0;

	// Call callback function to render non-indexed geometry
	if( m_fRenderNonIdx && m_pManager->m_bAttUserDefNI )
	{
		HRESULT hrResult = S_OK;

		m_pd3dDevice->SetRenderTarget( 0, m_pManager->m_pd3dBackBuffer );

		// Set uniform parameters
		float pFloat[2];

		// 1D-2D constant of vertex attributes texture
		pFloat[0] = 1.0f / (float) m_iVtAttTexSize;
		pFloat[1] = 1.0f / (float)( m_iVtAttTexSize * m_iVtAttTexSize );
		m_pEncodeNIFx->SetFloatArray( m_hEncodeNIFxIdxC, pFloat, 2 );

		D3DXMATRIX mWorld, mView, mProj;

		// Get current world, view, proj matrices
		m_pd3dDevice->GetTransform( D3DTS_WORLD,      &mWorld );
		m_pd3dDevice->GetTransform( D3DTS_PROJECTION, &mProj  );
		m_pd3dDevice->GetTransform( D3DTS_VIEW,       &mView  );

		// Apply local transformation
		D3DXMatrixMultiply( &mWorld, &m_mLocal, &mWorld );

		D3DXMATRIX mWorldView     = mWorld * mView;
		D3DXMATRIX mViewProj      = mView * mProj;
		D3DXMATRIX mWorldViewProj = mWorldView * mProj;

		// Set them to the current shader
		m_pEncodeNIFx->SetMatrix( m_hEncodeNIFxWVP, &mWorldViewProj );

		D3DXMATRIX mITW;
		D3DXMatrixIdentity( &mITW );
		//	float fDet;
		//	D3DXMatrixInverse( &mITW, &fDet, &mWorld );
		//	D3DXMatrixTranspose( &mITW, &mITW );
		m_pEncodeNIFx->SetMatrix( m_hEncodeNIFxITW, &mITW );

		// Texture 0: vertex position
		m_pEncodeNIFx->SetTexture( "Tex0", m_pd3dVtPosTex );

		// Texture 1: vertex normal
		if( m_pManager->m_bAttTBN )
		{
			// Vertex normal
			m_pEncodeNIFx->SetTexture( "Tex1", m_pd3dVtNormalTex );

			// Texture 3: vertex tangent and handedness
			m_pEncodeNIFx->SetTexture( "Tex3", m_pd3dVtTanHTex );
		}
		else
		if( m_pManager->m_bAttNormal )
			m_pEncodeNIFx->SetTexture( "Tex1", m_pd3dVtNormalTex );

		// Texture 2: vertex texture coordinates
		m_pEncodeNIFx->SetTexture( "Tex2", m_pd3dVtTexCoordTex );

		if( m_pManager->m_bAttCurv )
		{
			// Texture 4: vertex principal curvature and direction #1
			m_pEncodeNIFx->SetTexture( "Tex4", m_pd3dVtP1K1Tex );
			// Texture 5: vertex principal curvature and direction #2
			m_pEncodeNIFx->SetTexture( "Tex5", m_pd3dVtP2K2Tex );
		}

		if( m_pManager->m_bAttTensor2 )
		{
			// Texture 6: 2nd fundamental form
			m_pEncodeNIFx->SetTexture( "Tex6", m_pd3dVtTensor2Tex );
		}

		if( m_pManager->m_bAttTensor3 )
		{
			// Texture 7: tensor of curvature derivative
			m_pEncodeNIFx->SetTexture( "Tex7", m_pd3dVtTensor3Tex );
		}

		// Set effect
		hrResult = m_pEncodeNIFx->SetTechnique( "Encode" );
		if( FAILED( hrResult ) )
			return -1;

		UINT uiPasses;
		hrResult = m_pEncodeNIFx->Begin( &uiPasses, 0 );
		if( FAILED( hrResult ) )
			return -1;

		for( UINT iPass = 0; iPass < uiPasses; iPass++ )
		{
			hrResult = m_pEncodeNIFx->BeginPass( iPass );
			if( FAILED( hrResult ) )
				return -1;

			// Call callback function to render indexed geometry
			(*m_fRenderNonIdx)( this, m_pUserData );

			m_pEncodeNIFx->EndPass( );
		}

		m_pEncodeNIFx->End( );
	}
#elif defined(OGL2)
	glEnable( GL_TEXTURE_2D ); 

	// Re-enable rendering to the back buffer
    glBindFramebufferEXT( GL_FRAMEBUFFER_EXT, 0 );

	// Call callback function to render indexed geometry
	if( m_fRenderIdx )
	{
		// Use shader program for vertex-level deformations
		glUseProgramObjectARB( m_hRenderPO );

		// TODO: Set shader
		(*m_fRenderIdx)( this, 0, m_pUserData );

		// Disable program object
		glUseProgramObjectARB( 0 );
	}

	// Call callback function to render non-indexed geometry
	if( m_fRenderNonIdx && m_pManager->m_bAttUserDefNI )
	{
		// TODO: Set shader
		(*m_fRenderNonIdx)( this, m_pUserData );
	}

	glDisable( GL_TEXTURE_2D ); 
#endif

	return 0;
} // CIntObj::RenderDebug

////////////////////////////////////////////////////////////////////////////////
/// \brief Replace the occurrences of a given string within another string.
///
/// All required memory is allocated automatically.
///
/// \param iNumOcc Number of replacements to perform, starting from
///        the beginning of the string. If this is any negative value, all
///        occurrences will be replaced.
/// \param szString Null-terminated string to search into.
/// \param szLookFor Null-terminated string to search for.
/// \param szReplace Null-terminated string to replace with the instances of
///        szLookFor found in szString.
/// \param szOut Pointer to the resulting replaced text (allocated internally).
///
/// \return 0 on success.
///         -1 on error (not enough memory to create output string).
////////////////////////////////////////////////////////////////////////////////
int CIntObj::ReplaceString( int iNumOcc, char *szString, 
							const char *szLookFor, const char *szReplace, 
							char **szOut )
{
	if( iNumOcc < 0 ) iNumOcc = INT_MAX;

	int iLookForSize = (int) strlen( szLookFor );

	// Compute number of occurrences of the szLookFor string
	int iOccurrences = 0;
	char *pPtr = szString;
	while( pPtr = strstr( pPtr, szLookFor ) )
	{
		if( ++iOccurrences >= iNumOcc ) break;
		pPtr += iLookForSize;
	}

	// Allocate memory for output string
	int iAllocSize = (int) strlen( szString ) - iLookForSize * iOccurrences + 
					 (int) strlen( szReplace ) * iOccurrences + 1;
	char *szOutTmp = new char[ iAllocSize ];
	if( szOutTmp == NULL ) return -1;
	szOutTmp[0] = 0x00;

	// Replace each occurrence
	iOccurrences = 0;
	pPtr = szString;
	char *pPtrOut = NULL;
	while( pPtrOut = strstr( pPtr, szLookFor ) )
	{
		strncat( szOutTmp, pPtr, (int) (pPtrOut-pPtr) );
		strcat ( szOutTmp, szReplace );

		pPtr = pPtrOut + iLookForSize;

		if( ++iOccurrences >= iNumOcc )	break;
	}

	strcat( szOutTmp, pPtr );

	*szOut = szOutTmp;

	return 0;
} // CIntObj::ReplaceString

#if defined(D3D9)
/// Vertex format with transformed coordinates and a set of
/// texture coordinates for quad rendering.
#define FVF_TLCVERTEX ( D3DFVF_XYZRHW | D3DFVF_DIFFUSE | D3DFVF_TEX1 | \
						D3DFVF_TEXCOORDSIZE2(0) )

/// Definition of a transformed vertex for quad rendering.
typedef struct TLCVERTEX
{
	D3DXVECTOR4 p; ///< Transformed position.
    D3DCOLOR    c; ///< RGBA color.
	D3DXVECTOR2 t; ///< Texture coordinates.
} _TLCVERTEX;
#endif

////////////////////////////////////////////////////////////////////////////////
/// \brief Debug function to show the contents of the render targets.
///
/// \return None.
////////////////////////////////////////////////////////////////////////////////
void CIntObj::ShowRT( )
{
#if defined(D3D9)
	HRESULT hrResult = S_OK;

	m_pd3dDevice->SetVertexShader( NULL );
	m_pd3dDevice->SetPixelShader( NULL );
//	m_pd3dDevice->SetTexture( 0, m_pd3dVtPosTex );
	m_pd3dDevice->SetTexture( 0, m_pd3dVtNormalTex ); 
//	m_pd3dDevice->SetTexture( 0, m_pd3dVtTexCoordTex );
//	m_pd3dDevice->SetTexture( 0, m_pd3dVtTanHTex );
//	m_pd3dDevice->SetTexture( 0, m_pd3dVtP1K1Tex );
//	m_pd3dDevice->SetTexture( 0, m_pd3dVtP2K2Tex );

	m_pd3dDevice->SetFVF( FVF_TLVERTEX );
	m_pd3dDevice->SetStreamSource( 0, NULL, 0, sizeof(TLVERTEX) );

	// Render filled quad to hit every pixel/texel
	m_pd3dDevice->DrawPrimitiveUP( D3DPT_TRIANGLESTRIP, 2, 
		m_pScreenQuadVt, sizeof( TLVERTEX ) );

//	m_pd3dDevice->DrawPrimitiveUP( D3DPT_TRIANGLESTRIP, 2, 
//		m_pScreenQuadFace, sizeof( TLVERTEX ) );

	m_pd3dDevice->SetTexture( 0, NULL );

	// Draw ROI

	m_pd3dDevice->SetFVF( FVF_TLCVERTEX );
	m_pd3dDevice->SetStreamSource( 0, NULL, 0, sizeof( _TLCVERTEX ) );

	_TLCVERTEX pROI[5];
	
	// Contents
	// Set up data for quad rendering (vertex maps)
	pROI[0].p = D3DXVECTOR4( m_pManager->m_iLeft   - 0.0f, 
							 m_pManager->m_iTop    - 0.0f, 0.0f, 1.0f );
	pROI[1].p = D3DXVECTOR4( m_pManager->m_iRight  - 0.0f, 
							 m_pManager->m_iTop    - 0.0f, 0.0f, 1.0f );
	pROI[2].p = D3DXVECTOR4( m_pManager->m_iLeft   - 0.0f, 
							 m_pManager->m_iBottom - 0.0f, 0.0f, 1.0f );
	pROI[3].p = D3DXVECTOR4( m_pManager->m_iRight  - 0.0f, 
							 m_pManager->m_iBottom - 0.0f, 0.0f, 1.0f );

	pROI[0].t = D3DXVECTOR2( 0.0f, 0.0f );
	pROI[1].t = D3DXVECTOR2( 1.0f, 0.0f );
	pROI[2].t = D3DXVECTOR2( 0.0f, 1.0f );
	pROI[3].t = D3DXVECTOR2( 1.0f, 1.0f );

	pROI[0].c = D3DCOLOR_ARGB( 255, 255, 255, 255 );
	pROI[1].c = D3DCOLOR_ARGB( 255, 255, 255, 255 );
	pROI[2].c = D3DCOLOR_ARGB( 255, 255, 255, 255 );
	pROI[3].c = D3DCOLOR_ARGB( 255, 255, 255, 255 );
	pROI[4].c = D3DCOLOR_ARGB( 255, 255, 255, 255 );

	m_pd3dDevice->SetTexture( 0, m_pManager->m_pRTT[m_pManager->m_iNumRTs-1] );

	m_pd3dDevice->SetRenderState( D3DRS_ALPHABLENDENABLE, TRUE );
	m_pd3dDevice->SetRenderState( D3DRS_SRCBLEND, D3DBLEND_SRCALPHA );
	m_pd3dDevice->SetRenderState( D3DRS_DESTBLEND, D3DBLEND_INVSRCALPHA );

	m_pd3dDevice->DrawPrimitiveUP( D3DPT_TRIANGLESTRIP, 2, pROI, 
								   sizeof( _TLCVERTEX ) );

	m_pd3dDevice->SetRenderState( D3DRS_ALPHABLENDENABLE, FALSE );
	m_pd3dDevice->SetTexture( 0, NULL );

	// Outline
	pROI[0].p = D3DXVECTOR4( (float) m_pManager->m_iLeft, 
							 (float) m_pManager->m_iTop,    0.0f, 1.0f );
	pROI[1].p = D3DXVECTOR4( (float) m_pManager->m_iRight, 
							 (float) m_pManager->m_iTop,    0.0f, 1.0f );
	pROI[2].p = D3DXVECTOR4( (float) m_pManager->m_iRight, 
							 (float) m_pManager->m_iBottom, 0.0f, 1.0f );
	pROI[3].p = D3DXVECTOR4( (float) m_pManager->m_iLeft, 
							 (float) m_pManager->m_iBottom, 0.0f, 1.0f );
	pROI[4].p = D3DXVECTOR4( (float) m_pManager->m_iLeft, 
							 (float) m_pManager->m_iTop,    0.0f, 1.0f );

	m_pd3dDevice->DrawPrimitiveUP( D3DPT_LINESTRIP, 4, pROI, 
								   sizeof( _TLCVERTEX ) );

#elif defined(OGL2)
	// Save current viewport settings
	GLint vPrevVP[4];
	glGetIntegerv( GL_VIEWPORT, vPrevVP );

	// Save current projection and modelview matrices
	GLfloat vPrevMV[16], vPrevP[16];
	glGetFloatv( GL_MODELVIEW_MATRIX, vPrevMV );
	glGetFloatv( GL_PROJECTION_MATRIX, vPrevP );

	// Disable z-buffering (testing and writing) 
	glDisable( GL_DEPTH_TEST );

	// Disable shaders
	glUseProgramObjectARB( 0 );

	glMatrixMode( GL_PROJECTION );
	glLoadIdentity( );
	gluOrtho2D( 0, vPrevVP[2], 0, vPrevVP[3] );
	glMatrixMode( GL_MODELVIEW );
	glLoadIdentity( );

	glEnable( GL_TEXTURE_2D ); 
	glBindTexture( GL_TEXTURE_2D, m_uiVtPosTexRT      );
//	glBindTexture( GL_TEXTURE_2D, m_uiVtNormalTexRT   );
//	glBindTexture( GL_TEXTURE_2D, m_uiVtTexCoordTexRT );
//	glBindTexture( GL_TEXTURE_2D, m_uiVtTanHTexRT     );
//	glBindTexture( GL_TEXTURE_2D, m_uiVtP1K1TexRT     );
//	glBindTexture( GL_TEXTURE_2D, m_uiVtP2K2TexRT     );

	int iSize = m_iVtAttTexSize + 50;

	// Draw contents of the data textures
	glBegin( GL_QUADS );
	glTexCoord2f( 0.0f, 0.0f );
	glVertex2i( 0,  vPrevVP[3] - 0        );
	glTexCoord2f( 1.0f, 0.0f );
	glVertex2i( iSize, vPrevVP[3] - 0     );
	glTexCoord2f( 1.0f, 1.0f );
	glVertex2i( iSize, vPrevVP[3] - iSize );
	glTexCoord2f( 0.0f, 1.0f );
	glVertex2i( 0,  vPrevVP[3] - iSize    );
	glEnd( );

	// Draw ROI
	glBindTexture( GL_TEXTURE_2D, m_pManager->m_uiRTs[0] );

	glBegin( GL_QUADS );
		glTexCoord2f( 0.0f, 1.0f );
		glVertex2i( m_pManager->m_iLeft,  vPrevVP[3] - m_pManager->m_iTop + 1 );
		glTexCoord2f( 1.0f, 1.0f );
		glVertex2i( m_pManager->m_iRight + 1, vPrevVP[3] - m_pManager->m_iTop + 1 );
		glTexCoord2f( 1.0f, 0.0f );
		glVertex2i( m_pManager->m_iRight + 1, vPrevVP[3] - m_pManager->m_iBottom );
		glTexCoord2f( 0.0f, 0.0f );
		glVertex2i( m_pManager->m_iLeft,  vPrevVP[3] - m_pManager->m_iBottom );
	glEnd( );

	glBindTexture( GL_TEXTURE_2D, 0 );
	glDisable( GL_TEXTURE_2D );

	// Restore projection and modelview matrices
	glMatrixMode( GL_PROJECTION );
	glLoadIdentity( );
	glMultMatrixf( (const GLfloat*) vPrevP );
	glMatrixMode( GL_MODELVIEW );
	glLoadIdentity( );
	glMultMatrixf( (const GLfloat*) vPrevMV );

	// Restore viewport
	glViewport( vPrevVP[0], vPrevVP[1], vPrevVP[2], vPrevVP[3] );

	// Enable z-buffering (testing and writing) 
	glEnable( GL_DEPTH_TEST );
#endif
} // CIntObj::ShowRT

#if defined(D3D9)
////////////////////////////////////////////////////////////////////////////////
/// \brief Set the local transformation matrix used with this object.
///
/// \return None.
////////////////////////////////////////////////////////////////////////////////
void CIntObj::SetLocalTransform( D3DXMATRIX mLocal )
{
	m_mLocal = mLocal;
} // CIntObj::SetLocalTransform
#elif defined(OGL2)
////////////////////////////////////////////////////////////////////////////////
/// \brief Set the local transformation matrix used with this object.
///
/// \return None.
////////////////////////////////////////////////////////////////////////////////
void CIntObj::SetLocalTransform( GLfloat mLocal[16] )
{
	memcpy( m_mLocal, mLocal, sizeof(GLfloat) * 16 );
} // CIntObj::SetLocalTransform
#endif

////////////////////////////////////////////////////////////////////////////////
/// Copyright (C) 2007 Harlen Costa Batagelo. All rights reserved.
///
/// Permission is hereby granted, free of charge, to any person obtaining
/// a copy of this software and associated documentation files (the 
/// "Software"), to deal in the Software without restriction, including
/// without limitation the rights to use, copy, modify, merge, publish,
/// distribute, sublicense, and/or sell copies of the Software, and to
/// permit persons to whom the Software is furnished to do so, subject to
/// the following conditions:
///
/// The above copyright notice and this permission notice shall be
/// included in all copies or substantial portions of the Software.
///
/// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
/// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
/// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
/// IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
/// CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
/// TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
/// SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
////////////////////////////////////////////////////////////////////////////////